;;; -*- Mode:Lisp; Syntax:Common-Lisp; Package:KAPI; Base:10 -*-
;;-----------------------------------------------------------------------------
;;; kapi-process.cl
;;;
;;; KAPI lisp process support. 
;;;----------------------------------------------------------------------------
;;; Copyright (c)  1993, 1994 Lockheed Missiles and Space Company, Inc.
;;; and Enterprise Integration Technologies Corporation
;;;
;;; Permission to use, copy, modify, distribute, and sell this software and 
;;; its documentation for any purpose is hereby granted without fee, provided
;;; that (i) the above copyright notices and this permission notice appear in
;;; all copies of the software and related documentation, and (ii) the name of
;;; Enterprise Integration Technologies Corporation and Lockheed Missiles
;;; and Space Company may not be used in any advertising or publicity
;;; relating to the software without the specific, prior written permission
;;; of Enterprise Integration Technologies Corporation and Lockheed Missiles
;;; and Space Company.
;;; 
;;; THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
;;; EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
;;; WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
;;;
;;; IN NO EVENT SHALL ENTERPRISE INTEGRATION TECHNOLOGIES CORPORATION OR
;;; LOCKHEED MISSILES AND SPACE COMPANY BE LIABLE FOR ANY SPECIAL,
;;; INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY
;;; DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
;;; WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY
;;; THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
;;; PERFORMANCE OF THIS SOFTWARE.
;;;----------------------------------------------------------------------------
(eval-when (:load-toplevel :compile-toplevel :execute)
  (unless (find-package :kapi)
    (defpackage :kapi))
  (in-package :kapi)

  (require :process)              ;; For read-loop process.
  (use-package :multiprocessing))

;;;----------------------------------------------------------------------------
;;; Globals
;;;----------------------------------------------------------------------------
(defvar      *kqml-parse-hook*   nil)
(defconstant *kapi-process-name* "KAPI Process")

;;;----------------------------------------------------------------------------
;;; kapi-init opens a connection and starts a KAPI read loop.
;;; Return t if sucessful, nil otherwise.
;;;----------------------------------------------------------------------------
(defun kapi-init (ansurl name)
  (if (= (lisp-ksetup name ansurl 1) *k-ok*)
      (progn 
	(kapi-read-loop)
	t)
    (progn 
      (format *error-output* 
	      "ERROR: Unable to connect to transport ~a.~%"
	      ansurl)
      nil)))

;;;----------------------------------------------------------------------------
;;; kapi-read-loop starts a Lisp process to handle the read loop.
;;;----------------------------------------------------------------------------
(defun kapi-read-loop (&optional (sleeptime 1))
  (process-run-function
   `(:name ,*kapi-process-name* :priority 0)
 
   #'(lambda ()
       (do () ()
	   (funcall *kqml-parse-hook*)
	   (sleep sleeptime)))))


;;;End of file.
