;;; -*- Mode:Lisp; Syntax:Common-Lisp; Package:KAPI; Base:10 -*-
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                                                                         ;;;
;;; kapi.cl - Allegro Lisp Interface Functions for KAPI                     ;;;
;;;                                                                         ;;;
;;;        ABSTRACT: Lisp Interface Functions for KAPI stuff                ;;;
;;;                  for the functions in kapi.h/c                          ;;;
;;;    The functions implemented are KConnect, KDisconnect, KSendString,    ;;;
;;;  KGetString, KGetString_Or_Wait, KSendList, KGetList, KGetList_Or_Wait  ;;;
;;;                                                                         ;;;
;;;        Created by Siva Panchanatham on Sun 7, March '93  -  14:13:34    ;;;
;;;	   Allegro version by Larry Harada August '93			    ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Copyright (c)  1993, 1994 Lockheed Missiles and Space Company, Inc.
;;; and Enterprise Integration Technologies Corporation
;;;
;;; Permission to use, copy, modify, distribute, and sell this software and 
;;; its documentation for any purpose is hereby granted without fee, provided
;;; that (i) the above copyright notices and this permission notice appear in
;;; all copies of the software and related documentation, and (ii) the name of
;;; Enterprise Integration Technologies Corporation and Lockheed Missiles
;;; and Space Company may not be used in any advertising or publicity
;;; relating to the software without the specific, prior written permission
;;; of Enterprise Integration Technologies Corporation and Lockheed Missiles
;;; and Space Company.
;;; 
;;; THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
;;; EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
;;; WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
;;;
;;; IN NO EVENT SHALL ENTERPRISE INTEGRATION TECHNOLOGIES CORPORATION OR
;;; LOCKHEED MISSILES AND SPACE COMPANY BE LIABLE FOR ANY SPECIAL,
;;; INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY
;;; DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
;;; WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY
;;; THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
;;; PERFORMANCE OF THIS SOFTWARE.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(eval-when (:load-toplevel :compile-toplevel :execute)
  (unless (find-package :kapi)
    (defpackage :kapi))
  (in-package :kapi)

  (import 'user::string-char)
  (import 'user::*kapi-foreign-library-directory*)

  (require :foreign)
  (use-package :foreign-functions))

;;;Load the object files from the appropriate paths:
(eval-when (:load-toplevel :execute)
  (load "" 
	:foreign-files `(,(make-pathname 
			   :directory *kapi-foreign-library-directory*
			   :name "libkapi.a"))

	:unreferenced-lib-names '("_KLookup"
				  "_KSetDefaultANS"
				  "_KInit"
				  "_KSetUp"
				  "_KDisconnect"
				  "_KListen"
				  "_KCacheURL"
				  "_KUnCacheURL"
				  "_KGetOpenFd"
				  "_KRegister"
				  "_KUnRegister"
				  "_KSendString"
				  "_KGetString"
				  "_KGetString_or_Wait"
				  "_KPeek")))

				  
;;;----------------------------------------------------------------------
;;; Constants (see kapi.h)
;;;----------------------------------------------------------------------
(defconstant *k-max-message-size* 16384)  ;; See kapi.h

(defconstant *k-ok*              0)       ;; Return/error codes.

(defconstant *k-bad-kqml*        1)
(defconstant *k-no-ans-response* 2)
(defconstant *k-bad-url*         3)
(defconstant *k-no-sender*       4)
(defconstant *k-no-receiver*     5)
(defconstant *k-no-transport*    6)
(defconstant *k-no-msg*          7)
(defconstant *k-no-field*        8)
(defconstant *k-no-active-url*   9)

(defconstant *k-error*          -1)

;;;----------------------------------------------------------------------
;;; Foreign function definitions
;;;----------------------------------------------------------------------

;;;; Lisp KLOOKUP ;;;;

(defforeign 'LFI-KLookup
    :entry-point (convert-to-lang "KLookup" :language :c)  
    :arguments '(integer integer integer)
    :return-type :fixnum)

(let ((buf (ff:string-to-char* (excl:make-static-array
				*k-max-message-size*
				:element-type 'string-char))))

  (defun Lisp-KLookup (ansurl agentname)
    (values (LFI-KLookup (ff:string-to-char* ansurl)
			 (ff:string-to-char* agentname)
			 buf)
	    (ff:char*-to-string buf))))


;;;; Lisp KSETDEFAULTANS ;;;;

(defforeign 'LFI-KSetDefaultANS
    :entry-point (convert-to-lang "KSetDefaultANS" :language :c)  
    :arguments '(integer)
    :return-type :fixnum)

(defun Lisp-KSetDefaultANS (ansurl)
  (LFI-KSetDefaultANS (if (stringp ansurl) 
			  (ff:string-to-char* ansurl)
			ansurl)))



;;;; Lisp KINIT ;;;;

(defforeign 'LFI-KInit
    :entry-point (convert-to-lang "KInit" :language :c)  
    :return-type :fixnum)

(defun Lisp-KInit ()
  (LFI-KInit))


;;;; Lisp KSETUP ;;;;

(defforeign 'LFI-KSetUp
    :entry-point (convert-to-lang "KSetUp" :language :c)
    :arguments '(integer integer integer)
    :return-type :fixnum)

(defun Lisp-KSetUp (name ansurl lookup-and-listen)
  (LFI-KSetUp (ff:string-to-char* name)
	      (if (stringp ansurl) 
		  (ff:string-to-char* ansurl)
		ansurl)
	      lookup-and-listen))


;;;; LISP KDISCONNECT ;;;;

(defforeign 'LFI-KDisconnect
    :entry-point (convert-to-lang "KDisconnect" :language :c)
    :return-type :fixnum)

(defun Lisp-KDisconnect ()
  (LFI-KDisconnect))



;;;; LISP KLISTEN ;;;;

(defforeign 'LFI-KListen
    :entry-point (convert-to-lang "KListen" :language :c)
    :arguments '(integer)
    :return-type :fixnum)

(defun Lisp-KListen (url)
  (LFI-KListen (ff:string-to-char* url)))



;;;; LISP KCACHEURL ;;;;

(defforeign 'LFI-KCacheURL
    :entry-point (convert-to-lang "KCacheURL" :language :c)
    :arguments '(integer integer)
    :return-type :fixnum)

(defun Lisp-KCacheURL (agentname url)
  (LFI-KCacheURL (ff:string-to-char* agentname)
		 (ff:string-to-char* url)))



;;;; LISP KUNCACHEURL ;;;;

(defforeign 'LFI-KUnCacheURL
    :entry-point (convert-to-lang "KUnCacheURL" :language :c)
    :arguments '(integer integer)
    :return-type :fixnum)

(defun Lisp-KUnCacheURL (agentname url)
  (LFI-KUnCacheURL (if (stringp agentname) 
		       (ff:string-to-char* agentname)
		     agentname)
		   (if (stringp url)
		       (ff:string-to-char* url)
		     url)))


;;;; LISP KGETOPENFD ;;;;

(defforeign 'LFI-KGetOpenFd
    :entry-point (convert-to-lang "KGetOpenFd" :language :c)
    :arguments '(integer)
    :return-type :fixnum)

(defun Lisp-KGetOpenFd (first)
  (LFI-KGetOpenFd first))


;;;; LISP KREGISTER ;;;;

(defforeign 'LFI-KRegister
    :entry-point (convert-to-lang "KRegister" :language :c)
    :arguments '(integer integer integer)
    :return-type :fixnum)

(defun Lisp-KRegister (ansurl agentname agenturl)
  (LFI-KRegister (ff:string-to-char* ansurl)
		 (ff:string-to-char* agentname)
		 (ff:string-to-char* agenturl)))



;;;; LISP KUNREGISTER ;;;;

(defforeign 'LFI-KUnRegister
    :entry-point (convert-to-lang "KUnRegister" :language :c)
    :arguments '(integer integer integer)
    :return-type :fixnum)

(defun Lisp-KUnRegister (ansurl agentname agenturl)
  (LFI-KUnRegister (ff:string-to-char* ansurl)
		   (ff:string-to-char* agentname)
		   (ff:string-to-char* agenturl)))



;;;; LISP KSENDSTRING ;;;;

(defforeign 'LFI-KSendString
    :entry-point (convert-to-lang "KSendString" :language :c)
    :arguments '(integer)
    :return-type :fixnum)

(defun Lisp-KSendString (msg)
  (LFI-KSendString (ff:string-to-char* msg)))



;;;; LISP KGETSTRING ;;;;

(defforeign 'LFI-KGetString 
    :entry-point (convert-to-lang "KGetString" :language :c)
    :arguments '(integer)
    :return-type :fixnum)

(let ((buf (ff:string-to-char* (excl:make-static-array
				*k-max-message-size*
				:element-type 'string-char))))
  (defun Lisp-KGetString ()
    (values (LFI-KGetString buf)
	    (ff:char*-to-string buf))))
    

;;;; LISP KGETSTRING-OR-WAIT ;;;;

;;;; ***** Warning: This function does not block!! *******
(defforeign 'LFI-KGetString-Or-Wait
    :entry-point (convert-to-lang "KGetString_or_Wait" :language :c)
    :arguments '(integer)
    :return-type :fixnum)

(let ((buf (ff:string-to-char* (excl:make-static-array
				*k-max-message-size*
				:element-type 'string-char))))
  (defun Lisp-KGetString-Or-Wait ()
    (values (LFI-KGetString-Or-Wait buf)
	    (ff:char*-to-string buf))))


;;;; LISP KPEEK ;;;;

(defforeign 'LFI-KPeek
    :entry-point (convert-to-lang "KPeek" :language :c)
    :return-type :fixnum)

(defun Lisp-KPeek ()
  (LFI-KPeek))

;;; End of file.


