/* 
 * Copyright (c)  1993, 1994 Enterprise Integration Technologies Corporation
 * and Lockheed Missiles and Space Company, Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and 
 * its documentation for any purpose is hereby granted without fee, provided
 * that (i) the above copyright notices and this permission notice appear in
 * all copies of the software and related documentation, and (ii) the name of
 * Enterprise Integration Technologies Corporation and Lockheed Missiles
 * and Space Company may not be used in any advertising or publicity
 * relating to the software without the specific, prior written permission
 * of Enterprise Integration Technologies Corporation and Lockheed Missiles
 * and Space Company.
 * 
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
 *
 * IN NO EVENT SHALL ENTERPRISE INTEGRATION TECHNOLOGIES CORPORATION OR
 * LOCKHEED MISSILES AND SPACE COMPANY BE LIABLE FOR ANY SPECIAL,
 * INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY
 * DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY
 * THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 */

#include "kapi.h"
#include "tclKAPI.h"

/*
 * Forward declarations
 */
static int   KTCL_ksetDefaultANS( ClientData, Tcl_Interp*, int, char** );
static int   KTCL_kdisconnect( ClientData, Tcl_Interp*, int, char** );
static int   KTCL_ksend( ClientData, Tcl_Interp*, int, char** );
static int   KTCL_kget( ClientData, Tcl_Interp*, int, char** );
static int   KTCL_klisten( ClientData, Tcl_Interp*, int, char** );
static int   KTCL_kregister( ClientData, Tcl_Interp*, int, char** );
static int   KTCL_kunregister( ClientData, Tcl_Interp*, int, char** );
static int   KTCL_klookup( ClientData, Tcl_Interp*, int, char** );
static int   KTCL_kcache( ClientData, Tcl_Interp*, int, char** );
static int   KTCL_kuncache( ClientData, Tcl_Interp*, int, char** );

static int   KTCL_kqml_getValue( ClientData, Tcl_Interp*, int, char** );
static int   KTCL_kqml_setValue( ClientData, Tcl_Interp*, int, char** );

static void  KTCL_MsgPoll( ClientData );

/*
 * Local data
 */
static char MsgBuf[ 4096 ];

/*
 * KAPI_Init() registers the application specific commands with the 
 * TCL interpreter.
 */

int KAPI_Init( Tcl_Interp *interp )
{
    Tcl_CreateCommand(interp, "kqml_set_ans", KTCL_ksetDefaultANS, 
		      (ClientData) 0, (Tcl_CmdDeleteProc *) NULL );
    Tcl_CreateCommand(interp, "kqml_disconnect", KTCL_kdisconnect, 
		      (ClientData) 0, (Tcl_CmdDeleteProc *) NULL );
    Tcl_CreateCommand(interp, "kqml_send", KTCL_ksend, 
		      (ClientData) 0, (Tcl_CmdDeleteProc *) NULL );
    Tcl_CreateCommand(interp, "kqml_get", KTCL_kget, 
		      (ClientData) 0, (Tcl_CmdDeleteProc *) NULL );
    Tcl_CreateCommand(interp, "kqml_listen", KTCL_klisten, 
		      (ClientData) 0, (Tcl_CmdDeleteProc *) NULL );
    Tcl_CreateCommand(interp, "kqml_register", KTCL_kregister, 
		      (ClientData) 0, (Tcl_CmdDeleteProc *) NULL );
    Tcl_CreateCommand(interp, "kqml_unregister", KTCL_kunregister, 
		      (ClientData) 0, (Tcl_CmdDeleteProc *) NULL );
    Tcl_CreateCommand(interp, "kqml_lookup", KTCL_klookup, 
		      (ClientData) 0, (Tcl_CmdDeleteProc *) NULL );
    Tcl_CreateCommand(interp, "kqml_cache", KTCL_kcache, 
		      (ClientData) 0, (Tcl_CmdDeleteProc *) NULL );
    Tcl_CreateCommand(interp, "kqml_uncache", KTCL_kuncache, 
		      (ClientData) 0, (Tcl_CmdDeleteProc *) NULL );

    Tcl_CreateCommand(interp, "kqml_getValue", KTCL_kqml_getValue, 
		      (ClientData) 0, (Tcl_CmdDeleteProc *) NULL );
    Tcl_CreateCommand(interp, "kqml_setValue", KTCL_kqml_setValue, 
		      (ClientData) 0, (Tcl_CmdDeleteProc *) NULL );

    /* Init KAPI data structures. */
    KInit();

/*  There has got to be a better way than polling...  */
/*    Tk_CreateTimerHandler(1000, KTCL_MsgPoll, NULL); */

    return TCL_OK;
} /* KAPI_Init */

/*---------------------------------------------------------------------------
 * KTCL_MsgPoll()
 *---------------------------------------------------------------------------
 */
static void KTCL_MsgPoll(ClientData data)
{
    /* Do stuff */
} /* KTCL_MsgPoll */

/*---------------------------------------------------------------------------
 * KTCL_kdisconnect()
 *---------------------------------------------------------------------------
 */
static int KTCL_kdisconnect( client_data, interp, argc, argv )
ClientData client_data;
Tcl_Interp *interp;
int argc;
char **argv;
{
    int result, booleanValue;
    /* kqml_disconnect */

    if ( argc != 1 ) {
	Tcl_AppendResult(interp, "Wrong # args: should be \"", argv[0],
			 "\"", NULL);
	return TCL_ERROR;
    }

    KDisconnect();

    return TCL_OK;
} /* KTCL_kdisconnect */

/*---------------------------------------------------------------------------
 * KTCL_ksetDefaultANS()
 *---------------------------------------------------------------------------
 */
static int KTCL_ksetDefaultANS( client_data, interp, argc, argv )
ClientData client_data;
Tcl_Interp *interp;
int argc;
char **argv;
{
    int result;
    /* kqml_set_ans url */

    if ( argc != 2 ) {
	Tcl_AppendResult(interp, "Wrong # args: should be \"", argv[0],
			 " url\"", NULL);
	return TCL_ERROR;
    }

    result = KSetDefaultANS( argv[1] );
    switch (result) {
    case K_OK:
	return TCL_OK;
    case K_BAD_URL:
	Tcl_SetResult(interp, "K_BAD_URL: bad url for name server", TCL_STATIC);
	return TCL_ERROR;
    case K_ERROR:
    default:
	Tcl_SetResult(interp, "K_ERROR: in kqml_set_ans", TCL_STATIC);
	return TCL_ERROR;
    }
} /* KTCL_ksetDefaultANS */

/*---------------------------------------------------------------------------
 * KTCL_ksend()
 *---------------------------------------------------------------------------
 */
static int KTCL_ksend( client_data, interp, argc, argv )
ClientData client_data;
Tcl_Interp *interp;
int argc;
char **argv;
{
    int result;
    /* kqml_send kqml_string */

    if ( argc != 2 ) {
	Tcl_AppendResult(interp, "Wrong # args: should be \"", argv[0],
			 " kqmlMsg\"", NULL);
	return TCL_ERROR;
    }

    result = KSendString( argv[1] );
    switch (result) {
    case K_OK:
	return TCL_OK;
    case K_BAD_KQML:
	Tcl_SetResult(interp, "K_BAD_KQML: bad outgoing kqml", TCL_STATIC);
	return TCL_ERROR;
    case K_NO_RECEIVER:
	Tcl_SetResult(interp, "K_NO_RECEIVER: no receiver specified", TCL_STATIC);
	return TCL_ERROR;
    case K_NO_TRANSPORT:
	Tcl_SetResult(interp, "K_NO_TRANSPORT: no transport found", TCL_STATIC);
	return TCL_ERROR;
    case K_ERROR:
    default:
	Tcl_SetResult(interp, "K_ERROR: in kqml_send", TCL_STATIC);
	return TCL_ERROR;
    }
} /* KTCL_ksend */

/*---------------------------------------------------------------------------
 * KTCL_kget()
 *---------------------------------------------------------------------------
 */
static int KTCL_kget( client_data, interp, argc, argv )
ClientData client_data;
Tcl_Interp *interp;
int argc;
char **argv;
{
    int result, booleanValue;
    /* kqml_get ?wait_boolean? */
    /* Default is no wait */

    if ( argc != 1 && argc != 2) {
	Tcl_AppendResult(interp, "Wrong # args: should be \"", argv[0],
			 " ?wait?\"", NULL);
	return TCL_ERROR;
    }

    /* Yes we are using a global buffer to put the message in. */
    MsgBuf[0] = '\0';

    if (argc == 1) {
	booleanValue = 0;
    } else if ( Tcl_GetBoolean(interp, argv[1], &booleanValue) != TCL_OK ) {
	return TCL_ERROR;
    }

    if ( booleanValue ) {
	result = KGetString_or_Wait( MsgBuf );
    } else {
	result = KGetString( MsgBuf );
    }
    
    switch (result) {
    case K_OK:
	Tcl_AppendResult( interp, MsgBuf, (char *)NULL );
	return TCL_OK;
    case K_BAD_KQML:
	Tcl_SetResult(interp, "K_BAD_KQML: bad incoming kqml", TCL_STATIC);
	return TCL_ERROR;
    case K_NO_ACTIVE_URL:
	if ( booleanValue ) {
	    Tcl_SetResult(interp, "K_NO_ACTIVE_URL: no transport to get from", 
			  TCL_STATIC);
	    return TCL_ERROR;
	} else {
	    Tcl_ResetResult(interp);
	    return TCL_OK;
	}
    case K_ERROR:
    default:
	Tcl_SetResult(interp, "K_ERROR: in kqml_get", TCL_STATIC);
	return( TCL_ERROR );
    }
} /* KTCL_kget */

/*---------------------------------------------------------------------------
 * KTCL_klisten()
 *---------------------------------------------------------------------------
 */
static int KTCL_klisten( client_data, interp, argc, argv )
ClientData client_data;
Tcl_Interp *interp;
int argc;
char **argv;
{
    /* kqml_listen url [agentName callBackProc] */
    int result;
    
    if ( !(argc == 2 || argc == 4) ) {
	Tcl_AppendResult(interp, "Wrong # args: should be \"", argv[0],
			 " url [agentName callBackProc]\"", NULL);
	return TCL_ERROR;
    }

    if (argc == 4) {
	/* Set call back for incoming messages to agentName */
    }

    result = KListen( argv[1] );
    switch (result) {
    case K_OK:
	return TCL_OK;
    case K_BAD_URL:
	Tcl_SetResult(interp, "K_BAD_URL: bad url", TCL_STATIC);
	return TCL_ERROR;
    case K_ERROR:
    default:
	Tcl_SetResult(interp, "K_ERROR: in kqml_listen", TCL_STATIC);
	return TCL_ERROR;
    }
} /* KTCL_klisten */

/*---------------------------------------------------------------------------
 * KTCL_kregister()
 *---------------------------------------------------------------------------
 */
static int KTCL_kregister( client_data, interp, argc, argv )
ClientData client_data;
Tcl_Interp *interp;
int argc;
char **argv;
{
    /* kqml_register ans_url agentName url */
    int result;
    
    if (argc != 4) {
	Tcl_AppendResult(interp, "Wrong # args: should be \"", argv[0],
			 " ans_url agentName url\"", NULL);
	return TCL_ERROR;
    }

    result = KRegister( argv[1], argv[2], argv[3] );
    switch (result) {
    case K_OK:
	return TCL_OK;
    case K_BAD_URL:
	Tcl_SetResult(interp, "K_BAD_URL: bad url", TCL_STATIC);
	return TCL_ERROR;
    case K_NO_ANS_RESPONSE:
	Tcl_SetResult(interp, "K_NO_ANS_RESPONSE: ANS not responding", TCL_STATIC);
	return TCL_ERROR;
    case K_ERROR:
    default:
	Tcl_SetResult(interp, "K_ERROR: in kqml_register", TCL_STATIC);
	return TCL_ERROR;
    }
} /* KTCL_kregister */

/*---------------------------------------------------------------------------
 * KTCL_kunregister()
 *---------------------------------------------------------------------------
 */
static int KTCL_kunregister( client_data, interp, argc, argv )
ClientData client_data;
Tcl_Interp *interp;
int argc;
char **argv;
{
    /* kqml_unregister ans_url agentName url */
    int result;
    
    if (argc != 4) {
	Tcl_AppendResult(interp, "Wrong # args: should be \"", argv[0],
			 " ans_url agentName url\"", NULL);
	return TCL_ERROR;
    }

    result = KUnRegister( argv[1], argv[2], argv[3] );
    switch (result) {
    case K_OK:
	return TCL_OK;
    case K_BAD_URL:
	Tcl_SetResult(interp, "K_BAD_URL: bad url", TCL_STATIC);
	return TCL_ERROR;
    case K_NO_ANS_RESPONSE:
	Tcl_SetResult(interp, "K_NO_ANS_RESPONSE: no response from ANS", TCL_STATIC);
	return TCL_ERROR;
    case K_ERROR:
    default:
	Tcl_SetResult(interp, "K_ERROR: in kqml_unregister", TCL_STATIC);
	return TCL_ERROR;
    }
} /* KTCL_kunregister */

/*---------------------------------------------------------------------------
 * KTCL_klookup()
 *---------------------------------------------------------------------------
 */
static int KTCL_klookup( client_data, interp, argc, argv )
ClientData client_data;
Tcl_Interp *interp;
int argc;
char **argv;
{
    /* kqml_lookup ans_url agentName */
    int result;
    char buffer[1024];
    
    if (argc != 3) {
	Tcl_AppendResult(interp, "Wrong # args: should be \"", argv[0],
			 " ans_url agentName\"", NULL);
	return TCL_ERROR;
    }

    result = KLookup( argv[1], argv[2], buffer );
    switch (result) {
    case K_OK:
	/* TODO: make into a better Tcl list */
	Tcl_SetResult(interp, buffer, TCL_STATIC);
	return TCL_OK;
    case K_BAD_URL:
	Tcl_SetResult(interp, "K_BAD_URL: bad url", TCL_STATIC);
	return TCL_ERROR;
    case K_NO_ANS_RESPONSE:
	Tcl_SetResult(interp, "K_NO_ANS_RESPONSE: no response from ANS", TCL_STATIC);
	return TCL_ERROR;
    case K_ERROR:
    default:
	Tcl_SetResult(interp, "K_ERROR: in kqml_lookup", TCL_STATIC);
	return TCL_ERROR;
    }

} /* KTCL_klookup */

/*---------------------------------------------------------------------------
 * KTCL_kcache()
 *---------------------------------------------------------------------------
 */
static int KTCL_kcache( client_data, interp, argc, argv )
ClientData client_data;
Tcl_Interp *interp;
int argc;
char **argv;
{
    /* kqml_cache agentName url */
    int result;
    char buffer[1024];
    
    if (argc != 3) {
	Tcl_AppendResult(interp, "Wrong # args: should be \"", argv[0],
			 " agentName url\"", NULL);
	return TCL_ERROR;
    }

    result = KCacheURL(argv[1], argv[2]);
    switch (result) {
    case K_OK:
	return TCL_OK;
    case K_BAD_URL:
	Tcl_SetResult(interp, "K_BAD_URL: bad url", TCL_STATIC);
	return TCL_ERROR;
    case K_ERROR:
    default:
	Tcl_SetResult(interp, "K_ERROR: in kqml_cache", TCL_STATIC);
	return TCL_ERROR;
    }
} /* KTCL_kcache */

/*---------------------------------------------------------------------------
 * KTCL_kuncache()
 *---------------------------------------------------------------------------
 */
static int KTCL_kuncache( client_data, interp, argc, argv )
ClientData client_data;
Tcl_Interp *interp;
int argc;
char **argv;
{
    /* kqml_uncache agentName */
    int result;
    char buffer[1024];
    
    if (argc != 2) {
	Tcl_AppendResult(interp, "Wrong # args: should be \"", argv[0],
			 " agentName\"", NULL);
	return TCL_ERROR;
    }

    result = KUnCacheURL(argv[1], argv[2]);
    switch (result) {
    case K_OK:
	return TCL_OK;
    case K_ERROR:
    default:
	Tcl_SetResult(interp, "K_ERROR: in kqml_uncache", TCL_STATIC);
	return TCL_ERROR;
    }
} /* KTCL_kuncache */


/* Commands to build up KQML messages */

/*---------------------------------------------------------------------------
 * KTCL_kqml_setValue()
 *---------------------------------------------------------------------------
 */
static int KTCL_kqml_setValue( client_data, interp, argc, argv )
ClientData client_data;
Tcl_Interp *interp;
int argc;
char **argv;
{
    int result;
    char *curStr, *newStr, *key = NULL, *value, *resultStr;

    /* Command: kqml_setValue [kqmlStr|"new"] [key] value */
    /* Example: kqml_setValue new foo
       kqml_setValue new :bar do
       kqml_setValue "(foo :bar hi :bax bye)" do
       kqml_setValue "(foo :bar hi :bax bye)" :bar hello
       kqml_setValue "(foo :bar hi :bax bye)" :boo casper
    */

    if (argc != 3 && argc != 4) {
	return TCL_ERROR;
    }

    if ( !strcmp(argv[1], "new") ) {
	curStr = NULL;
    } else {
	curStr = argv[1];
    }

    if (argc == 4) {
	key = argv[2];
	value = argv[3];
    } else {
	key = NULL;
	value = argv[2];
    }

    resultStr = interp->result;
    result = KSetField(curStr, key, value, &resultStr, TCL_RESULT_SIZE);
    switch (result) {
    case K_OK:
	if (resultStr != interp->result) {
	    Tcl_SetResult(interp, resultStr, TCL_DYNAMIC);
	}
	return TCL_OK; 
    case K_BAD_KQML:
	Tcl_SetResult(interp, "K_BAD_KQML: bad kqml", TCL_STATIC);
	return TCL_ERROR;
    case K_ERROR:
    default:
	Tcl_SetResult(interp, "K_ERROR: in kqml_setValue", TCL_STATIC);
    }
} /* KTCL_kqml_setValue */

/*---------------------------------------------------------------------------
 * KTCL_kqml_getValue()
 *---------------------------------------------------------------------------
 */
static int KTCL_kqml_getValue( client_data, interp, argc, argv )
ClientData client_data;
Tcl_Interp *interp;
int argc;
char **argv;
{
    int result;
    char *varValue, *resultStr, *key = NULL;

    /* Command: kqml_getValue kqmlStr [key] */

    if (argc != 2 && argc != 3) {
	return TCL_ERROR;
    }

    varValue = argv[1];

    if (argc = 3) {
	key = argv[2];
    }

    resultStr = interp->result;
    result = KGetField(varValue, key, &resultStr, TCL_RESULT_SIZE);
    switch (result) {
    case K_OK:
	if (resultStr != interp->result) {
	    Tcl_SetResult(interp, resultStr, TCL_DYNAMIC);
	}
	return TCL_OK; 
    case K_NO_FIELD:
	/* Just return the empty string - not really an error. */
	Tcl_ResetResult(interp);
	return TCL_OK;
    case K_BAD_KQML:
	Tcl_SetResult(interp, "K_BAD_KQML: bad kqml", TCL_STATIC);
	return TCL_ERROR;
    case K_ERROR:
    default:
	Tcl_SetResult(interp, "K_ERROR: in kqml_getValue", TCL_STATIC);
    }
} /* KTCL_kqml_getValue */

