/*
 * messaging database
 *
 * Inheritance overview:
 *
 * data
 *  |
 * node
 *  | 
 * legacy
 *  |
 * resource 
 *  |
 * area 
 *  |
 * engine   <- Current Code
 *
 * begin : 20- 6-1999 BastardAI
 *         13- 1-2001 Jarl van Katwijk, started redesign to 0.5 OOP structure.
 *
 */
#ifndef ENGINE
#define ENGINE


/*
 * INCLUDES
 */
#include <glib.h>
#include <gtk/gtk.h>
#include <pthread.h>
#include <DFPADM.hh>
#include "messaging_area.h"


/*
 * DEFINES...
 */
/* the 2 main Node's */
#define PIPELINE_MO_NAME "pipeLine_NODE"
#define PIPELINE_MO_DESCRIPTION "pipeLine_NODE"
#define VISUAL_MO_NAME "visual_NODE"
#define VISUAL_MO_DESCRIPTION "visual_NODE_description"
#define NO_DFPADM


/*
 * FUNCTIONS
 */
class engine: public area {
    public:
        inline engine() {}
        inline engine(glong DLid,glong MDLid,gchar* Password) {
            /* init values of variables to default */
            running = -1;
            bound2DL = FALSE;            /* TODO : install signal handlers */
            shuttingdown = FALSE;

            /* Security */
            ENGINE_DL_ID = DLid;
            ENGINE_MDL_ID = MDLid;
            ENGINE_DL_PASSWORD = g_string_new("engine");
            
            /* For now just one master dl (mdl) per BL */
            if (MDLid != 1) {
               g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_ERROR,"Invalid Master DL id (%li)",MDLid);
            }

            /* Create area 1 */
            if (DLid == MDLid) {
                secure_register_dl(ENGINE_DL_ID,ENGINE_DL_PASSWORD->str,ENGINE_DL_ID,"");
            }

            /* Allocate 2 basic members */
            pipeLine = secure_node_new((gchar*)PIPELINE_MO_NAME,(gchar*)PIPELINE_MO_DESCRIPTION,"",PipeLineNode,NULL,NULL,NULL,NULL,ENGINE_DL_ID,ENGINE_DL_PASSWORD->str);
            secure_register_node(pipeLine,ENGINE_DL_ID,ENGINE_DL_PASSWORD->str);
            visual_central = secure_node_new((gchar*)VISUAL_MO_NAME,(gchar*)VISUAL_MO_DESCRIPTION,"",VisualCentralNode,NULL,NULL,NULL,NULL,ENGINE_DL_ID,ENGINE_DL_PASSWORD->str);
            secure_register_node(visual_central,ENGINE_DL_ID,ENGINE_DL_PASSWORD->str);
            secure_node_set_allowedfilter(visual_central,"{Accept nothing}",ENGINE_DL_ID,ENGINE_DL_PASSWORD->str);

            /* load and setup plugins */
            secure_setup_nodes(ENGINE_DL_ID,ENGINE_DL_PASSWORD->str,(gpointer)this);

            /* setup binder-2-adm */
            bound2DL = FALSE;

            /* Apply hardcoded PIPER logic */
            /* TODO: move to a config file */
            if (rules(1,(glong)ENGINE_DL_ID,(gchar*)ENGINE_DL_PASSWORD->str)==FALSE) {
                g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_CRITICAL,"Applying the hardcoded logic for Piper mode failed");
            }

            /* start main loop */
            gtk_main();
        }

        virtual ~engine() {
            g_string_free(ENGINE_DL_PASSWORD,TRUE);
            g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_MESSAGE,"Shutting down BL...");

            /* reset signal handlers */
            signal(SIGTERM,SIG_DFL);
            signal(SIGSEGV,SIG_DFL);
            signal(SIGINT,SIG_DFL);
            /* shutdown db */
            shuttingdown = TRUE;

            g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_MESSAGE,"Shutdown complete, terminating BL...");
            return;
        };


        /*
         * Variables
         */
        gboolean shuttingdown;
        gboolean deadendstream_active;
        pthread_t gtktree;
        gint gtkthread;
        gint running;
        gboolean bound2DL;
        /* Special node's and data's */
        glong pipeLine;
        glong visual_central;
        glong deadEnd;
        /* corba stuff */
        CORBA::ORB_var DFPADM_corba_orb;
        CORBA::Object_var DFPADM_corba_object;
        DFPADM_var dfpadmref;
        CORBA::Object ObtainObj;
        CORBA::Object RepresentationObj;
        glong ENGINE_DL_ID;
        glong ENGINE_MDL_ID;
        GString *ENGINE_DL_PASSWORD;
       

        /*
         * Functions
         */

        /* Data streaming */
        void processor(gpointer Nodeparentchild,glong DLid,gchar* Password);
        gboolean pipeline_in(glong Node,glong DLid,gchar* Password);
        gboolean regexpmatch(gchar *string, gchar *filter);

        /* GUI's */
        gint alarmGETsocketXID(void);
        gint statusbarGETsocketXID(void);
        GtkWidget *gtkplugnew(gint XID,const gchar *text);

        /* Nodes */
        gboolean register_node(glong nodeobj,glong DLid,gchar* Password);
        gboolean registerPLUGIN2DL(glong Node,glong DLid,gchar* Password);
        gboolean unregister_node(glong Node);
        gboolean rules(gint mode,glong DLid,gchar* Password);
        glong get_node(glong Node);

        /* 
         * Friends
         */
        friend gint engine_bind2DL_timeout(void *);


    private:
        gpointer gtkplugdestroy(GtkWidget*, gpointer);
        gpointer gtkplugdestroyevent(GtkWidget*,gpointer);
        gboolean build_Piper_Paths(glong DLid,gchar* Password);
        gpointer runADMclientORB(gpointer data);
        gboolean bind2DL();
        gboolean is_bound2DL();
        gint bind2DL_timeout(void *);
         
};



//UNUSED; typedef struct _messaging_plugin messaging_plugin;
//UNUSED; struct _messaging_plugin {
//UNUSED; 	GString *fname;      /* filename */
//UNUSED; 	GString *dname;      /* description name */
//UNUSED; 	gint status;         /* TRUE/FALSE, loaded/not loaded */
//UNUSED; 	nodeType_t nodeType; /* messaging object type, see messaging_mobject.h */
//UNUSED; };

/* db internal functions needed by corba wrappers */
guint messaging_db_save_record(GIOChannel *savefile,gpointer data,gint count);
gint messaging_db_bind2DL_timeout(gpointer data);
//gboolean messaging_db_registerPLUGIN2DL(messaging_object *MO);

#endif
