#include <stdlib.h>
#include <fstream.h>
#include <string.h>
#include <stdio.h>
#include <time.h>
#include <sys/time.h>
#include <unistd.h>

#include <glib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stddef.h>
#include <regex.h>
#include <fnmatch.h>
#include <gmodule.h>
#include <dirent.h>
#include <signal.h>
#include <gtk/gtk.h>

#include "messaging.h"
#include "messaging_data.h"
#include "messaging_node.h"
// #include "../plugins/messaging_plugin.h"

/*
 *
 */


#ifndef LEGACY
#define LEGACY

class legacy {
    public:
        inline legacy() {
            /* check if modules are supported on this platform */
            if (!check_dl()) {
                g_error("Your environment does not support dynamic loading of modules.");
            }

            /* activate threads */
            if (!g_thread_supported()) {
                g_thread_init(NULL);
            }

            DB_LOCKED = FALSE;

            /* Setup log handler */
            log_handler_piper_id = g_log_set_handler (PIPER_LOG_DOMAIN, (GLogLevelFlags)(G_LOG_LEVEL_WARNING | G_LOG_LEVEL_MESSAGE | G_LOG_LEVEL_INFO | G_LOG_LEVEL_DEBUG | G_LOG_LEVEL_CRITICAL | G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION),log_handler_piper, this);
            log_handler_bl_id = g_log_set_handler (PIPER_BL_LOG_DOMAIN, (GLogLevelFlags)(G_LOG_LEVEL_WARNING | G_LOG_LEVEL_MESSAGE | G_LOG_LEVEL_INFO | G_LOG_LEVEL_DEBUG | G_LOG_LEVEL_CRITICAL | G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION),log_handler_bl, this);

            /* create directories */
            GString *base_dir = g_string_new("");
            GString *full_dir = g_string_new("");
            g_string_sprintf(base_dir,"%s/piper_info/",getenv("HOME"));
            mkdir(base_dir->str,S_IRWXU);

            g_string_sprintf(full_dir,"%s/history/",base_dir->str);
            mkdir(full_dir->str,S_IRWXU);
            g_string_sprintf(full_dir,"%s/ior/",base_dir->str);
            mkdir(full_dir->str,S_IRWXU);
            g_string_sprintf(full_dir,"%s/pids/",base_dir->str);
            mkdir(full_dir->str,S_IRWXU);
            g_string_sprintf(full_dir,"%s/storage/",base_dir->str);
            mkdir(full_dir->str,S_IRWXU);
            g_string_sprintf(full_dir,"%s/save/",base_dir->str);
            mkdir(full_dir->str,S_IRWXU);
            g_string_sprintf(full_dir,"%s/temp/",base_dir->str);
            mkdir(full_dir->str,S_IRWXU);
 
            g_string_free(base_dir,TRUE);
            g_string_free(full_dir,TRUE);
            
            /* write pid file */
            gint pid;
            gint pidfd;
            GString *pidfilename;
            gchar buf[32];

            /* check for an old pid file */
            pidfilename = g_string_new(getenv("HOME"));
            g_string_append(pidfilename,"/piper_info/pids/BL.pid");
            pidfd = open(pidfilename->str,O_RDONLY);
            if (pidfd != -1) {
                g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_ERROR,"There's already an BL instance running\n(or the BL crashed, so you'll have to remove ~/piper_info/pids/BL.pid)");
            }
            g_string_free(pidfilename,TRUE);

            /* write pid file to ~/piper_info/pids/BL.pid */
            pidfilename = g_string_new(getenv("HOME"));
            g_string_append(pidfilename,"/piper_info");
            mkdir(pidfilename->str,S_IRWXU);
            g_string_append(pidfilename,"/pids");
            mkdir(pidfilename->str,S_IRWXU);
            g_string_free(pidfilename,TRUE);

            pid = getpid();
            pidfilename = g_string_new(getenv("HOME"));
            g_string_append(pidfilename,"/piper_info/pids/BL.pid");
            pidfd = open(pidfilename->str,O_RDWR | O_CREAT,S_IRWXU);
            if (pidfd == -1) {
                g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_ERROR,"Error %i during CREATE|APPEND of file %s",errno,pidfilename->str);
                return;
            }
            snprintf(buf,32,"%d\n",pid);
            write(pidfd,buf,strlen(buf));
            close(pidfd);
            g_string_free(pidfilename,TRUE);
        }

        virtual ~legacy() {
            GString *pidfilename;
            /* Remove log handlers */
            g_log_remove_handler(PIPER_LOG_DOMAIN,log_handler_piper_id);
            g_log_remove_handler(PIPER_BL_LOG_DOMAIN,log_handler_bl_id);

            /* remove pid file */
            pidfilename = g_string_new(getenv("HOME"));
            g_string_append(pidfilename,"/piper_info/pids/BL.pid");
            unlink(pidfilename->str);
            g_string_free(pidfilename,TRUE);

            /* exit */
            g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_INFO,"BL clean shutdown");
            exit(0);
        }

        /*
         * FUNCTIONALITY
         */
        gboolean historyIO_create(void);
        gboolean historyIO_open(node* Node);
        gboolean historyIO_close(node* Node);
        glong    historyIO_write(node* Node);

        guint load_record(GIOChannel *loadfile,gchar *data,gint count, gchar* label);
        guint save_record(GIOChannel *savefile,gpointer data,gint count);

        void idle(glong delay);

        /* Plugins */
        gboolean check_dl(void);
        plugin_maker* load_plugin(gchar*,glong,gpointer);
        //gboolean node_link_plugin(node* Node,BL_plugin* plugin);

        /* Locking */
        void BL_lock(const gchar *text);
        void BL_unlock(const gchar *text);
        void BL_locknunlock(const gchar *text);

        /* Friends */
        friend void log_handler_piper(const gchar *log_domain,GLogLevelFlags log_level,const gchar *message,gpointer user_data);
        friend void log_handler_bl(const gchar *log_domain,GLogLevelFlags log_level,const gchar *message,gpointer user_data);

        /*
         * VARIABLES
         */
        gboolean DB_LOCKED;
        guint log_handler_piper_id;
        guint log_handler_bl_id;
};

#endif

