/*
 * The node (main messaging object) api
 * this is the structure in which data resides inside the
 * messaging system, NO other structure will be used
 *
 *
 * begin : 19- 6-1999, BastardAI
 *          8- 7-1999, Note Java-varinaming style
 *		   24-12-1999, Split code into api & structs 
 *					   during separation of ADM & DFP
 *         13- 1-2001, Started new 0.5 design cq OOP based.
 *
 */

#include <gtk/gtk.h>
#include "DL2BL.hh"
#include "messaging.h"
#include "messaging_data.h"

#ifndef NODE
#define NODE

/* 
 * DEFINES
 */
// #define MO_CACHE
// #define MO_CACHE_ELEMENTS 32


/* struct proto's */
typedef struct _uriS uriS;
typedef struct _MOpnc MOpnc;

struct _uriS {
    glong           instanceID;
    glong           subnetID;
    glong           nodeID;
};


/* the main node class, this is where the core node magic takes place */ 
class node {
    public:
        inline node() {
            g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_WARNING,"Wrong node constructor called, use node(const gchar*,const gchar*,const nodeType_t) instead");
        }
        inline node(const gchar *Dname,const gchar *Ddescription,const nodeType_t type,glong DLid) {
            /* allocate fields */
            dname = g_string_new(Dname);
            dDescription = g_string_new(Ddescription);
            dataDescription = g_string_new("undefined data");
            historyIO = NULL;
            historyFilename = g_string_new("");
            allowedFilter = g_string_new("");
            dataPassedOver = FALSE;
            configInit = g_string_new("");
            configRuntime = g_string_new("");

            /* setup uri */
            uri = (uriS*)g_malloc(sizeof(uriS));
            uri->instanceID = 1;
            uri->subnetID = DLid;
            /* allocate and setup data */
            historyIO = NULL;
            dataBlockedList = NULL; /* single linked list for historic data buffering */
            relationList = NULL; /* linkedlist of relations to other MO's */
            externalRelations = 0;
            /* default values */
            dataSequence = 0; /* default current to last MO in followup order */
            addresSpaceList = NULL; /* TODO implement addressSpace logic */
            nodeType = type; /* set MO type */
            procedure_running = FALSE;
            /* accept all data by default */
            g_string_sprintf(allowedFilter,"%s","");
            status = Passing;
            procedure_handled = TRUE;
            /* various procedure pointers to NULL */
            init_ref = NULL;
            cleanup_ref = NULL;
            activate_ref = NULL;
        }
        virtual ~node() {
            /* free node memory */
            g_string_free(dname,TRUE);
            g_string_free(dDescription,TRUE);
            g_string_free(dataDescription,TRUE);
            g_string_free(historyFilename,TRUE);
            g_string_free(allowedFilter,TRUE);
            /* clean LOCKED LIST */
            g_free(uri);
            g_string_free(configRuntime,TRUE);
            g_string_free(configInit,TRUE);
        }

        /*
         * variables
         */
        uriS *uri; /* universal relocation identifier */
        GString *dname; /* displayed name */
        GString *dDescription; /* displayed description */
        nodeType_t nodeType; /* messaging_object type */
        statusType_t status; /* status */
        GList *addresSpaceList; /* List of DL id's that are allowed to this MO */
        data* Data; /* data block of current MO */
        GString *dataDescription; /* Description of data block, sensors put this string into the MDOs they produce */
        gint dataSequence; /* followup number of data block ,N = first; N--; 0 = last */
        gboolean dataPassedOver; /* db internal detection dead ends */
        GSList *dataBlockedList; /* list of blocked data chunks (messaging_data_object*) */

        GString *historyFilename; /* Filename of .. */
        GIOChannel *historyIO; /* .. logfile after dataBlockedList */
        GList *relationList; /* other MObjects to which current one has some sort of relation to. */
        gulong externalRelations;   /* Number of relations from other nodes to this node */
        GString *allowedFilter; /* This string must be matched by offered data in order to accept it */
        gboolean procedure_handled; /* flag to indicate if procedure is handled */
        gboolean procedure_running; /* flag to indicate if procedure is running */

        /* OLD members. */
        // gboolean UIsetup; /* FALSE until fully setup, see NOTE1 below */
        // gboolean (*UIsetup_ref)(); /* FALSE until fully setup, see NOTE1 below */
        // GtkWidget* (*visualPlug_buildProcedure_ref)(node *Node,gint XID); /* builds visual UI */
        // GtkWidget* (*configPlug_buildProcedure_ref)(node *Node,gint XID); /* builds config UI */

        /*
         * pointers to node's function implementations. Conrresponding class
         * fuctions are wrappers that call code pointed to by these pointers
         */
        gboolean (*procedure_ref)(); /* operation on data by current mobject */
        gboolean (*cleanup_ref)(); /* cleanup handler, for procedure */
        GString *configInit; /* initialize configuration of MO */
        GString *configRuntime; /* runtime configuration of MO */
        gboolean (*init_ref)(glong Node,gpointer _Engine); /* initialize code */
        gboolean (*activate_ref)(); /* activation code */

        /*
         * various functionality
         */


        //DISABLED; inline GtkWidget* visualPlug_buildProcedure(gint XID) {
        //DISABLED;     GtkWidget *result;
        //DISABLED;     result = (*visualPlug_buildProcedure_ref)(this,XID);
        //DISABLED;     return result;
        //DISABLED; }

        /*
         * conversion fuctionality
         */
        gchar* nodetype_to_char(nodeType_t nt);
        gchar* rtype_to_char(rType_t rt);
        gchar* status_to_char(statusType_t status);
        gchar* allowedFilter_to_char(GString *filter);

};


/*
 * Structs
 */
struct _MOpnc {
    glong child;
    glong parent;
    gboolean threaded;
};

#endif

