/*
 *
 * Resource class manages the resources on the host system. It keeps track
 * of allocated data and node objects, schedules execution of nodes, handles 
 * relocation, etc.
 *
 * TODO: spend more as 1 minute on the intro text
 *
 */

#include "messaging_legacy.h"
#include "messaging_relation.h"

#ifndef RESOURCE
#define RESOURCE

#define DATA_MAXIMUM_OBJECTS 65535*8      // Experimental value, based upon various literature
#define NODE_MAXIMUM_OBJECTS 65535        // Experimental value, based upon various literature

class resource: public legacy {
    public:
        inline resource() {
            glong i;

            /* we aint got any dl registered */
            any_dl = FALSE;

            /* initialise locked lists */
            for (i=0;i<DATA_MAXIMUM_OBJECTS;i++) {
                DATA_LOCKED_LIST[i] = NULL;
            }
            for (i=0;i<NODE_MAXIMUM_OBJECTS;i++) {
                NODE_LOCKED_LIST[i] = NULL;
            }
            for (i=0;i<MAXDLID;i++) {
                DLS[i] = NULL;
            }

            /* Totals NODES and DATAS */
            Nodes = 0;
            NODE_DNAME_HASH = g_hash_table_new(g_str_hash,g_str_equal);
            NODE_FILTER_HASH = g_hash_table_new(g_str_hash,g_str_equal);
        }
        virtual ~resource() {
            save_state();
            /* remove all MO's */
            node *Node;
            while (g_list_length(NODES)>0) {
                 Node = (node*)g_list_last(NODES)->data;
                 g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_WARNING,"Destructing node %s",Node->dname->str);
                 if (Node->cleanup_ref != NULL) {
                     Node->cleanup_ref();
                 }
                 NODES = g_list_remove(NODES,Node);
                 delete Node;
            }
            NODES = NULL;
        }


        /*
         * FUNCTIONALITY
         */

        /* new & destruct of node and data */
        node* node_new(gchar *Dname,gchar *Ddescription,gchar *allowedfilter,nodeType_t type,glong DLid,gpointer init_ref,gpointer procedure_ref,gpointer cleanup_ref, gpointer activate_ref);
        data* data_new(gchar* DataDescription,guint RawDataLen);
        gboolean node_delete(node *Node);
        gboolean data_delete(data *Data);

        /* Lock lists management */
        gboolean data_id_lock(data* Data);
        gboolean data_id_unlock(data* Data);
        gboolean node_id_lock(node* Node);
        gboolean node_id_unlock(node* Node);

        /* area management */
        gboolean register_dl(glong dlID,gchar *password,glong mdlID,gchar *password_area);
        gboolean unregister_dl(glong dlID,gchar *password);

        /* Load & save */
        gboolean setup_nodes(gpointer Engine);
        gboolean save_state(void);
        gboolean load_state(gpointer Engine);
        gboolean load_plugins(gpointer Engine);

        /* Lookup functionality */
        node* node_lookup_by_dname(gchar* Dname);
        node* node_lookup_by_filter(gchar* Filter);

        /* Node stuff */
        gboolean register_node(gint DLid,node* Node,gpointer Engine);
        gboolean unregister_node(node* Node);
        gboolean activate_nodes(void);
        gboolean node_add_blockedlist(node* Node,data* Data);
        node* noderef_to_node(glong Node);
        glong node_to_noderef(node* Node);
        gboolean node_initialize(node* Node,gpointer init_ref,gpointer procedure_ref,gpointer cleanup_ref, gpointer activate_ref);

        /* Node info */
        node* node_get_relation(node* node,glong relation);
        glong node_get_currentrelations(node* node);
        glong node_get_childs(node* node);
        gpointer node_get_activate(node* node);
        GString* node_get_datadescription(node* node);
        glong nodes(void); /* return number of availeble nodes */

        /* Data */
        data* dataref_to_data(glong Data);
        glong data_to_dataref(data* Data);

        /* Relating related stuff */
        gboolean relate_node_to_node(node* parent,node* child);
        gboolean relate_node_to_filter(gchar *parent_filter,node* Node);
        gboolean relate_node_to_dname(gchar *parent_name,node* Node);
        gboolean relate_dname_to_node(node* Node,gchar *child_node);
        gboolean relate_dname_to_dname(gchar *parent_name,gchar *child_name);
        gboolean unrelate_node_to_dname(gchar *relation_name,node* Node);
        gboolean unrelate_node_to_node(node* parent,node* child);

        /* Dummies for overloading */
        virtual void conflictportal(node*,node*,rType_t) {};

        /* Friends */
        friend gint dirfilter(const struct dirent *entry);


        /*
         * VARIABLES
         */

        /* Lock lists for the ids of classes of types data and node */
        data* DATA_LOCKED_LIST[DATA_MAXIMUM_OBJECTS];
        node* NODE_LOCKED_LIST[NODE_MAXIMUM_OBJECTS];

        /* Lookup lists */
        GHashTable* NODE_DNAME_HASH;
        GHashTable* NODE_FILTER_HASH;

        /* Total lists */
        GList* NODES; /* Glist of all nodes */
        GList* DATAS; /* Glist of all data blocks */
        messaging_security_DL *DLS[MAXDLID];

        /* Total nodes */
        glong Nodes;

        /* DL */
        gboolean any_dl;
};

#endif

