/*
 * kaHTTP.c -- KQML API HTTP transport specific routines
 *
 * Copyright (c)  1993, 1994 Enterprise Integration Technologies Corporation
 * and Lockheed Missiles and Space Company, Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and 
 * its documentation for any purpose is hereby granted without fee, provided
 * that (i) the above copyright notices and this permission notice appear in
 * all copies of the software and related documentation, and (ii) the name of
 * Enterprise Integration Technologies Corporation and Lockheed Missiles
 * and Space Company may not be used in any advertising or publicity
 * relating to the software without the specific, prior written permission
 * of Enterprise Integration Technologies Corporation and Lockheed Missiles
 * and Space Company.
 * 
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
 *
 * IN NO EVENT SHALL ENTERPRISE INTEGRATION TECHNOLOGIES CORPORATION OR
 * LOCKHEED MISSILES AND SPACE COMPANY BE LIABLE FOR ANY SPECIAL,
 * INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY
 * DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY
 * THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 */

#ifdef DO_HTTP_TRANS

#include <kapi_int.h>
#include <kaHTTP.h>

#define HTTPPORT 80

/*
 *----------------------------------------------------------------------
 *
 * httpParseURL --
 *
 *	Parses URLs beginning with 'http://'
 *
 * Results:
 *	ParsedURL if successful
 *      NULL for failure
 *
 * Side effects:
 *      none
 *
 *----------------------------------------------------------------------
 */

ParsedURL *
#if defined(__STDC__) || defined(__cplusplus)
httpParseURL(char *urlstring, int ttype)
#else
httpParseURL(urlstring, ttype)
     char *urlstring;
     int ttype;
#endif
{
  ParsedURL *purl = (ParsedURL *) calloc(1,sizeof(ParsedURL));
  HTTPInfo *info = (HTTPInfo *) calloc(1,sizeof(HTTPInfo));
  char tmpbuffer[MAXURLLENGTH], *t = NULL;

  bzero(tmpbuffer, MAXURLLENGTH);

  if (urlstring == NULL || strncasecmp(urlstring,"http://", 7))
    return(NULL);
  strcpy(tmpbuffer, &urlstring[7]); /* Skip "http://" part */

  strcpy(info->host, strtok(tmpbuffer, ":"));

  if ((t=strtok(NULL,"")) && 
      (info->port = atoi(t))) {
    /* specified port is fine */
  }
  else {
    info->port = HTTPPORT;
  }

  info->server = 0; /* default to client */
  purl->fd = -1;
  purl->transtype = ttype;
  purl->state = CLOSED_CON;
  purl->transinfo = (void *) info;
  strcpy(purl->url, urlstring);
  strcpy(purl->name, "none");
  purl->alternate = NULL;

  return(purl);
}


/*
 *----------------------------------------------------------------------
 *
 * httpListen --
 *
 *	Listen on http url host & port
 *
 * Results:
 *	1 if successful
 *      0 for failure
 *
 * Side effects:
 *      Possible establishment of connection
 *
 *----------------------------------------------------------------------
 */

int
#if defined(__STDC__) || defined(__cplusplus)
httpListen(ParsedURL *purl)
#else
httpListen(purl)
     ParsedURL *purl;
#endif
{
  HTTPInfo *info = (HTTPInfo *) purl->transinfo;

  if ((purl->fd =
       getTCPSocketAddr(info->port)) != -1) {
    if (TCPlisten(purl->fd) != -1) {
      purl->state = LISTENING_CON;
    } else {
#if (DEBUG > 0)
      fprintf(stderr, "KConnect: Can't listen to socket.\n");
#endif
      purl->state = CONNECTION_ERROR;
      return(0);
    }
  } else {
#if (DEBUG > 0)
    fprintf(stderr, "KConnect: Can't find socket.\n");
#endif
    purl->state = CONNECTION_ERROR;
    return(0);
  }
    
  return(1);
}


/*
 *----------------------------------------------------------------------
 *
 * httpConnect --
 *
 *	Accept connection on http url host & port
 *
 * Results:
 *	parsedURL of new connection
 *      NULL for failure
 *
 * Side effects:
 *      Possible establishment of connection, updating of OpenConns hashtab
 *
 *----------------------------------------------------------------------
 */

ParsedURL *
#if defined(__STDC__) || defined(__cplusplus)
httpConnect(ParsedURL *purl, int ttype)
#else
httpConnect(purl, ttype)
     ParsedURL *purl;
     int ttype;
#endif
{
  int fd, flg;
  ParsedURL *newpURL;
  HTTPInfo *info = NULL;
  char connectionName[MAXNAMESIZE];
  HashEntry *entry = NULL;

  bzero(connectionName, MAXNAMESIZE);

  fd = TCPaccept(purl->fd);
  if (fd <= 0)
    return NULL;

  newpURL = (ParsedURL *) calloc(1,sizeof(ParsedURL));
  info = (HTTPInfo *) calloc(1,sizeof(HTTPInfo));

  newpURL->fd = fd;
  strcpy(info->host, "unknown");
  info->port = 0;
  info->server = 1;

  newpURL->state = OPEN_CON;
  newpURL->transinfo = info;
  newpURL->transtype = ttype;

  sprintf(connectionName, "%d", fd);

  strcpy(newpURL->name, connectionName);

  entry = CreateHashEntry(&OpenConns, connectionName, &flg);
  if (!flg) {
    DeletePurl(GetHashValue(entry));
    DeleteHashEntry(entry);
    entry = CreateHashEntry(&OpenConns, connectionName, &flg);
  }

  SetHashValue(entry, newpURL);
  return(newpURL);
}


/*
 *----------------------------------------------------------------------
 *
 * httpDisConnect --
 *
 *      Terminate connection on http url host & port
 *
 * Results:
 *	returns 1
 *
 * Side effects:
 *      removal of connection, updating of OpenConns hashtab
 *
 *----------------------------------------------------------------------
 */

int
#if defined(__STDC__) || defined(__cplusplus)
httpDisconnect(ParsedURL *purl)
#else
httpDisconnect(purl)
     ParsedURL *purl;
#endif
{  
  struct linger l;
  int j;
  HTTPInfo *info = NULL;

  if (purl->state == OPEN_CON) {
    l.l_onoff = 0;
    l.l_linger = 0.0;
    j = setsockopt(purl->fd, SOL_SOCKET, SO_LINGER,
		   (char *) &l, sizeof(l));
    info = (HTTPInfo *) purl->transinfo;
#if (DEBUG > 0)
    fprintf(stderr, "Closing HTTP file descriptor on port %d\n",
	    info->port);
#endif
    close(purl->fd);
    purl->state = CLOSED_CON;
  }

  return(1);
}


/*
 *----------------------------------------------------------------------
 *
 * httpGet --
 *
 *      Get message on http url.
 *
 * Results:
 *	returns message if successful.
 *      NULL for failure.
 *
 * Side effects:
 *      Possible removal of connection, updating of OpenConns hashtab
 *
 *----------------------------------------------------------------------
 */

char *
#if defined(__STDC__) || defined(__cplusplus)
httpGet(ParsedURL *purl, char *str)
#else
httpGet(purl, str)
     ParsedURL *purl;
     char *str;
#endif
{
  char buffer[K_MAX_MESSAGE_SIZE];
  char *kqmlstr = NULL;
  HashEntry *entry = NULL;
  HTTPInfo *info = (HTTPInfo *) purl->transinfo;

  bzero(buffer, K_MAX_MESSAGE_SIZE);

  HTTPrecv(buffer, K_MAX_MESSAGE_SIZE, purl->fd);

  /* If acting as a client, need to close connection */
  if (!info->server) {
    purl->state = CLOSED_CON;
    close(purl->fd);
    entry = FindHashEntry(&OpenConns, purl->name);
    DeleteHashEntry(entry);
  }

  if (buffer == NULL) 
    return NULL;

  if ((kqmlstr = mime2kqml(buffer)) == NULL)
    return NULL;

  strcpy(str, kqmlstr);
  free(kqmlstr);

  return(str);
}


/*
 *----------------------------------------------------------------------
 *
 * httpSend --
 *
 *      Send message on http url
 *
 * Results:
 *	returns 1 if successful
 *      -1 for failure
 *
 * Side effects:
 *      Possible new connection, updating of OpenConns hashtab
 *
 *----------------------------------------------------------------------
 */

int
#if defined(__STDC__) || defined(__cplusplus)
httpSend(ParsedURL *purl, char *buf)
#else
httpSend(purl, buf)
     ParsedURL *purl;
     char *buf;
#endif
{
  int status, flg;
  char buffer[K_MAX_MESSAGE_SIZE];
  char *tmpbuf = NULL;
  ParsedURL *apurl = NULL;
  HTTPInfo *info = (HTTPInfo *) purl->transinfo;
  HashEntry *entry = NULL;

  /* turn the KQML message in buf into a proper HTTP request, e.g.
     by turning the performative name into a method name, a
     :object parameter value into the method's argument, the
     :language parameter value into a Content-type header (KB/KIF
     by default), the :content parm value into the MIME body, and
     other parameter name/value pairs into header lines.  Take
     the server's response and feed it into KGetString's buffers.
     */

  bzero(buffer, K_MAX_MESSAGE_SIZE);

  tmpbuf = kqml2mime(buf);
  if (tmpbuf == NULL) 
    return(-1);

  strcpy(buffer, tmpbuf);
  free(tmpbuf);

  /* Since HTTP queries are transaction-based, need to reconnect to
     the HTTP server everytime a new send is required. */

  if (purl->state != OPEN_CON) {
    purl->fd = TCPconnect(info->host,
			  info->port);

    purl->state = OPEN_CON;

    if (purl->fd == -1)
      return -1;

    entry = CreateHashEntry(&OpenConns, purl->name, &flg);
    if (!flg) {
      DeletePurl(GetHashValue(entry));
      DeleteHashEntry(entry);
      entry = CreateHashEntry(&OpenConns, purl->name, &flg);
    }
    SetHashValue(entry, purl);
  }

  status = TCPsend(buffer, purl->fd);

  /* If acting as a server, close connection after send */
  if (info->server) {
    purl->state = CLOSED_CON;
    close(purl->fd);
    entry = FindHashEntry(&OpenConns, purl->name);
    DeleteHashEntry(entry);
  }

  if (status == -1) {
#if (DEBUG > 0)
    fprintf(stderr, "Kasend: Could not send message to server.\n");
#endif
    return 0;
  }
  return(1);
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/*       HTTP-based ANS (Agent Name Server) Code                   */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

int
#if defined(__STDC__) || defined(__cplusplus)
httpQueryANS(char* aname, ParsedURL *anspurl, char *returnbuf)
#else
httpQueryANS(aname, anspurl, returnbuf)
     char *aname;        /* the name of the agent to lookup */
     ParsedURL *anspurl; /* the location information for the ANS */
     char *returnbuf;    /* string containing newline delimited urls */
#endif
{
  char buffer[K_MAX_MESSAGE_SIZE];
  HTTPInfo *info = (HTTPInfo *) anspurl->transinfo;

  bzero(buffer, K_MAX_MESSAGE_SIZE);

  /* Make a HTTP connection to ANS. 
     For now, each agent will have its own file whose contents are composed
     of a set of URLs as described in kapi.h.
     
     The files are stored under $HTTPD_HOME/htdocs/agent/ where HTTPD_HOME
     corresponds to the location where the NCSA httpd server was installed.
     */
     
  if ((anspurl->fd =
       TCPconnect(info->host,
		  info->port)) == -1) {
#if (DEBUG > 0)
    fprintf(stderr, "KLookup: ANS server via HTTP is not available.\n");
#endif
    return K_NO_ANS_RESPONSE;
  }
  
  anspurl->state = OPEN_CON;
  
  /* Authentication might be needed */
  
  sprintf(buffer, "GET /agent/%s\n", aname);
  TCPsend(buffer, anspurl->fd);
  
  TCPrecv(buffer, K_MAX_MESSAGE_SIZE, anspurl->fd);
  
  if(buffer==NULL)
    return(K_ERROR);

  strcpy(returnbuf, buffer);
  close(anspurl->fd);
  anspurl->fd = 0;
  anspurl->state = CLOSED_CON;
  
  /* Return transports list. */
  return(K_OK);
}

/*      */
int
#if defined(__STDC__) || defined(__cplusplus)
httpRegisterANS(ParsedURL *anspurl, char* aname, char *aurl)
#else
httpRegisterANS(anspurl, aname, aurl)
     ParsedURL *anspurl;  /* the name server parsed url */
     char *aname;        /* the name of the agent to lookup */
     char *aurl;         /* the url to add */
#endif
{
  char buffer[K_MAX_MESSAGE_SIZE];
  HTTPInfo *info = (HTTPInfo *) anspurl->transinfo;
  
  bzero(buffer, K_MAX_MESSAGE_SIZE);

  /* Make a HTTP connection to ANS. 
     For now, each agent will have its own file whose contents are composed
     of a set of URLs as described in kapi.h.
     
     The files are stored under $HTTPD_HOME/htdocs/agent/ where HTTPD_HOME
     corresponds to the location where the NCSA httpd server was installed.
     */
  
  if ((anspurl->fd =
       TCPconnect(info->host,
		  info->port)) == -1) {
#if (DEBUG > 0)
    fprintf(stderr, "ANSLookup: ANS server via HTTP is not available.\n");
#endif
    return K_NO_ANS_RESPONSE;
  }
  
  
  /* Authentication might be needed */
  sprintf(buffer, "GET /cgi-bin/kapi_register?%s=%s\n", aname, aurl);
  TCPsend(buffer, anspurl->fd);

  bzero(buffer, K_MAX_MESSAGE_SIZE);
  
  HTTPrecv(buffer, K_MAX_MESSAGE_SIZE, anspurl->fd);
  
  close(anspurl->fd);
  anspurl->fd = 0;
  
  return(K_OK);
}

/*      */
int
#if defined(__STDC__) || defined(__cplusplus)
httpUnRegisterANS(ParsedURL *anspurl, char* aname, char *aurl)
#else
httpUnRegisterANS(anspurl, aname, aurl)
     ParsedURL *anspurl;  /* the name server parsed url */
     char *aname;        /* the name of the agent to lookup */
     char *aurl;         /* the url to add */
#endif
{
  char buffer[K_MAX_MESSAGE_SIZE];
  HTTPInfo *info = (HTTPInfo *) anspurl->transinfo;

  bzero(buffer, K_MAX_MESSAGE_SIZE);

  /* Make a HTTP connection to ANS. 
     For now, each agent will have its own file whose contents are composed
     of a set of URLs as described in kapi.h.
     
     The files are stored under $SERVER_DOCROOT/agent/ where SERVER_DOCROOT
     corresponds to the location where the NCSA httpd server's root document
     directory was installed.
     */
  
  if ((anspurl->fd =
       TCPconnect(info->host,
		  info->port)) == -1) {
#if (DEBUG > 0)
    fprintf(stderr, "ANSLookup: ANS server via HTTP is not available.\n");
#endif
    return K_NO_ANS_RESPONSE;
  }
  
  /* Authentication might be needed */
  
  sprintf(buffer, "GET /cgi-bin/kapi_unregister?%s=%s\n", aname, aurl);
  TCPsend(buffer, anspurl->fd);
  
  bzero(buffer, K_MAX_MESSAGE_SIZE);

  HTTPrecv(buffer, K_MAX_MESSAGE_SIZE, anspurl->fd);
  
  close(anspurl->fd);
  
  anspurl->fd = 0;
  return(K_OK);
}
  

#endif
  
