/*
 * server.c -- example KAPI program
 *
 * Copyright (c)  1993, 1994 Enterprise Integration Technologies Corporation
 *
 * Permission to use, copy, modify, distribute, and sell this software and 
 * its documentation for any purpose is hereby granted without fee, provided
 * that (i) the above copyright notices and this permission notice appear in
 * all copies of the software and related documentation, and (ii) the name of
 * Enterprise Integration Technologies Corporation may not be used in any 
 * advertising or publicity relating to the software without the specific, 
 * prior written permission of Enterprise Integration Technologies Corporation.
 * 
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
 *
 * IN NO EVENT SHALL ENTERPRISE INTEGRATION TECHNOLOGIES CORPORATION  BE
 * LIABLE FOR ANY SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF
 * ANY KIND, OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
 * PROFITS, WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY
 * THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 */

#include	<stdio.h>
#include	<string.h>
#include	"kapi.h"

#define         AGENTNAME       "server1"
#define         AGENT_URL       SERVER1_URL      

/* variable controlling number of error messages */
int msg_no=0;

int main(argc, argv)
  int argc;
  char *argv[];
{
  char	serverurl[20],
	ans[2], message[K_MAX_MESSAGE_SIZE],
	reply_msg[K_MAX_MESSAGE_SIZE],
	*p;

  /* This section demonstrates use of KAPI parsing utilities */
  char *performative=NULL;
  char *sender=NULL;
  char *receiver=NULL;
  char *reply_with=NULL;

  KeyList *kl = NULL;
   
  /* The following approach is used to keep non-ANSI compilers happy.
     gcc allows direct initializtion of the KTable as in
  KeydPair KTable[] = {
    {":sender", &sender},
    {":receiver", &receiver},
    {":reply_with", &reply_with},
    {(char *) NULL, (char **) NULL},
    }; */

  KeydPair KTable[4];
  char *key1 = ":sender";
  char *key2 = ":receiver";
  char *key3 = ":reply-with";
  KTable[0].keyword = key1;
  KTable[0].value_address = &sender;
  KTable[1].keyword = key2;
  KTable[1].value_address = &receiver;
  KTable[2].keyword = key3;
  KTable[2].value_address = &reply_with;
  KTable[3].keyword = (char *) NULL;
  KTable[3].value_address = (char **) NULL;


  /* By default this agent will receive messages at
     tcp://'HOST':9374 */
  sprintf(serverurl,"tcp://%s:%s", getenv("HOST"), SERVER_PORT);

  /* argv[1] if given is the URL of a ANS to connect to */
  if (argc > 1) {
    if (KRegister(argv[1], AGENTNAME, serverurl)) 
      printf("<%s> registered new protocol: %s\n", AGENTNAME,
	     serverurl);
    else {
      printf("Registration failed\n");
      exit(0);
    }
    /* Lookup the agent with ANS to check */
    printf("The client transport protocols are:\n%s\n",
	   KLookup(argv[1], AGENTNAME, message));
    
    /* Connect the agent to the ANS */
    if (KSetUp(AGENTNAME, argv[1], 1) == -1) {
      printf("Error: cannot connect to agent name server.\n");
      exit(0);
    } 
  } else {
    /* If no ANS is given manually listen on URL */
    KSetUp(AGENTNAME, NULL, 0);
    KCacheURL("client", "tcp://piano:9347");

    if (KListen(serverurl) < 0) {
      fprintf(stderr,"KListen failure\n");
      exit(0);
    }
    printf("** Running without an Agent Name Server **\n");
  }

  printf("\n ************ %s ************\n", argv[0]);

  while (1) { 
    /* get reply */
    if (KGetString_or_Wait(message) != NULL) {
      printf("Message received: %s\n\n", message);

      if (!KParse(message, &performative, KTable, &kl) && sender) {
	sprintf(reply_msg,"(tell :sender server :receiver %s :content (Message %s was received))", sender, message);
      
	/* send reply to sender */
	if (!KSendString(reply_msg)) 
	  printf("Error: cannot send message to destination %s.\n",
		 sender);
      } else
	printf("Error: cannot reply to message");
    }      
    printf("\n ============== %d ==============\n", ++msg_no);
  } 
}
