/*
 * server1.c -- example KAPI program
 *
 * Copyright (c)  1993, 1994 Enterprise Integration Technologies Corporation
 * and Lockheed Missiles and Space Company, Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and 
 * its documentation for any purpose is hereby granted without fee, provided
 * that (i) the above copyright notices and this permission notice appear in
 * all copies of the software and related documentation, and (ii) the name of
 * Enterprise Integration Technologies Corporation and Lockheed Missiles
 * and Space Company may not be used in any advertising or publicity
 * relating to the software without the specific, prior written permission
 * of Enterprise Integration Technologies Corporation and Lockheed Missiles
 * and Space Company.
 * 
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
 *
 * IN NO EVENT SHALL ENTERPRISE INTEGRATION TECHNOLOGIES CORPORATION OR
 * LOCKHEED MISSILES AND SPACE COMPANY BE LIABLE FOR ANY SPECIAL,
 * INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY
 * DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY
 * THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 */

#include	<stdio.h>
#include	<string.h>
#include	<kapi.h>
#include        <example-setup.h>

#define         AGENTNAME       "server1"
#define         AGENT_URL       SERVER1_URL

/* variable controlling number of error messages */
int msg_no=0;

int main(argc, argv)
  int argc;
  char *argv[];
{
  char	ans[2], message[K_MAX_MESSAGE_SIZE], reply_msg[K_MAX_MESSAGE_SIZE];

  /* This section demonstrates use of KAPI parsing utilities */
  char *performative=NULL;
  char *sender=NULL;
  char *receiver=NULL;
  char *reply_with=NULL;

  KeyList *kl = NULL;
   
  /* The following approach is used to keep non-ANSI compilers happy.
     gcc allows direct initializtion of the KTable as in
  KeydPair KTable[] = {
    {":sender", &sender},
    {":receiver", &receiver},
    {":reply_with", &reply_with},
    {(char *) NULL, (char **) NULL},
    }; */

  KeydPair KTable[4];
  char *key1 = ":sender";
  char *key2 = ":receiver";
  char *key3 = ":reply-with";
  KTable[0].keyword = key1;
  KTable[0].value_address = &sender;
  KTable[1].keyword = key2;
  KTable[1].value_address = &receiver;
  KTable[2].keyword = key3;
  KTable[2].value_address = &reply_with;
  KTable[3].keyword = (char *) NULL;
  KTable[3].value_address = (char **) NULL;


  KSetUp(AGENTNAME, NULL, 0);
  KCacheURL("client", CLIENT_URL);

  if (KListen(AGENT_URL) != K_OK) {
    fprintf(stderr,"KListen failure\n");
    exit(0);
  }


  printf("\n ************ %s ************\n", argv[0]);

  while (1) { 

    bzero(message, K_MAX_MESSAGE_SIZE);
    bzero(reply_msg, K_MAX_MESSAGE_SIZE);

    /* get reply */
    if (KGetString_or_Wait(message) == K_OK) {
      printf("Message received: %s\n\n", message);

      if (!KParse(message, &performative, KTable, &kl) && sender) {
	sprintf(reply_msg,"(tell :sender server :receiver %s :content (Message %s was received))", sender, message);
      
	/* send reply to sender */
	if (KSendString(reply_msg) != K_OK) 
	  printf("Error: cannot send message to destination %s.\n",
		 sender);
      } else
	printf("Error: cannot reply to message");
    }      
    printf("\n ============== %d ==============\n", ++msg_no);
  } 
}
