;;; -*- Mode:Lisp; Syntax:Common-Lisp; Package:KAPI; Base:10 -*-
;;-----------------------------------------------------------------------------
;;; kapi-util.cl
;;;
;;; Miscellaneous KAPI utility functions.
;;;----------------------------------------------------------------------------
;;; Copyright (c)  1993, 1994 Lockheed Missiles and Space Company, Inc.
;;; and Enterprise Integration Technologies Corporation
;;;
;;; Permission to use, copy, modify, distribute, and sell this software and 
;;; its documentation for any purpose is hereby granted without fee, provided
;;; that (i) the above copyright notices and this permission notice appear in
;;; all copies of the software and related documentation, and (ii) the name of
;;; Enterprise Integration Technologies Corporation and Lockheed Missiles
;;; and Space Company may not be used in any advertising or publicity
;;; relating to the software without the specific, prior written permission
;;; of Enterprise Integration Technologies Corporation and Lockheed Missiles
;;; and Space Company.
;;; 
;;; THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
;;; EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
;;; WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
;;;
;;; IN NO EVENT SHALL ENTERPRISE INTEGRATION TECHNOLOGIES CORPORATION OR
;;; LOCKHEED MISSILES AND SPACE COMPANY BE LIABLE FOR ANY SPECIAL,
;;; INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY
;;; DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
;;; WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY
;;; THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
;;; PERFORMANCE OF THIS SOFTWARE.
;;;----------------------------------------------------------------------------
(eval-when (:load-toplevel :compile-toplevel :execute)
  (unless (find-package :kapi)
    (defpackage :kapi))
  (in-package :kapi))


;;;; LISP KSENDLIST ;;;;

(defun Lisp-KSendList (l-list)
  (Lisp-KSendString (format nil "~s" l-list)))


;;;; LISP KGETLIST ;;;;

(defun Lisp-KGetList ()
  (read-from-string (Lisp-KGetString)))


;;;; LISP KGETLIST-OR-WAIT ;;;;
    
(defun Lisp-KGetList-Or-Wait ()
  (read-from-string (Lisp-KGetString-Or-Wait)))


;;;----------------------------------------------------------------------------
;;; escape-field-value ensures that the value of the field will be read
;;; by Lisp as a string. Useful for maintaining case sensitive sender and 
;;; receiver names. 
;;; An optional filterfn can be passed to do additional processing on
;;; on the field.
;;;----------------------------------------------------------------------------
(defun escape-field-value (field str &optional filterfn)
  (let ((ref (search field str))
	s1 start end)
    (if (or (null ref)                                       ;; No field.
	    (null (setq s1 (search " " (subseq str ref)))))  ;; No value.
	str
      (progn 
	(setq start (+ 1 ref s1))
	(setq end   (+ start (or (search " :" (subseq str start))
				 (search ")"  (subseq str start)))))
	(concatenate 'string 
	  (subseq str 0 start)
	  (format nil "\"~a\"" 
		  (if filterfn
		      (funcall filterfn (subseq str start end))
		    (subseq str start end)))
	  (subseq str end))))))

;;;----------------------------------------------------------------------------
;;; remove-semicolon is a filterfn that removes all text following and 
;;; including a semicolon. This is specifically meant to be used in 
;;; conjunction with escape-field-value to convert "foo;7" to "foo". 
;;; 
;;; Note: Applying this to :sender agent names subverts the efficiency
;;;  gained by reusing an open connection. C'est la vie.
;;;----------------------------------------------------------------------------
(defun remove-semicolon (str)
  (subseq str 0 (position #\; str)))

;;;----------------------------------------------------------------------------
;;; kapi-read returns a list of the message string.
;;;----------------------------------------------------------------------------
(defun kapi-read ()
  (multiple-value-bind (ret str) (lisp-kgetstring)
    (when (= ret *k-ok*)
      str)))
	   
;;; End of file.