/*
 * kaMBUS.c -- KQML API MBUS transport specific routines
 *
 * Copyright (c)  1993, 1994 Enterprise Integration Technologies Corporation
 * and Lockheed Missiles and Space Company, Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and 
 * its documentation for any purpose is hereby granted without fee, provided
 * that (i) the above copyright notices and this permission notice appear in
 * all copies of the software and related documentation, and (ii) the name of
 * Enterprise Integration Technologies Corporation and Lockheed Missiles
 * and Space Company may not be used in any advertising or publicity
 * relating to the software without the specific, prior written permission
 * of Enterprise Integration Technologies Corporation and Lockheed Missiles
 * and Space Company.
 * 
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
 *
 * IN NO EVENT SHALL ENTERPRISE INTEGRATION TECHNOLOGIES CORPORATION OR
 * LOCKHEED MISSILES AND SPACE COMPANY BE LIABLE FOR ANY SPECIAL,
 * INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY
 * DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY
 * THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 */

#ifdef DO_MBUS_TRANS

#include <kapi_int.h>

#include <kasexp.h>
#include <api.h>
#include <kaMBUS.h>

#define  MBUSPORT   0x956

int MBLogLevel = 0;
static int mbusconns = 0;

/*
 *----------------------------------------------------------------------
 *
 * mbusParseURL --
 *
 *	Parses URLs beginning with 'mbus://'
 *
 * Results:
 *	ParsedURL if successful
 *      NULL for failure
 *
 * Side effects:
 *      none
 *
 *----------------------------------------------------------------------
 */

ParsedURL *
#if defined(__STDC__) || defined(__cplusplus)
mbusParseURL(char *urlstring, int ttype)
#else
mbusParseURL(urlstring, ttype)
     char *urlstring;
     int ttype;
#endif
{
  ParsedURL *purl = (ParsedURL *) calloc(1,sizeof(ParsedURL));
  ParsedURL *tmpurl = NULL;
  MBUSInfo *info = (MBUSInfo *) calloc(1,sizeof(MBUSInfo));
  MBUSInfo *tmpinfo = NULL;
  HashEntry *entry = NULL;
  char tmpbuffer[MAXURLLENGTH];
  char *hostNport = NULL, *t = NULL;
  int i = 0;

  bzero(tmpbuffer, MAXURLLENGTH);

  if (urlstring == NULL || strncasecmp(urlstring,"mbus://", 7))
    return(NULL);
  strcpy(tmpbuffer, &urlstring[7]); /* Skip "mbus://" part */
  
  if ((hostNport = strtok(tmpbuffer, "/")) == NULL ||
      (t = strtok(NULL, "")) == NULL) {
#if (DEBUG > 0)
    fprintf(stderr, "Bad mbus URL\n");
#endif
    return NULL;
  }

  strcpy(info->agentname, t);
  strcpy(info->host, strtok(hostNport, ":"));

  if ((t=strtok(NULL,"")) && 
      (info->port = atoi(t))) {
    /* specified port is fine */
  }
  else {
    info->port = MBUSPORT;
  }

  info->mbconn = 0;
  purl->fd = 0;
  purl->transtype = ttype;
  purl->state = CLOSED_CON;

  while (i++ < mbusconns) {
    /* Find out if already connected to this mbus */
    sprintf(tmpbuffer,"MBCONN%d", i);
    if (entry = FindHashEntry(&OpenConns, tmpbuffer)) {
      tmpurl = (ParsedURL *) GetHashValue(entry);
      tmpinfo = (MBUSInfo *) tmpurl->transinfo;
      if (strcasecmp(info->host, tmpinfo->host) == 0 &&
	  info->port == tmpinfo->port) {
	info->mbconn = tmpinfo->mbconn;
	purl->fd = tmpurl->fd;
	purl->state = SHARED_CON;
	break;
      }
    }
  }

  purl->transinfo = (void *) info;
  strcpy(purl->url, urlstring);
  strcpy(purl->name, info->agentname);
  purl->alternate = NULL;

  return(purl);
}


/*
 *----------------------------------------------------------------------
 *
 * mbusListen --
 *
 *	Listen on mbus url host & port
 *
 * Results:
 *	1 if successful
 *      0 for failure
 *
 * Side effects:
 *      Possible establishment of connection
 *
 *----------------------------------------------------------------------
 */

int
#if defined(__STDC__) || defined(__cplusplus)
mbusListen(ParsedURL *purl)
#else
mbusListen(purl)
     ParsedURL *purl;
#endif
{
  MBUSInfo *info = (MBUSInfo *) purl->transinfo;
  char buf[K_MAX_MESSAGE_SIZE];
  HashEntry *entry = NULL;
  char key[MAXNAMESIZE];
  int flg;

  bzero(buf, K_MAX_MESSAGE_SIZE);

  /* connect to the appropriate mbus and register interest in
     your messages */

  sprintf(buf, "(accept \".*\" %s)", info->agentname);

  if ((info->mbconn =
       MBConnect(info->host,
		 info->port)) &&
      MBtransmit_Cstring(info->mbconn, buf)
      == strlen(buf)) {


    purl->state = SHARED_CON;
    purl->fd = MBconnection_fd(info->mbconn);

    sprintf(key, "MBCONN%d", ++mbusconns);
    strcpy(purl->name, key);

    entry = CreateHashEntry(&OpenConns, key, &flg);
    SetHashValue(entry, purl);

  } else {
    purl->state = CONNECTION_ERROR;
    return(0);
  }
    
  return(1);
}


/*
 *----------------------------------------------------------------------
 *
 * mbusConnect --
 *
 *	Accept connection on mbus url host & port
 *
 * Results:
 *	parsedURL of new connection
 *      NULL for failure
 *
 * Side effects:
 *      Possible establishment of connection, updating of OpenConns hashtab
 *
 *----------------------------------------------------------------------
 */

ParsedURL *
#if defined(__STDC__) || defined(__cplusplus)
mbusConnect(ParsedURL *purl, int ttype)
#else
mbusConnect(purl, ttype)
     ParsedURL *purl;
     int ttype;
#endif
{
  /* Should never be called */
#if (DEBUG > 0)
  fprintf(stderr, "Call to accept for mbus: shouldn't happen\n");
#endif
  return(NULL);
}

/*
 *----------------------------------------------------------------------
 *
 * mbusDisConnect --
 *
 *      Terminate connection on mbus url host & port
 *
 * Results:
 *	returns 1
 *
 * Side effects:
 *      removal of connection, updating of OpenConns hashtab
 *
 *----------------------------------------------------------------------
 */

int
#if defined(__STDC__) || defined(__cplusplus)
mbusDisconnect(ParsedURL *purl)
#else
mbusDisconnect(purl)
     ParsedURL *purl;
#endif
{
  MBUSInfo *info = (MBUSInfo *) purl->transinfo;

  MBtransmit_Cstring(info->mbconn, "(close)");
  purl->state = CLOSED_CON;
  close(purl->fd);
  return(1);
}

/*
 *----------------------------------------------------------------------
 *
 * mbusGet --
 *
 *      Get message on mbus url.
 *
 * Results:
 *	returns message if successful.
 *      NULL for failure.
 *
 * Side effects:
 *      Possible removal of connection, updating of OpenConns hashtab
 *
 *----------------------------------------------------------------------
 */

char *
#if defined(__STDC__) || defined(__cplusplus)
mbusGet(ParsedURL *purl, char *str)
#else
mbusGet(purl, str)
     ParsedURL *purl;
     char *str;
#endif
{
  t_sexp MBNextMessage(), MBcar(), MBnthcdr();
  char *MBCstring();
  t_sexp nm, val;
  MBUSInfo *info = (MBUSInfo *) purl->transinfo;

  /* The sleep call was put in a previous version with
     no documentation as to why
     sleep(1);
     */
  val = MBNextMessage(info->mbconn);
  if (val == NULL) return NULL;

  nm = MBcar(MBnthcdr(val, 2));
  if (nm == NULL) return NULL;

  KSexpToString(nm, str);
  MBfree(nm);
  return(str);
}


/*
 *----------------------------------------------------------------------
 *
 * mbusSend --
 *
 *      Send message on mbus url
 *
 * Results:
 *	returns 1 if successful
 *      -1 for failure
 *
 * Side effects:
 *      Possible new connection, updating of OpenConns hashtab
 *
 *----------------------------------------------------------------------
 */

int
#if defined(__STDC__) || defined(__cplusplus)
mbusSend(ParsedURL *purl, char *buf)
#else
mbusSend(purl, buf)
     ParsedURL *purl;
     char *buf;
#endif
{
  ParsedURL *newpurl = NULL;
  MBUSInfo *newinfo = NULL;
  MBUSInfo *info = (MBUSInfo *) purl->transinfo;
  char tmpbuf[MAXNAMESIZE];
  char buf2[K_MAX_MESSAGE_SIZE];
  HashEntry *entry = NULL;
  int flg;
  int paren=0,quote=0,bs=0,inquote=0,sp=0,iqparen=0,nextnormal=0;
  char *foo;
  char *sender = NULL;

  /* This function contains legacy code that apparently fixes a problem
     with mbus's parsing behavior */

  /* first, expand quote characters into (quote ...) to accommodate
     mbus's sexpression format */

#define CANDG  {*foo++ = *buf;continue;}
#define NORMAL(c) ((c!=' ')&&(c!='(')&&(c!='"'))&&(c!=')')

  if ( KGetField( buf, ":sender", &sender, 0 ) ) {
#if (DEBUG > 0)
      fprintf( stderr, "Error: No sender field - message not sent.\n" );
#endif
      return( -1 );
  }

  sprintf(foo=buf2, "(\"%s\" \"%s\" ", sender, info->agentname);
    
  while (*foo) foo++;
  do {
    if (*buf=='\\') {bs^=1;CANDG}
    if (bs) CANDG
      if (*buf=='"') {quote^=1;CANDG}
    if (quote) CANDG
      if(*buf=='(') {paren++;CANDG}
      else if(*buf==')') paren--;
    if (!inquote)
      {
	if(*buf=='\'')
	  {
	    sprintf(foo,"(quote ");
	    iqparen=paren;
	    foo+=7;
	    inquote=1;
	    if(NORMAL(*(buf+1)))
	      nextnormal=1;
	  }
	else
	  *foo++ = *buf;
      }
    else
      {
	if(!nextnormal) {
	  if((!quote)&&(iqparen==paren))
	    {
	      inquote=0;
	      *foo++=')';
	    }
	}
	else
          {
	    if(!NORMAL(*buf))
	      {
	        inquote=0;
	        *foo++=')';
	        nextnormal=0;
	      }
	  }
	*foo++ = *buf;
      }
  } while(*++buf);

  if(inquote) *foo++=')';
  *foo++ = ')'; /* the close-paren of the triple */
  *foo='\0';

  if (purl->state == CLOSED_CON) {
    /* connect to the appropriate mbus and register interest in
       your messages */
    newpurl = (ParsedURL *) calloc(1,sizeof(ParsedURL));
    newinfo = (MBUSInfo *) calloc(1,sizeof(MBUSInfo));

    sprintf(tmpbuf, "(accept \".*\" %s)", sender);
    if ( sender != NULL ) {
	free( sender );
    }

    if ((info->mbconn =
	 MBConnect(info->host,
		   info->port)) &&
	MBtransmit_Cstring(info->mbconn, tmpbuf)
	== strlen(tmpbuf)) {
      purl->state = SHARED_CON;
      purl->fd = MBconnection_fd(info->mbconn);


      newpurl->transtype = purl->transtype;
      newpurl->state = SHARED_CON;
      newpurl->fd = purl->fd;
      newinfo->mbconn = info->mbconn;
      strcpy(newinfo->host, info->host);
      newinfo->port = info->port;
      newpurl->transinfo = newinfo;

      sprintf(tmpbuf, "MBCONN%d", ++mbusconns);
      strcpy(newpurl->name, tmpbuf);

      entry = CreateHashEntry(&OpenConns, tmpbuf, &flg);
      SetHashValue(entry, newpurl);

    } else {
      purl->state = CONNECTION_ERROR;
      return(0);
    }
  }
#if (DEBUG > 0)
  fprintf(stderr, "Sending on MBUS: %s\n", buf2);
#endif
  return(MBtransmit_Cstring(info->mbconn, buf2));
}


#endif

    
