// PL sensor,
// 1-11-2000, jarl van katwijk, started,
#include "messaging.h"
#include "plugin_sensor_PL.h"
#include "gmodule.h"
#include <stdio.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>

// Corba includes :
#include "plugin_sensor_PL_main.h"

// Globals
static engine *Engine = NULL;
static glong Node;
static glong Data;
static pthread_t BBLclient_ORB_tree;
static gint BBLclient_ORB_thread;
static piper::BL2PL::PL_var bl2plref;
static CORBA::ORB_var BL2PL_corba_orb = NULL;
static gboolean BL2DL_Connected = FALSE;

/*
 * Code
 */

gboolean sensor_PL_procedure() {
	if (BL2DL_Connected) {
		bl2plref->upload(Engine->secure_data_get_rawdata(Engine->secure_node_get_data(Node,Engine->ENGINE_DL_ID,Engine->ENGINE_DL_PASSWORD->str),Engine->ENGINE_DL_ID,Engine->ENGINE_DL_PASSWORD->str));
		bl2plref->execute();
	}
    Engine->secure_node_set_datadescription(Node,(gchar*)sensor_PL_description(),Engine->ENGINE_DL_ID,Engine->ENGINE_DL_PASSWORD->str); /* Set data description */
	/* reset procedure_running */
    Engine->secure_node_set_procedure_running(Node,FALSE,Engine->ENGINE_DL_ID,Engine->ENGINE_DL_PASSWORD->str);
    return TRUE;
}

gboolean sensor_PL_cleanup() {
    /* cancel PL execution */
	if (BL2DL_Connected) {
		bl2plref->cancel();
	}

    /* give a kill signal to the PL */
    bl2plref->shutdown();

    #ifdef HAVE_PTHREAD_CANCEL
    if (BL2DL_Connected) {
        pthread_cancel(BBLclient_ORB_thread);
    }
    #endif

    /* reset procedure_running */
    Engine->secure_node_set_procedure_running(Node,FALSE,Engine->ENGINE_DL_ID,Engine->ENGINE_DL_PASSWORD->str);

    return TRUE;
}

gboolean sensor_PL_activate() {
	gint fd;
	gint filelen;
	guint bytesread;
	gint loaderror;
	GIOChannel *file;
	struct stat filestat;
	gchar *xmldocument[65000];

	// locate source
	// TODO: replace this by a buf messaging_mobject_data_object_data_reconstruct
	fd = open("/usr/local/share/overflow/examples/hello.n",O_RDONLY);
    if (fd == -1) {
        g_warning("Error %i during file open\n",errno);
        return FALSE;
    }
	// read XML source
	file = g_io_channel_unix_new(fd);
	fstat(fd,&filestat);
	filelen = filestat.st_size;
	//xmldocument = (gchar*)g_malloc(filelen+1);
	//memset(xmldocument,0,filelen+1);
	loaderror = g_io_channel_read(file,(gchar*)&xmldocument,filelen,&bytesread);
	close(fd);
	if (BL2DL_Connected) {
		bl2plref->ping();
		bl2plref->upload((char*)xmldocument);
		bl2plref->execute(); /* testing */
	}
	return TRUE;
}

void* plugin_sensor_PL_runBBLclientORB(gpointer data) {
	CORBA::Object_var BL2PL_corba_object;
    gint dummy_argc = 1;
    gchar *dummy_argv[1];
    FILE *ifp;
    gchar *ior;
    gchar filebuffer[1024];
    GString *iorFile;
    /* build dummy args */
    dummy_argc = 1;
    dummy_argv[0] = (gchar*)"BBLplugin";

    // Read IOR file.
    //TODO: execute BBL if BBL aint ready...
    /* define IOR file location */
    iorFile = g_string_new(getenv("HOME"));
    iorFile = g_string_append(iorFile, "/piper_info");
    mkdir(iorFile->str,S_IRWXU);
    iorFile = g_string_append(iorFile, "/ior");
    mkdir(iorFile->str,S_IRWXU);
    iorFile = g_string_append(iorFile, "/bl2pl.ior");

    /* read ior file */
    ifp = fopen(iorFile->str,"r");
    if( ifp == NULL ) {
        g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_WARNING,"PL IOR file %s not found. ",iorFile->str);
        g_string_free(iorFile,TRUE);
        return 0;
    }
    fgets(filebuffer,1024,ifp);
    ior = g_strdup(filebuffer);
    //g_print("IOR = %s\n",ior);
    fclose(ifp);

    try {
        BL2PL_corba_orb = CORBA::ORB_init(dummy_argc, dummy_argv, "omniORB3");
        BL2PL_corba_object = BL2PL_corba_orb->string_to_object(ior);
        bl2plref = piper::BL2PL::PL::_narrow(BL2PL_corba_object);
        if( CORBA::is_nil(bl2plref) ) {
            cerr << "Can't narrow reference to type BL2PL (or it was nil).";
            return 0;
        }
		BL2DL_Connected = TRUE;
	    BL2PL_corba_orb->run();
	    BL2PL_corba_orb->destroy();
    }
    catch(CORBA::COMM_FAILURE& ex) {
        cerr << "Caught system exception COMM_FAILURE -- unable to contact the object ";
        return 0;
    }
    catch(CORBA::SystemException&) {
        cerr << "Caught a CORBA::SystemException in PL sensor plugin.";
        return 0;
    }
    catch(CORBA::Exception&) {
        cerr << "Caught CORBA::Exception.";
        return 0;
    }
    catch(omniORB::fatalException& fe) {
        cerr << "Caught omniORB::fatalException:";
        cerr << "  file: " << fe.file() << endl;
        cerr << "  line: " << fe.line() << endl;
        cerr << "  mesg: " << fe.errmsg() << endl;
        return 0;
    }
    catch(...) {
        cerr << "Caught unknown exception.";
        return 0;
    }
    if (!bl2plref) {
        g_message("DFP cannot bind to %s", ior);
        return 0;
    }
    return 0;
}

gboolean sensor_PL_init(glong NodeID,engine *_Engine) {
    Engine = _Engine;
    Node = NodeID;
    Data = Engine->secure_node_get_data(Node,Engine->ENGINE_DL_ID,Engine->ENGINE_DL_PASSWORD->str);

	BBLclient_ORB_thread = pthread_create(&BBLclient_ORB_tree,NULL,plugin_sensor_PL_runBBLclientORB,NULL);
	return TRUE;
}

#define PLUGIN_PL_NAME_SENSOR "PL plugin sensor"
#define PLUGIN_PL_NAME_COLLECTOR "PL plugin collector"
#define PLUGIN_PL_DESCRIPTION "Collects and handles PL XML streams"
#define PLUGIN_PL_SENSOR_DESCRIPTION "PL plugin sensor"
#define PLUGIN_PL_COLLECTOR_DESCRIPTION "PL plugin collector"

const gchar* sensor_PL_name() {
    return "PL plugin sensor";
}

const gchar* sensor_PL_description() {
    return "Collects and handles PL streams";
}

nodeType_t sensor_PL_nodetype() {
    return (nodeType_t)SensorNode;
}

