// PL sensor main body, or BBL (BabyBl),
// 5-11-2000, jarl van katwijk, started,
//
#include "messaging_config.h"
#include "messaging.h"
// various others :
#include "gmodule.h"
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <tree.h>
#include <signal.h>
// Overflow includes : 
#include "ParameterSet.h"
#include "ObjectRef.h"
#include "UIDocument.h"
#include <string>
#include "path.h"
// Corba includes :
#include "plugin_sensor_PL_main.h"

/*
  Globals
 */
UIDocument *doc;
pthread_t PLtree;
gint PLthread;
CORBA::ORB_var orb;

/*
 * Prototypes
 */
void plugin_sensor_destruct(int sig);

/*
 * Code
 */
// Various :
gpointer plugin_sensor_PL_main_activate_threaded(gpointer data)
{
	ParameterSet param;
	doc->run(param);
	g_print("\n");
	return 0;
}

// Corba implementations :
CORBA::Boolean BL2PL_i::ping(void)
{
	g_print("pong!\n");
	return TRUE;
}

void BL2PL_i::shutdown(void)
{
	plugin_sensor_destruct(0);
        return;
}

piper::BL2PL::stringseq* BL2PL_i::availablenodes(void)
{
	vector<string> allnodes;
	piper::BL2PL::stringseq_var nodes = new piper::BL2PL::stringseq();
	guint i;

	allnodes = doc->getAvailableNodes(); /* get all nodes */
	nodes->length(allnodes.size()); /* set size of returned sequence */
	for (i=0;i<allnodes.size();i++) {
		nodes[i] = CORBA::string_dup(allnodes[i].c_str());
	}
	return nodes._retn();
}

char* BL2PL_i::upload(const char *xmldocument)
{
    //char* result;
	gint CONTAINER;
    CONTAINER = 0;

	if ((xmldocument==NULL)||(strlen(xmldocument)==0)) {
		g_print("PL ERROR: invalid XML data stream received.\n");
		return CORBA::string_dup("ERROR");
	} else {
    }
    try {
        scanDL();
    }
    catch (BaseException *e) {
        e->print();
        delete e;
        exit(1);
    }
    UIDocument::loadAllInfo();
    doc = new UIDocument("/dev/zero");
	doc->loadFromMemory((char*)xmldocument,strlen(xmldocument));
    //doc->load();

    //TODO: let this make more sense :)
    return CORBA::string_dup("Done");
}

char* BL2PL_i::update(const char *xmldocument)
{
	char* result = "";
    return result;
}

CORBA::Boolean BL2PL_i::execute(void)
{
    CORBA::Boolean result = FALSE;
    ParameterSet param;
    doc->run(param);
    g_print("\n");
	//PLthread = pthread_create(&PLtree,NULL,plugin_sensor_PL_main_activate_threaded,NULL);
    return result;
}
 
CORBA::Boolean BL2PL_i::cancel(void)
{
	CORBA::Boolean result;
        result = FALSE;
#ifdef HAVE_PTHREAD_CANCEL
	if (pthread_cancel(PLthread)==0) {
		result = TRUE;
	} else {
		result = FALSE;
	}
#endif
	return result;
}

// Corba setup :
void plugin_sensor_PL_main_start_Corba_server(int dummy_argc,char** dummy_argv)
{
	GString* iorFile;
	try {
		// start up the ORB and POA
		orb = CORBA::ORB_init(dummy_argc, dummy_argv, "omniORB3");
		CORBA::Object_var obj = orb->resolve_initial_references("RootPOA");
		PortableServer::POA_var poa = PortableServer::POA::_narrow(obj);

		// get our object to serve out and activate it
		BL2PL_i* mainConn = new BL2PL_i();
		PortableServer::ObjectId_var mainConnId = poa->activate_object(mainConn);
		// get an object reference and write the IOR to a file
		CORBA::Object_var objRef = mainConn->_this();
		CORBA::String_var stringIor(orb->object_to_string(objRef));
		/* define IOR file location */
		iorFile = g_string_new(getenv("HOME"));
		iorFile = g_string_append(iorFile, "/piper_info");
		mkdir(iorFile->str,S_IRWXU);
		iorFile = g_string_append(iorFile, "/ior");
		mkdir(iorFile->str,S_IRWXU);
		// TODO: make possible to have unlimited nr. of ior so multiple PL's can run at once
		iorFile = g_string_append(iorFile, "/bl2pl.ior");
		ofstream outStream;
		outStream.open(iorFile->str);
		outStream << stringIor;
		outStream.close();
		g_print("wrote IOR file %s\n",iorFile->str);
		// only the POA holds a reference to the connection interface now
		mainConn->_remove_ref();
		// Activate the POA manager and run the orb
		PortableServer::POAManager_var pMan = poa->the_POAManager();
		pMan->activate();
		orb->run();
		orb->destroy();
	}
	catch(CORBA::SystemException&) {
		cerr << "Caught a CORBA::SystemException in the PL." << endl;
	}
	catch(CORBA::Exception&) {
		cerr << "Caught a CORBA::Exception." << endl;
	}
	catch(omniORB::fatalException& fe) {
		cerr << "Caught an omniORB::fatalExecption:" << endl;
		cerr << "  file: " << fe.file() << endl;
		cerr << "  line: " << fe.line() << endl;
		cerr << "  mseg: " << fe.errmsg() << endl;
	}
	catch(...) {
		cerr << "Caught an unknown exception." << endl;
	}
	return;
}
	
void plugin_sensor_destruct(int sig)
{
    GString *pidfilename;
    gchar buf[32];
    gint pid;
    g_print("shutting down PL...\n");
    /* reset signal handlers */
    signal(SIGTERM,SIG_DFL);
    signal(SIGSEGV,SIG_DFL);
    signal(SIGINT,SIG_DFL);
    /* remove pid file */

    pidfilename = g_string_new(getenv("HOME"));
    pid = getpid();
    snprintf(buf,32,"%d",pid);
    g_string_append(pidfilename,"/piper_info/pids/PL.pid.");
    g_string_append(pidfilename,buf);
    unlink(pidfilename->str);
    g_string_free(pidfilename,TRUE);
    orb->shutdown(FALSE);
}

int main(int argc,char** argv)
{
    gint pid;
    gint pidfd;
    GString *pidfilename;
    gchar buf[32];
    /* display version */
    g_log("PL",G_LOG_LEVEL_MESSAGE,"Piper PL Overflow 0.1");
    /* bind some signals */
    signal(SIGTERM,plugin_sensor_destruct);
    //disabled while being unstable; signal(SIGSEGV,plugin_sensor_destruct);
    signal(SIGINT,plugin_sensor_destruct);
    /* write pid file to ~/piper_info/pids/PL.pid */
    pidfilename = g_string_new(getenv("HOME"));
    g_string_append(pidfilename,"/piper_info");
    mkdir(pidfilename->str,S_IRWXU);
    g_string_append(pidfilename,"/pids");
    mkdir(pidfilename->str,S_IRWXU);
    g_string_free(pidfilename,TRUE);

    pid = getpid();
    snprintf(buf,32,"%d",pid);
    pidfilename = g_string_new(getenv("HOME"));
    g_string_append(pidfilename,"/piper_info/pids/PL.pid.");
    g_string_append(pidfilename,buf);
    pidfd = open(pidfilename->str,O_RDWR | O_CREAT,S_IRWXU);
    if (pidfd == -1)
    {
        g_warning("Error %i during CREATE|APPEND of file %s\n",errno,pidfilename->str);
        return FALSE;
    }
    snprintf(buf,32,"%d\n",pid);
    write(pidfd,buf,strlen(buf));
    close(pidfd);
    g_string_free(pidfilename,TRUE);

    /* boot PL layer */
    plugin_sensor_PL_main_start_Corba_server(argc,argv);
    g_print("terminating PL...\n");
    return 0;
}



