/* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *   guname: System information dialog. 
 *
 *   Copyright (C) 1998 Havoc Pennington <hp@pobox.com> except marquee code.
 *   Expanded by Jarl van  Katwijk
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */
#include <stdio.h>
#include "messaging.h"
#include "plugin_sensor_localhost.h"

#include "gmodule.h"
#include <unistd.h>
#include <string.h> /* memset */
#include <sys/utsname.h>
#include <errno.h>
#include <sys/stat.h>

#define DEBIAN_STRING "Debian GNU/Linux"
#define REDHAT_STRING "Red Hat Linux"
#define SUSE_STRING "Suse Linux"
#define STAMPEDE_STRING "Stampede Linux"
#define MANDRAKE_STRING "Mandrake Linux"
#define SLACKWARE_STRING "Slackware linux"
#define PLUGIN_LOCALHOST_NAME "Localhost plugin"


/************************************
  Globals
  ***********************************/
static const gchar * info[end_system_info];
static guint sensor_timer;
//const gchar * descriptions[];
static glong Node; /* this is passed by the engine::processor to the plugin */
static glong Data;
static engine *Engine = NULL; /* pointer to active processing engine */


/*
 * Prototypes
 */
static gpointer get_portable_info();
static gpointer get_linux_info();

/*
 * Code
 */

gboolean g_file_exists (const gchar *filename) {
    struct stat s;
    g_return_val_if_fail (filename != NULL,FALSE);
    return stat (filename, &s) == 0;
}

gint load_system_info(gpointer data) {
	gboolean first_time = TRUE;

	if (first_time) {
	    /* Set all the pointers in the array to NULL */
		memset(&info, '\0', sizeof(info));
		first_time = FALSE;
	} else { /* Clear out the array. */
		gint i = 0;
		while ( i < end_system_info ) {
			if (info[i] != NULL) g_free((gpointer)info[i]);
			info[i] = NULL;
			++i;
		}
	}
	get_portable_info();
	get_linux_info();

	/* copy message line into sensor MO */
	//REPLACED, no longer a gui by default, need to ; offset = (glong)localhost_sensor_mo->MDO->rawdata;
	//REPLACED; strcpy((char*)offset,(char*)PLUGIN_DESCRIPTION);
	//REPLACED; offset += strlen(PLUGIN_DESCRIPTION)+1;
	//REPLACED; offset_old = offset;n = 1;
	//REPLACED; for (i=0;i<(end_system_info);i++) {
	//REPLACED; 	if (!(info[i]==NULL)) {
	//REPLACED; 		sprintf((char*)offset,info[i]);
	//REPLACED; 		offset += strlen(info[i])+1;
	//REPLACED; 	} else {
	//REPLACED; 		sprintf((char*)offset,"-");
	//REPLACED; 		offset += strlen("-")+1;
	//REPLACED; 	}
	//REPLACED; }
	//REPLACED; strcpy((char*)offset,"{tail}");

    /* TODO: New message copy code */
    Data = Engine->secure_node_get_data(Node,Engine->ENGINE_DL_ID,Engine->ENGINE_DL_PASSWORD->str);
    Engine->secure_data_clean_rawdata(Data,Engine->ENGINE_DL_ID,Engine->ENGINE_DL_PASSWORD->str);
    Engine->secure_data_set_datadescription(Data,(gchar*)sensor_localhost_description(),Engine->ENGINE_DL_ID,Engine->ENGINE_DL_PASSWORD->str);



	/* Put the data stream into the engine
     * -----------------------------------
     * The engine will accept the message and search for collector objects that accept the stream
     */
    Engine->pipeline_in(Node, Engine->ENGINE_DL_ID, Engine->ENGINE_DL_PASSWORD->str);


	/* parse runtime config if timeout>0 */
    GString *config = Engine->secure_node_get_configruntime(Node,Engine->ENGINE_DL_ID,Engine->ENGINE_DL_PASSWORD->str);
    if (config != NULL) {
    	if (atoi((char*)config->str)>0) {
	    	sensor_timer = gtk_timeout_add(atoi((char*)config->str)*1000,load_system_info,data);
        }
    } else {
        printf("sensor_localhost: configruntime == NULL?!?   (NodeID=%li)\n",Node);
    }

	return FALSE;
}

static gpointer get_portable_info() {
	struct utsname buf;

	if ( uname(&buf) == -1 )
		g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_WARNING,"uname() failed (%i)\n",errno);
	info[si_OS] = g_strdup(buf.sysname);
	info[si_release] = g_strdup(buf.release);
	info[si_OS_version] = g_strdup(buf.version);
	info[si_CPU_type] = g_strdup(buf.machine);
	info[si_host] = g_strdup(buf.nodename);
	/* info[si_domain] = g_strdup(buf.domainname); */ /* man page wrong? */
	info[si_user] = g_strdup(getlogin());
	info[si_display] = g_strdup(getenv("DISPLAY")); /* hmm */
	//temp disabled because fork() stalls; get_uptime();
	/* Add more later */
	return NULL;
}

static gpointer get_linux_info() {
	/* Identify distribution (really this could be compiled in) */
	if (g_file_exists("/etc/debian_version")) {
		FILE * f;
		gchar buf[10];
		info[si_distribution] = g_strdup(DEBIAN_STRING);
		f = fopen("/etc/debian_version", "r");
		if (f) {
			fscanf(f, "%8s", buf);
			info[si_distribution_version] = g_strdup(buf);
			fclose(f);
		}
	} else if (g_file_exists("/etc/redhat-release")) {
		FILE *f;
		gchar buf[80];
		info[si_distribution] = g_strdup(REDHAT_STRING);
		f = fopen("/etc/redhat-release", "r");
		if (f) {
			fgets(buf, 79, f);
			if (strchr(buf, ' ')) {
				info[si_distribution_version] = g_strdup(strchr(buf, ' ')+1);
			} else {
				info[si_distribution_version] = g_strdup(buf);
			}
		fclose(f);
		}
	} else if (g_file_exists("/etc/slackware-version")) {
		FILE *f;
		gchar buf[80];
		info[si_distribution] = g_strdup(SLACKWARE_STRING);
		f = fopen("/etc/slackware-version", "r");
		if (f) {
			fgets(buf, 79, f);
			if (strchr(buf, ' ')) {
				info[si_distribution_version] = g_strdup(strchr(buf, ' ')+1);
			} else {
				info[si_distribution_version] = g_strdup(buf);
			}
			fclose(f);
		}
	} else {
		/* This is about the only case where we'll put Unknown instead
		 of simply not showing the info. */
		info[si_distribution_version] = g_strdup(("Unknown"));
	}
	/* More to come. */
	return NULL;
}

gboolean sensor_localhost_procedure() {
	/* dummy, nothing to do */
    /* reset procedure_running */
    Engine->secure_node_set_procedure_running(Node,FALSE,Engine->ENGINE_DL_ID,Engine->ENGINE_DL_PASSWORD->str);
    return TRUE;
}

gboolean sensor_localhost_cleanup() {
    /* dummy, nothing to do */
    /* reset procedure_running */
    gtk_timeout_remove(sensor_timer);
    Engine->secure_node_set_procedure_running(Node,FALSE,Engine->ENGINE_DL_ID,Engine->ENGINE_DL_PASSWORD->str);
    return TRUE;
}

gboolean sensor_localhost_activate() {
    GString *config = g_string_new(Engine->secure_node_get_configruntime(Node,Engine->ENGINE_DL_ID,Engine->ENGINE_DL_PASSWORD->str)->str);
	sensor_timer = gtk_timeout_add(atoi((char*)config->str)*1000,load_system_info,NULL);
	return TRUE;
}

gboolean sensor_localhost_init(glong NodeID,engine* _Engine) {
    //PLUGIN_LOCALHOST_SENSOR_DESCRIPTION "localhost plugin sensor"
    //PLUGIN_LOCALHOST_COLLECTOR_DESCRIPTION "localhost plugin collector"

    /* publish engine */
    Engine = _Engine;
    /* publish local node ID */
    Node = NodeID;
    /* Publish local Data ID */
    Data = Engine->secure_node_get_data(Node,Engine->ENGINE_DL_ID,Engine->ENGINE_DL_PASSWORD->str);
    /* default configure the localhost stream to an interval of 60 seconds */
    Engine->secure_node_set_configruntime(Node,(gchar*)"10",Engine->ENGINE_DL_ID,Engine->ENGINE_DL_PASSWORD->str);
	return TRUE;
}

const gchar* sensor_localhost_name() {
    return "Localhost plugin sensor";
}

const gchar* sensor_localhost_description() {
    return "Collects and handles localhost information";
}

nodeType_t sensor_localhost_nodetype() {
    return (nodeType_t)SensorNode;
}

