/*
 * syslogd sensor, bastardAI
 * - lots of code taken out the GNU syslogd sources.
 * - all messaging db related
 */
#include "messaging.h"
#include "plugin_sensor_syslogd.h"

static glong Node;
static glong Data;
static engine* Engine = NULL;

        /* Variables */
        static gint     finet;           /* Internet datagram socket */
        static gint     LogPort;         /* port number for INET connections */
        static gint     inetm,funix;
        static gchar    *LocalDomain;    /* our local domain name */
        static gchar    *previous_syslogd_message; /* */
        static gchar    **StripDomains;  /* these domains may be stripped before writing logs */
        static gint     InetInuse;       /* non-zero if INET sockets are being used */
        static gchar    **LocalHosts;    /* these hosts are logged with their hostname */
        static gint     AcceptRemote;    /* receive messages that come via UDP */
        static gchar    LocalHostName[MAXHOSTNAMELEN+1]; /* our hostname */
        static gint     syslogd_gint_tag;


/*
 * Class implementations
 */
gboolean sensor_syslogd_procedure() {
	/* nothing to do */
    /* reset procedure_running */
    Engine->secure_node_set_procedure_running(Node,FALSE,Engine->ENGINE_DL_ID,Engine->ENGINE_DL_PASSWORD->str);
    return TRUE;
}

gboolean sensor_syslogd_cleanup() {
    /* clean local activity & reset procedure_running */
    gdk_input_remove(syslogd_gint_tag);
    shutdown(finet,2);
    close(finet);

    Engine->secure_node_set_procedure_running(Node,FALSE,Engine->ENGINE_DL_ID,Engine->ENGINE_DL_PASSWORD->str);
    return TRUE;
}

gboolean sensor_syslogd_activate() {
	/* already activated? if so, return */
	// if (activated = TRUE)
	//	return FALSE;
	/* attach the UDP syslog stream to GLIB 
	 * now all UDP 514 traffic will be handled by handle_syslogd_socket that
	 * will copy the data into the system by filling syslog_sensor_mo with the
	 * data and calling messaging_db_pipeline_in(syslog_sensor_mo). */
	syslogd_gint_tag = gdk_input_add(syslogd_gint_tag,GDK_INPUT_READ,(GdkInputFunction)handle_syslogd_socket,NULL);
	/* set flag */
	// activated = TRUE;
	return TRUE;
}

gboolean sensor_syslogd_init(glong NodeID,engine *_Engine) {
	/* 
	 * here the plugin registers it's stream(s).
	 * 2 groups are setup here: input and output streams.
	 * each group is made up of 6 streams: chat, file, 
	 * 	URL, high, normal and low, they can be NULL.
	 * Most common stream is the SCROLLING OUTPUT, this 
	 * 	will generate a dedicated tab in the stream window
	 * Streams can be directed to other messaging deamons.
	 */
    Engine = _Engine;
    Node = NodeID;
    Data = Engine->secure_node_get_data(Node,Engine->ENGINE_DL_ID,Engine->ENGINE_DL_PASSWORD->str);
	/* setup the syslogd stream */
	syslogd_gint_tag = _sensor_syslogd_init(NULL);
	return TRUE;
}

const gchar *cvthname(struct sockaddr_in *f,gpointer dummy) {
    /* Return a printable representation of a host address. */
	struct hostent *hp;
	register gchar *p;
	gint count;

	if (f->sin_family != AF_INET) {
		g_print("Malformed from address.\n");
		return ("???");
	}
	hp = gethostbyaddr((gchar *) &f->sin_addr, sizeof(struct in_addr),f->sin_family);
	if (hp == 0) {
		g_print("Host name for your address unknown.\n");
		return (inet_ntoa(f->sin_addr));
	}
	/* Convert to lower case, just like LocalDomain above */
	for (p = (gchar *)hp->h_name; *p ; p++)
		if (isupper(*p))
			*p = tolower(*p);
	/*
	 * Notice that the string still contains the fqdn, but your
	 * hostname and domain are separated by a '\0'.
	 */
	if ((p = index(hp->h_name, '.'))) {
		if (strcmp(p + 1, LocalDomain) == 0) {
			*p = '\0';
			return (hp->h_name);
		} else {
			if (StripDomains) {
				count=0;
				while (StripDomains[count]) {
					if (strcmp(p + 1, StripDomains[count]) == 0) {
						*p = '\0';
						return (hp->h_name);
					}
				count++;
				}
			}
			if (LocalHosts) {
				count=0;
				while (LocalHosts[count]) {
					if (!strcmp(hp->h_name, LocalHosts[count])) {
						*p = '\0';
						return (hp->h_name);
					}
					count++;
				}
			}
		}
	}
	return (hp->h_name);
}

gpointer handle_syslogd_socket(gpointer data,gint source,GdkInputCondition condition) {
	fd_set unixm, readfds;
	register gint i;
	struct sockaddr_in frominet;
	gchar *from,line[MAXLINE +1],*p,*q,*number,*t;
	guint len;
	gint c,pri;
	GString *string;
	gint compare = TRUE;
	/* setup data block */
	string = g_string_new("");
	inetm = source;
	FD_ZERO(&unixm);
	FD_ZERO(&readfds);
	/* Add the Internet Domain Socket to the list of read
	 * descriptors. */
	if (InetInuse && AcceptRemote)
		FD_SET(inetm, &readfds);
	if (InetInuse && AcceptRemote && FD_ISSET(inetm, &readfds)) {
		len = sizeof(frominet);
		memset(line, '\0', sizeof(line));
		i = recvfrom(finet, line, MAXLINE - 2, 0, (struct sockaddr *) &frominet, &len);
		if (i > 0) {
			line[i] = line[i+1] = '\0';
			from = (gchar *)cvthname(&frominet,NULL);
			g_string_append(string,from);
			/* test for special codes */
			pri = DEFUPRI;
			p = line;
			t = number;
			if ( strcmp(previous_syslogd_message,p) == 0 )
				compare = FALSE;
			previous_syslogd_message = (gchar *) g_malloc(strlen(p)+1);
			(void)strcpy(previous_syslogd_message,p);
			if (*p == '<') {
				pri = 0;
				while (*++p != '>')
					pri = 10 * pri + (*p - '0');
				if (*p == '>')
					++p;
			}
			if (pri &~ (LOG_FACMASK|LOG_PRIMASK))
				pri = DEFUPRI;
			q = line;
			while ((c = *p++ & 0177) != '\0' && q < &line[sizeof(line) - 1])
				if (c == '\n')
					*q++ = ' ';
				else if (iscntrl(c)) {
					*q++ = '^';
					*q++ = c ^ 0100;
				} else
					*q++ = c;
			*q = '\0';
			g_string_append(string," ");
			g_string_append(string,line);
		} else if (i < 0 && errno != EINTR) {
				g_print("recvfrom inet\n");
				sleep(10);
		}
	}
	/* copy message line into sensor MO */
    glong Noderef = Node;
    glong Dataref = Engine->secure_node_get_data(Noderef,Engine->ENGINE_DL_ID,Engine->ENGINE_DL_PASSWORD->str);
    glong datalen = Engine->secure_data_get_rawlen(Dataref,Engine->ENGINE_DL_ID,Engine->ENGINE_DL_PASSWORD->str);
	if (string->len < datalen)
		len = string->len;
	else
		len = datalen;
	Engine->secure_data_set_rawdata(Dataref,string->str,len,Engine->ENGINE_DL_ID,Engine->ENGINE_DL_PASSWORD->str);
	/* poll the db , it will accept the message and search for collector
	 * objects that accept the data. */
    Engine->pipeline_in(Noderef,Engine->ENGINE_DL_ID,Engine->ENGINE_DL_PASSWORD->str);

	g_string_free(string,TRUE);
	return NULL;
}

gint _sensor_syslogd_init(gpointer user_data) {
	struct sockaddr_in sin;
	register gchar *p;
	struct hostent *hent;
	gint bind_result;

    inetm = 0;
    funix = 0;
    previous_syslogd_message = (gchar*)("empty");
    StripDomains = NULL;
    InetInuse = 0;
    AcceptRemote = 1;

    finet = socket(AF_INET, SOCK_DGRAM, 0);
    if (finet >= 0) {
		auto gint on = 1;
        struct servent *sp;
        sp = getservbyname("syslog", "udp");
        if (sp == NULL) {
        	errno = 0;
            g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_WARNING,"network logging disabled (syslog/udp service unknown).");
            g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_WARNING,"see syslogd(8) for details of whether and how to enable it.");
		} else {
			sin.sin_family = AF_INET;
            sin.sin_port = LogPort = sp->s_port;
            sin.sin_addr.s_addr = 0;
            if ( setsockopt(finet,SOL_SOCKET,SO_REUSEADDR,(gchar *) &on,sizeof(on))) {
				g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_WARNING,"setsockopt, suspending inet");
			} else {
                bind_result = bind(finet, (struct sockaddr *) &sin, sizeof(sin));
				if (bind_result<0) 
					g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_WARNING,"bind, suspending inet");
				else {
					inetm = finet;
                    InetInuse = 1;
                }
			}
		}
	} else { g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_WARNING,"syslog: Unknown protocol, suspending inet service.");}
    /* Build 'LocalDomain' */
    (gpointer) gethostname(LocalHostName, sizeof(LocalHostName));
	if ( (p = index(LocalHostName, '.')) ) {
		*p++ = '\0';
		LocalDomain = p;
	} else {
        LocalDomain = (gchar*)"";
        hent = gethostbyname(LocalHostName);
        sprintf(LocalHostName, "%s", hent->h_name);
        if ( (p = index(LocalHostName, '.')) ) {
	         *p++ = '\0';
             LocalDomain = p;
         }
	}
    /* Convert to lower case to recognize the correct domain laterly */
    for (p = (char *)LocalDomain; *p ; p++)
   		if (isupper(*p)) {
			*p = tolower(*p);
		}
	return(finet);
}

/*
 * DEFINES...
 *
 */
#define PLUGIN_SYSLOG_NAME "Syslog daemon"
#define PLUGIN_SYSLOG_NAME_SENSOR "Syslogd plugin sensor"
#define PLUGIN_SYSLOG_NAME_COLLECTOR "Syslogd plugin collector"
#define PLUGIN_SYSLOG_DESCRIPTION "sensor"
#define PLUGIN_SYSLOG_SENSOR_DESCRIPTION "Syslogd plugin sensor"
#define PLUGIN_SYSLOG_COLLECTOR_DESCRIPTION "Syslogd plugin collector"

const gchar* sensor_syslogd_name() {
    return "Syslogd plugin sensor";
}

const gchar* sensor_syslogd_description() {
    return "Syslogd plugin sensor handles syslogd connects";
}

nodeType_t sensor_syslogd_nodetype() {
    return (nodeType_t)SensorNode;
}

