#include "messaging_plugin.h"

#ifndef SENSOR_SYSLOGD
#define SENSOR_SYSLOGD

#include <unistd.h>
#include <gmodule.h>
#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <utmp.h>
#include <ctype.h>
#include <string.h>
#include <setjmp.h>
#include <stdarg.h>
#include <sys/syslog.h>
#include <sys/errno.h>
#include <sys/ioctl.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <sys/socket.h>
#include <sys/file.h>
#include <fcntl.h>
#include <sys/uio.h>
#include <sys/un.h>
#include <sys/time.h>
#include <signal.h>
#include <netinet/in.h>
#include <netdb.h>
#include <sys/syscall.h>
#include <arpa/nameser.h>
#include <arpa/inet.h>
#include <resolv.h>

/*
 * DEFINES...
 *
 */
#define MAXLINE         1024    /* maximum line length */
#define F_CONSOLE       3       /* console terminal */
#define UNAMESZ         8       /* length of a login name */
#define MAXUNAMES       20      /* maximum number of user names */
#define MAXFNAME        200     /* max file pathname length */
#define MAXSVLINE       240     /* maximum saved line length */
#define DEFUPRI         (LOG_USER|LOG_NOTICE)
#define DEFSPRI         (LOG_KERN|LOG_CRIT)
#define IGN_CONS        0x001   /* don't print on console */
#define SYNC_FILE       0x002   /* do fsync on file after printing */
#define ADDDATE         0x004   /* add a date to the message */
#define MARK            0x008   /* this message is a mark */
#define WIDGET_HEIGHT   48

/* proto's */
gboolean sensor_syslogd_procedure();
gboolean sensor_syslogd_cleanup();
gboolean sensor_syslogd_activate();
gboolean sensor_syslogd_init(glong Node,engine *Engine);
const gchar* sensor_syslogd_name();
const gchar* sensor_syslogd_description();
nodeType_t sensor_syslogd_nodetype();


gint _sensor_syslogd_init(gpointer user_data);
gpointer format_syslogd_line (gchar *msg);
gpointer handle_syslogd_socket(gpointer data,gint source,GdkInputCondition condition);

/* - Auto plugin registration code -------------------------------*/

extern "C" {
    plugin_maker* maker_sensor_syslogd(){
        plugin_maker *plug = (plugin_maker*)g_malloc(sizeof(plugin_maker));
        plug->procedure = sensor_syslogd_procedure;
        plug->cleanup   = sensor_syslogd_cleanup;
        plug->activate  = sensor_syslogd_activate;
        (gpointer)plug->init      = sensor_syslogd_init;
        plug->name      = sensor_syslogd_name;
        plug->description = sensor_syslogd_description;
        plug->nodetype = sensor_syslogd_nodetype;

        return plug;
    }

    class proxy {
        public:
            proxy() {
                /* fill in plugin info */
                BL_plugins["sensor_syslogd"] = maker_sensor_syslogd;
                /* register this plugin to plugins list */
                g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_DEBUG,"Registered sensor_syslogd");
            }
    };

    // our one instance of the proxy
    proxy p;
}

// This structure represents the files that will have log
// copies printed.
struct filed {
#ifndef SYSV
        struct  filed *f_next;          /* next in linked list */
#endif
        gshort  f_type;                 /* entry type, see below */
        gshort  f_file;                 /* file descriptor */
        time_t  f_time;                 /* time this was last written */
        u_char  f_pmask[LOG_NFACILITIES+1];     /* priority mask */
        union {
                gchar    f_uname[MAXUNAMES][UNAMESZ+1];
                struct {
                        gchar    f_hname[MAXHOSTNAMELEN+1];
                        struct sockaddr_in      f_addr;
                } f_forw;               /* forwarding address */
                gchar    f_fname[MAXFNAME];
        } f_un;
        gchar    f_prevline[MAXSVLINE];          /* last message logged */
        gchar    f_lasttime[16];                 /* time of last occurrence */
        gchar    f_prevhost[MAXHOSTNAMELEN+1];   /* host from which recd. */
        gint     f_prevpri;                      /* pri of f_prevline */
        gint     f_prevlen;                      /* length of f_prevline */
        gint     f_prevcount;                    /* repetition cnt of prevline */
        gint     f_repeatcount;                  /* number of "repeated" msgs */        gint     f_flags;                        /* store some additional flags */
};

#endif


