/*
 * festival visual for the gMessaging system.
 * lots of code out of the festival_client.c example,
 * which was written by Alan W Black (awb@cstr.ed.ac.uk) at March 1999
 * TODO: remove the gnome-stall during long texts 
 *
 * 28-08-1999, BastardAI
 *
 */

/*
 * INCLUDES
 */
#include "plugin_visual_festival.h"
#include <gtk/gtk.h>
#include <gmodule.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>
#include <netinet/in.h>
#include <arpa/inet.h>

/* 
 * DEFINES 
 */

/*
 * globals
 */
static engine* Engine = NULL;
static glong Node;
static glong Data;
static gchar *socket_receive_file_to_buff(gint fd,gint *size);
static GtkWidget *plug;
/* For testing endianness */
static gint fapi_endian_loc = 1;

gpointer delete_FT_Info(FT_Info *info) {
    if (info != 0) {  g_free(info);}
	return NULL;
}

gint save_FT_Wave_snd(FT_Wave *wave, const gchar *filename) {
    FILE *fd;
	struct {
        guint    magic;  	/* magic number */
        guint    hdr_size;   /* size of this header */
        gint 	 data_size;  /* length of data (optional) */
        guint    encoding;   /* data encoding format */
        guint    sample_rate;/* samples per second */
        guint    channels;   /* number of interleaved channels */
    } header;
    gshort sw_short;
    gint i;

    if ((filename == 0) || (strcmp(filename,"stdout") == 0) || (strcmp(filename,"-") == 0))
        fd = stdout;
    else if ((fd = fopen(filename,"wb")) == NULL) {
            g_error("save_FT_Wave: can't open file \"%s\" for writing\n",filename);
            return FALSE;
		}
    header.magic = (guint)0x2e736e64;
	header.hdr_size = sizeof(header);
	header.data_size = 2 * wave->num_samples;
	header.encoding = 3; /* short */
	header.sample_rate = wave->sample_rate;
	header.channels = 1;
	if (FAPI_LITTLE_ENDIAN) {
        /* snd is always sparc/68000 byte order */
        header.magic = SWAPINT(header.magic);
        header.hdr_size = SWAPINT(header.hdr_size);
        header.data_size = SWAPINT(header.data_size);
        header.encoding = SWAPINT(header.encoding);
        header.sample_rate = SWAPINT(header.sample_rate);
        header.channels = SWAPINT(header.channels);
	}
    /* write header */
	if (fwrite(&header, sizeof(header), 1, fd) != 1)
	    return FALSE;
	if (FAPI_BIG_ENDIAN)
	    fwrite(wave->samples,sizeof(gshort),wave->num_samples,fd);
	else {
        /* have to swap */
    	    for (i=0; i < wave->num_samples; i++) {
            sw_short = SWAPSHORT(wave->samples[i]);
            fwrite(&sw_short,sizeof(gshort),1,fd);
        }
    }
    if (fd != stdout)
	    fclose(fd);
    return TRUE;
}


static FT_Info *festival_default_info() {
    FT_Info *info;
    info = (FT_Info *)g_malloc(1 * sizeof(FT_Info));

    info->server_host = (gchar*)FESTIVAL_DEFAULT_SERVER_HOST;
    info->server_port = FESTIVAL_DEFAULT_SERVER_PORT;
    info->text_mode = (gchar*)FESTIVAL_DEFAULT_TEXT_MODE;
    info->server_fd = -1;
    return info;
}

static gint festival_socket_open(const gchar *host, gint port) {
    /* Return an FD to a remote server */
	struct sockaddr_in serv_addr;
	struct hostent *serverhost;
	gint fd;

	fd = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
    if (fd < 0) {
	    g_print("festival_client: can't get socket\n");
	    return FALSE;
	}
	memset(&serv_addr, 0, sizeof(serv_addr));
	if ((serv_addr.sin_addr.s_addr = inet_addr(host)) == -1) {
	    /* its a name rather than an ipnum */
	    serverhost = gethostbyname(host);
	    if (serverhost == (struct hostent *)0) {
	        g_print("festival_client: gethostbyname failed\n");
	        return FALSE;
	    }
	    g_memmove(&serv_addr.sin_addr,serverhost->h_addr, serverhost->h_length);
	}
	serv_addr.sin_family = AF_INET;
	serv_addr.sin_port = htons(port);
    if (connect(fd, (struct sockaddr *)&serv_addr, sizeof(serv_addr)) != 0) {
	    g_print("festival_client: connect to server failed\n");
	    return FALSE;
    }
    return fd;
}

static gint nist_get_param_int(gchar *hdr, gchar *field, gint def_val) {
    gchar *p;
	gint val;

	if (((p=strstr(hdr,field)) != NULL) &&
	  (strncmp(" -i ",p+strlen(field),4) == 0)) {
	    sscanf(p+strlen(field)+4,"%d",&val);
	    return val;
	}
	else
	    return def_val;
}

static gint nist_require_swap(gchar *hdr) {
    gchar *p;
	gchar *field = (gchar*)"sample_byte_format";

	if ((p=strstr(hdr,field)) != NULL)
	{
	    if (((strncmp(" -s2 01",p+strlen(field),7) == 0) && FAPI_BIG_ENDIAN) ||
	       ((strncmp(" -s2 10",p+strlen(field),7) == 0)
	       && FAPI_LITTLE_ENDIAN))
			return FALSE;
    }
	return TRUE; /* if unknown assume native byte order */
}

static gchar *client_accept_s_expr(gint fd) {
    /* Read s-expression from server, as a char * */
    gchar *expr;
    gint filesize;

    expr = socket_receive_file_to_buff(fd,&filesize);
    expr[filesize] = '\0';
    return expr;
}

static FT_Wave *client_accept_waveform(gint fd) {
    /* Read waveform from server */
	gchar *wavefile;
	gint filesize;
	gint num_samples, sample_rate, i;
	FT_Wave *wave;

    wavefile = socket_receive_file_to_buff(fd,&filesize);
    wave = 0;
    /* I know this is NIST file and its an error if it isn't */
    if (filesize >= 1024) {
	    num_samples = nist_get_param_int(wavefile,(gchar*)"sample_count",0);
	    sample_rate = nist_get_param_int(wavefile,(gchar*)"sample_rate",16000);
	    if ((num_samples*sizeof(gshort))+1024 == filesize) {
	        wave = (FT_Wave *)g_malloc(sizeof(FT_Wave));
	        wave->num_samples = num_samples;
	        wave->sample_rate = sample_rate;
	        wave->samples = (gshort *)g_malloc(num_samples*sizeof(gshort));
		    g_memmove(wave->samples,wavefile+1024,num_samples*sizeof(gshort));
		    if (nist_require_swap(wavefile))
		        for (i=0; i < num_samples; i++)
		    wave->samples[i] = SWAPSHORT(wave->samples[i]);
		}
	}
   	g_free(wavefile);
    return wave;
}

static gchar *socket_receive_file_to_buff(gint fd,gint *size) {
    /* Receive file (probably a waveform file) from socket using   */
    /* Festival key stuff technique, but long winded I know, sorry */
    /* but will receive any file without closeing the stream or    */
    /* using OOB data                                              */
    static gchar *file_stuff_key = (gchar*)"ft_StUfF_key"; /* must == Festival's key */
    gchar *buff;
    gint bufflen;
    gint n,k,i;
    gchar c;

    bufflen = 1024;
    buff = (gchar *)g_malloc(bufflen);
    *size=0;
    for (k=0; file_stuff_key[k] != '\0';) {
	    n = read(fd,&c,1);
	    if (n==0) break;  /* hit stream eof before end of file */
	    	if ((*size)+k+1 >= bufflen) {
                /* +1 so you can add a NULL if you want */
		        bufflen += bufflen/4;
		        buff = (gchar *)realloc(buff,bufflen);
		    }
		    if (file_stuff_key[k] == c)
		        k++;
		    else if ((c == 'X') && (file_stuff_key[k+1] == '\0')) {
                /* It looked like the key but wasn't */
		        for (i=0; i < k; i++,(*size)++)
		        buff[*size] = file_stuff_key[i];
		        k=0;
		        /* omit the stuffed 'X' */
		    } else {
		        for (i=0; i < k; i++,(*size)++)
		        buff[*size] = file_stuff_key[i];
		        k=0;
		        buff[*size] = c;
		        (*size)++;
		    }
    }
    return buff;
}

FT_Info *festivalOpen(FT_Info *info) {
    /* Open socket to server */
	if (info == 0)
	    info = festival_default_info();
	info->server_fd =
	    festival_socket_open(info->server_host, info->server_port);
	    if (info->server_fd == -1)
	    	return NULL;
    return info;
}

FT_Wave *festivalStringToWave(FT_Info *info,gchar *text) {
    FT_Wave *wave;
	FILE *fd;
	gchar *p;
	gchar ack[4];
	gint n;

    if (info == 0)
	    return FALSE;
    if (info->server_fd == -1) {
	    g_print("festival_client: server connection unopened\n");
	    return FALSE;
    }
    fd = fdopen(dup(info->server_fd),"wb");
    /* Copy text over to server, escaping any quotes */
    fprintf(fd,"(SayText \"");
    for (p=text; p && (*p != '\0'); p++) {
	    if ((*p == '"') || (*p == '\\'))
		putc('\\',fd);
	    putc(*p,fd);
    }
	fprintf(fd,"\" \"%s\")\n",info->text_mode);
    fclose(fd);
    /* Read back info from server */
    /* This assumes only one waveform will come back, also LP is unlikely */
    wave = 0;
    do {
	    for (n=0; n < 3; )
	    n += read(info->server_fd,ack+n,3-n);
	    ack[3] = '\0';
	    if (strcmp(ack,"WV\n") == 0)         /* receive a waveform */
	        wave = client_accept_waveform(info->server_fd);
	    else if (strcmp(ack,"LP\n") == 0)    /* receive an s-expr */
	        client_accept_s_expr(info->server_fd);
	    else if (strcmp(ack,"ER\n") == 0) {   /* server got an error */
	        g_print("festival_client: server returned error\n");
	        break;
	    }
	} while (strcmp(ack,"OK\n") != 0);
	return wave;
}

gint festivalClose(FT_Info *info) {
    if (info == 0)
    	return FALSE;
    if (info->server_fd != -1)
	    close(info->server_fd);
    return TRUE;
}

gboolean visual_festival_procedure() {
    GString *message = NULL;
    gchar *server=0;
    gint port=-1;
    gchar *text=0;
    gchar *mode=0;
    GString *temp;
    FT_Info *info;
    FT_Wave *wave;

    /* put the text to be spoken in text */
    // TOO LONG!! text = (char*)MO->data;
    temp = g_string_new(Engine->secure_node_get_datadescription(Node,Engine->ENGINE_DL_ID,Engine->ENGINE_DL_PASSWORD->str)->str);
    g_string_append(temp,"   activity  ");
    text = temp->str;

    info = festival_default_info();
    if (server != 0)
        info->server_host = server;
    if (port != -1)
        info->server_port = port;
    if (mode != 0)
        info->text_mode = mode;
    info = festivalOpen(info);
    wave = festivalStringToWave(info,text);
    if (wave != 0)
    save_FT_Wave_snd(wave,"./messaging_visual_festival.wav");
    festivalClose(info);
    message = g_string_new((gchar*)Engine->secure_data_get_rawdata(Engine->secure_node_get_data(Node,Engine->ENGINE_DL_ID,Engine->ENGINE_DL_PASSWORD->str),Engine->ENGINE_DL_ID,Engine->ENGINE_DL_PASSWORD->str));
    /* mark data as transfered */
    Engine->secure_node_set_datapassedover(Node,TRUE,Engine->ENGINE_DL_ID,Engine->ENGINE_DL_PASSWORD->str);
    g_string_free(message,TRUE);
    g_string_free(temp,TRUE);
    /* reset procedure_running */
     Engine->secure_node_set_procedure_running(Node,FALSE,Engine->ENGINE_DL_ID,Engine->ENGINE_DL_PASSWORD->str);
    return TRUE;
}

gboolean visual_festival_cleanup() {
	return TRUE;
}

gboolean visual_festival_activate() {
    gint result;
    result = TRUE;
    return result;
}

gboolean visual_festival_init(glong NodeID,engine *_Engine) {
    Engine = _Engine;
    Node = NodeID;
    Data = Engine->secure_node_get_data(Node,Engine->ENGINE_DL_ID,Engine->ENGINE_DL_PASSWORD->str);
    /* register FESTIVAL as a visual */
    return TRUE;
}

const gchar* visual_festival_name() {
    return "visual Festival MO";
}

const gchar* visual_festival_description() {
    return "client for a festival server";
}

nodeType_t visual_festival_nodetype() {
    return (nodeType_t)VisualNode;
}

