#ifndef VISUAL_FESTIVAL
#define VISUAL_FESTIVAL

#include "messaging_plugin.h"

/* various internal functions */
#define FESTIVAL_DEFAULT_SERVER_HOST "localhost"
#define FESTIVAL_DEFAULT_SERVER_PORT 1314
#define FESTIVAL_DEFAULT_TEXT_MODE "fundamental"

/* proto's */
gboolean visual_festival_procedure();
gboolean visual_festival_cleanup();
gboolean visual_festival_activate();
gboolean visual_festival_init(glong Node,engine *Engine);
const gchar* visual_festival_name();
const gchar* visual_festival_description();
nodeType_t visual_festival_nodetype();

typedef struct FT_Info
{
    gint encoding;
	gchar *server_host;
	gint server_port;
	gchar *text_mode;
	gint server_fd;
} FT_Info;

typedef struct FT_Wave
{
    gint num_samples;
    gint sample_rate;
    gshort *samples;
} FT_Wave;

gpointer delete_FT_Wave(FT_Wave *wave);
gpointer delete_FT_Info(FT_Info *info);

#define SWAPSHORT(x) ((((unsigned)x) & 0xff) << 8 | \
   (((unsigned)x) & 0xff00) >> 8)
#define SWAPINT(x) ((((unsigned)x) & 0xff) << 24 | \
   (((unsigned)x) & 0xff00) << 8 | \
   (((unsigned)x) & 0xff0000) >> 8 | \
   (((unsigned)x) & 0xff000000) >> 24)

/* Sun, HP, SGI Mips, M68000 */
#define FAPI_BIG_ENDIAN (((gchar *)&fapi_endian_loc)[0] == 0)
/* Intel, Alpha, DEC Mips, Vax */
#define FAPI_LITTLE_ENDIAN (((gchar *)&fapi_endian_loc)[0] != 0)

/* If called with NULL will attempt to access using defaults */
FT_Info *festivalOpen(FT_Info *info);
FT_Wave *festivalStringToWave(FT_Info *info,gchar *text);
gint festivalClose(FT_Info *info);

/* - Auto plugin registration code -------------------------------*/

extern "C" {
    plugin_maker* maker_visual_festival(){
        plugin_maker *plug = (plugin_maker*)g_malloc(sizeof(plugin_maker));
        plug->procedure = visual_festival_procedure;
        plug->cleanup   = visual_festival_cleanup;
        plug->activate  = visual_festival_activate;
        (gpointer)plug->init      = visual_festival_init;
        plug->name      = visual_festival_name;
        plug->description = visual_festival_description;
        plug->nodetype  = visual_festival_nodetype;

        return plug;
    }

    class proxy {
        public:
            proxy() {
                /* fill in plugin info */
                BL_plugins["visual_festival"] = maker_visual_festival;
                /* register this plugin to plugins list */
                g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_DEBUG,"Registered visual_festival");
            }
    };

    // our one instance of the proxy
    proxy p;
}

#endif

