/*
 * GMS SECURITY MODULE - Bstrd 5 februari 2000
 *
 * This is responsible for correct operation of the DFP conserning:
 * - authorisation - basically done
 * - encryption - none done
 * - connections - none done
 * - deadlocks\loops\autocatalism - none
 * - overflows - none
 * 25-11-2000:	Jarl van Katwijk, added MOP2MOC checking
 */
 
//#include "DFPADM.hh"
#include "messaging.h"
#include "messaging_area.h"
#include "messaging_engine.h"
#include <X11/Xlib.h>
#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <gdk/gdkx.h>



/* public engine */
extern engine *Engine;



/*
 * Functions
 */

gboolean area::secure_register_dl(glong dlID,gchar *password,glong mdlID,gchar *password_area) {
    /* TODO: password checking */

    return register_dl(dlID,password,mdlID,password_area);
}

gboolean area::secure_unregister_dl(glong dlID,gchar *password) {
    return unregister_dl(dlID,password);
}

/*
 * Typecast data fields out rawmem
 */
gboolean area::secure_typecast(glong Data, glong DLid, gchar* Password) {
    /* Is DLid allowed to access node? */
    if (!authorized_dl_data(DLid,Password,Data)) {
        return FALSE;
    }

    /* Get data object */
    data* Data_obj = dataref_to_data(Data);

    return Data_obj->typecast(Data_obj);
}

/*
 * Load and config available nodes\plugins
 */
gboolean area::secure_setup_nodes(glong DLid,gchar *password,gpointer _Engine) {

    /* Publish engine */
    Engine = (engine*)_Engine;

    /* Is DLid allowed to access node? */
    if (!authorized_dl(DLid,password)) {
        return FALSE;
    }

    return setup_nodes(Engine);
}


/*
 * Check if DL is autorised, cq logged in succesfully
 */
gboolean area::authorized_dl(glong DLid,gchar *Password) {

    /* Valid dlId? */
	if (! ((DLid>0)&&(DLid<MAXDLID) )) {
        g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_DEBUG,"DL has no valid ID (%i)",DLid);
        return FALSE;
    }

    /* dlId registered? */
    if (DLS[DLid] == NULL) {
        g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_DEBUG,"DL %i is not registered",DLid);
        return FALSE;
    }

    /* Password stucture correct (not too simplistic etc) ? */
    // TODO

	return TRUE;
}


/*
 * Check if DL is autorised to access a node
 */
gboolean area::authorized_dl_node(glong DLid,gchar *Password,glong Node) {

    /* DL authorized at all? */
    if (!authorized_dl(DLid,Password)) {
        g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_DEBUG,"DL is not authorized");
        return FALSE;
    }

    /* DL allowed to access Node? */
    if (Node<0) {
        return FALSE;
    }

    //TODO !!!

    return TRUE;
}


/*
 * Check if DL is autorised to access a data
 */
gboolean area::authorized_dl_data(glong DLid,gchar *Password,glong Data) {

    /* DL authorized at all? */
    if (!authorized_dl(DLid,Password)) {
        return FALSE;
    }

    /* DL allowed to access Data? */
    //TODO !!!

    return TRUE;
}


/*
 * dummy func.
 */
gint ignore_x_error(Display* d, XErrorEvent* e) {
	g_print("ignore_x_error called\n");
	return 0;
}


/* 
 * Get Xwindow ID from X server
 */
gint messaging_security_get_window_id(Window win, char *title, guint32 *wid)
{
	Window root_return;
	Window parent_return;
	Window *children;
	guint nchildren;
	guint i;
	gchar *tit;
	gint ret = FALSE;
	gint (*oldErrorHandler)(Display*, XErrorEvent*);

	XQueryTree(GDK_DISPLAY(),win,&root_return,&parent_return,&children,&nchildren);

	oldErrorHandler = XSetErrorHandler(ignore_x_error);
	for(i=0;i<nchildren;i++)
	{
		if (!XFetchName(GDK_DISPLAY(),children[i],&tit))
		{
			continue;
		}
		if(tit)
		{
			if(strcmp(tit,title)==0)
			{
				XFree(tit);
				*wid = children[i];
				ret = TRUE;
				break;
			}
			XFree(tit);
		}
	}
	XSetErrorHandler(oldErrorHandler);
	for(i=0;!ret && i<nchildren;i++)
		ret=messaging_security_get_window_id(children[i],title,wid);
	if(children)
		XFree(children);
	return ret;
}


/*
 * Check if node allowes datastreaming from data
 */
gboolean area::secure_node_allowes_data(glong Node,glong Data,glong DLid,gchar* Password) {

    /* Check if DL is allowed to access Node */
    if (!authorized_dl_node(DLid,Password,Node)) {
        return FALSE;
    }

    /* OK, secure, get data and node objects */
    data* Data_obj = dataref_to_data(Data);
    node* Node_obj = noderef_to_node(Node);

    /* Check if Data has any description at all */
    if (Data_obj->datadescription->len == 0) {
        g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_WARNING,"Data %li of Node %s has no description, transaction canceled",Data_obj->ID,Node_obj->dname->str);
        return FALSE;
    }

    /* Check originating Node (node that put data into system, should always be a sensor */
    node* Node_Data = noderef_to_node(Data_obj->originatorID);
    if (Node_Data->nodeType != SensorNode) {
        return FALSE;
    }

    /* Is the destination the pipeline? */
    if (Node_obj->nodeType == PipeLineNode) {
        // OK
    } else {
        /* Does Node's allowedFilter matches datadescription? */
        if ( !Engine->regexpmatch(Data_obj->datadescription->str, Node_obj->allowedFilter->str)) {
            g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_WARNING,"Node %s in area %li does not allow data block %li, transaction canceled",Node_obj->dname->str,Node_obj->uri->subnetID,Data_obj->ID);
            g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_DEBUG,"Node->allowedFilter = %s,  Data->datadescription = %s",Node_obj->allowedFilter->str,Data_obj->datadescription->str);
            return FALSE;
        }
    }

    g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_INFO,"Data block %i is allowed to access Node %s",Data_obj->ID,Node_obj->dname->str);
    return TRUE;
}


glong area::secure_node_lookup_by_dname(gchar* dname,glong DLid,gchar* Password) {
    glong Node;

    /* Find node for dname */
    Node = node_to_noderef(node_lookup_by_dname(dname));

    /* Is DLid allowed to access node? */
    if (!authorized_dl_node(DLid,Password,Node)) {
        return -2;
    }

    /* All ok, return node */
    return Node;
}


glong area::secure_node_lookup_by_filter(gchar* Filter,glong DLid,gchar* Password) {
    glong Node;

    /* Find node for filter */
    Node = node_to_noderef(node_lookup_by_filter(Filter));

    /* Is DLid allowed to access node? */
    if (!authorized_dl_node(DLid,Password,Node)) {
        return -2;
    }

    /* All ok, return node */
    return Node;
}


gboolean area::secure_relate_node_to_node(glong Parent_node,glong Child_node,glong DLid,gchar* Password) {
    /* simply create a relation */
    gint oops = FALSE;
    glong currrel,i;

    /* Is DLid allowed to access nodes? */
    if (!authorized_dl_node(DLid,Password,Parent_node)) {
        return -1;
    }
    if (!authorized_dl_node(DLid,Password,Child_node)) {
        return -2;
    }

    node* Parent_node_obj = noderef_to_node(Parent_node);
    node* Child_node_obj = noderef_to_node(Child_node);
    // TODO: conflicting relations should be filtered out 
    // DONE: same conflicts
    currrel = node_get_currentrelations(Parent_node_obj);
    for (i=0;i<currrel;i++) {
        if (node_get_relation(Parent_node_obj,i) == Child_node_obj) {
            oops = TRUE;
            i = currrel;
            secure_conflictportal(Parent_node,Child_node,SameRT,DLid,Password);
            return FALSE;
        }
    }
    if (oops==FALSE) {
        relate_node_to_node(Parent_node_obj,Child_node_obj);
    }

    /* Passed all checks, setup relation */
    relate_node_to_node(Parent_node_obj,Child_node_obj);
    return TRUE;
}

gboolean area::secure_relate_dname_to_dname(gchar* parent,gchar* child,glong DLid,gchar* Password) {
    /* Relate a node based on dname to node based on dname */
    node* parent_obj = node_lookup_by_dname(parent);
    node* child_obj = node_lookup_by_dname(child);
    
    //g_print("parent_obj = %p (%s),  child_obj = %p (%s)\n",parent_obj,parent,child_obj,child);
    /* Authorization */
    if (!authorized_dl_node(DLid,Password,node_to_noderef(parent_obj))) {
        return FALSE;
    }

    if (!authorized_dl_node(DLid,Password,node_to_noderef(child_obj))) {
        return FALSE;
    }

    return relate_node_to_node(parent_obj,child_obj);
}

GString* area::secure_node_get_dname(glong Node,glong DLid,gchar* Password) {
    /* Returns dname of node */
    /* Authorization */
    if (!authorized_dl_node(DLid,Password,Node)) {
        return FALSE;
    }

    return noderef_to_node(Node)->dname;
}

glong area::secure_node_get_name(glong Node,glong DLid,gchar* Password) {
    /* Returns name (=id) of node, used for backward compatibility */

    /* Authorization */
    if (!authorized_dl_node(DLid,Password,Node)) {
        return FALSE;
    }

    return Node;
}

GString* area::secure_node_get_description(glong Node,glong DLid,gchar* Password) {
    /* Returns description of node */
    node* NodeObj;

    /* Authorization */
    if (!authorized_dl_node(DLid,Password,Node)) {
        return FALSE;
    }

    NodeObj = noderef_to_node(Node);
    if (NodeObj == NULL) {
        return NULL;
    } else {
        return NodeObj->dname;
    }
}

nodeType_t area::secure_node_get_nodetype(glong Node,glong DLid,gchar* Password) {
    /* Returns type of node */
    /* Authorization */
    if (!authorized_dl_node(DLid,Password,Node)) {
        return UnknownNode;
    }

    node* Node_obj = noderef_to_node(Node);

    if (Node_obj == NULL) {
        return UnknownNode;
    } else {
        return Node_obj->nodeType;
    }
}

gpointer area::secure_node_get_procedure(glong Node,glong DLid,gchar* Password) {
    /* Returns procedure of node */
    /* Authorization */
    if (!authorized_dl_node(DLid,Password,Node)) {
        return NULL;
    }

    node* Node_obj = noderef_to_node(Node);
    if (Node_obj == NULL) {
        return NULL;
    } else {
        return (gpointer)Node_obj->procedure_ref;
    }
}

statusType_t area::secure_node_get_status(glong Node,glong DLid,gchar* Password) {
    /* Returns status of node */
    /* Authorization */
    if (!authorized_dl_node(DLid,Password,Node)) {
        return Unknown;
    }

    return noderef_to_node(Node)->status;
}

GString* area::secure_node_get_allowedfilter(glong Node,glong DLid,gchar* Password) {
    /* Returns filter which defines what data is allowed by node */
    /* Authorization */
    if (!authorized_dl_node(DLid,Password,Node)) {
        return NULL;
    }

    return noderef_to_node(Node)->allowedFilter;
}

GString* area::secure_node_get_datadescription(glong Node,glong DLid,gchar* Password) {
    /* Returns description of node thats given to it's data */
    /* Authorization */
    if (!authorized_dl_node(DLid,Password,Node)) {
        return NULL;
    }

    return noderef_to_node(Node)->dataDescription;
}

glong area::secure_node_get_relation(glong Node,glong relation_nr,glong DLid,gchar* Password) {
    /* returns a relation of node */
    /* Authorization */
    if (!authorized_dl_node(DLid,Password,Node)) {
        return -1;
    }

    /* Check if relation is available at all */
    if (relation_nr > g_list_length(noderef_to_node(Node)->relationList)) {
        g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_WARNING,"Node %s hasn't got a relation nr %i (%i relations)",noderef_to_node(Node)->dname->str,relation_nr,g_list_length(noderef_to_node(Node)->relationList));
        return -1;
    }

    relation *Relation = (relation*)g_list_nth_data(noderef_to_node(Node)->relationList,relation_nr);
    node* Node_relation = noderef_to_node(Relation->Node);
    if (Node_relation != NULL) {
        return Node_relation->uri->nodeID;
    } else {
        return -1;
    }
}

glong area::secure_node_get_currentrelations(glong Node,glong DLid,gchar* Password) {
    /* Returns number of relations to node (NOT current childs) */
    /* Authorization */
    if (!authorized_dl_node(DLid,Password,Node)) {
        return -1;
    }

    return noderef_to_node(Node)->externalRelations;
}

glong area::secure_node_get_data(glong node,glong DLid,gchar* Password) {
    /* Returns referance ID to data of node */
    /* Authorization */
    if (!authorized_dl_node(DLid,Password,node)) {
        return -1;
    }

    return data_to_dataref(noderef_to_node(node)->Data);
}

glong area::secure_node_get_childs(glong Node,glong DLid,gchar* Password) {
    /* Returns number of childs of node */
    /* Authorization */
    if (!authorized_dl_node(DLid,Password,Node)) {
        return -1;
    }

    return g_list_length(noderef_to_node(Node)->relationList);
}

glong area::secure_node_get_nth_child(glong node,glong child,glong DLid,gchar* Password) {
    /* Returns a child of node */
    /* Authorization */
    if (!authorized_dl_node(DLid,Password,node)) {
        return -1;
    }

    /* TODO: Implement */
    g_error("glong secure_node_get_nth_child(glong node,glong child,glong DLid,gchar* Password) UNIMPLEMENTED");

    return -1;
}

gboolean area::secure_node_get_datapassedover(glong Node,glong DLid,gchar* Password) {
    /* Return status of datapassedover of node */
    /* Authorization */
    if (!authorized_dl_node(DLid,Password,Node)) {
        return FALSE;
    }

    node* Node_obj = noderef_to_node(Node);

    if (Node_obj == NULL) {
        return TRUE; /* None exsisting node, so always passed */
    } else {
        return Node_obj->dataPassedOver;
    }

    return FALSE;
}

GString* area::secure_node_get_configinit(glong node,glong DLid,gchar* Password) {
    /* Return status of configInit (default configuration of node) */

    /* Authorization */
    if (!authorized_dl_node(DLid,Password,node)) {
        return FALSE;
    }

    /* TODO: Implement */
    g_error("area::secure_node_get_configinit UNIMPLEMENTED");

    return FALSE;
}

GString* area::secure_node_get_configruntime(glong Node,glong DLid,gchar* Password) {
    /* Return status of configRuntime (normal configuration of node) */

    /* Authorization */
    if (!authorized_dl_node(DLid,Password,Node)) {
        return NULL;
    }

    GString *config = g_string_new(noderef_to_node(Node)->configRuntime->str);
    return config;
}

uriS* area::secure_node_get_uri(glong Node,glong DLid,gchar* Password) {
    /* Returns uri of node */
    node* NodeObj;

    /* Authorization */
    if (!authorized_dl_node(DLid,Password,Node)) {
        return FALSE;
    }

    NodeObj = noderef_to_node(Node); 
    if (NodeObj != NULL) {
        return NodeObj->uri;
    } else {
        return NULL;
    }
}

glong area::secure_nodes(glong DLid,gchar* Password) {
    /* Returns number of active node in area DL belongs to */
    /* Authorization */
    if (!authorized_dl(DLid,Password)) {
        return FALSE;
    }

    // TODO: Return ML's nodes, not the total nodes for all ML's
    return Nodes;
}

gboolean area::secure_node_get_procedure_handled(glong Node,glong DLid,gchar* Password) {
    /* Returns status of procedure handled of node */
    /* Authorization */
    if (!authorized_dl_node(DLid,Password,Node)) {
        return FALSE;
    }

    node* Node_obj = noderef_to_node(Node);

    if (Node_obj == NULL) {
        return TRUE; /* None exsisting node, so always handled */
    } else {
        return Node_obj->procedure_handled;
    }
}

gboolean area::secure_node_get_procedure_running(glong Node,glong DLid,gchar* Password) {
    /* Returns status of procedure running of node */
    node* NodeObj = noderef_to_node(Node);

    /* Authorization */
    if (!authorized_dl_node(DLid,Password,Node)) {
        return FALSE;
    }

    /* Return status */
    return NodeObj->procedure_running;
}

gpointer area::secure_node_get_visualplug_buildprocedure(glong node,glong DLid,gchar* Password) {
    /* Returns pointer to visualplug_buildprocedure of node */
    /* Authorization */
    if (!authorized_dl_node(DLid,Password,node)) {
        return FALSE;
    }

    /* TODO: Implement */
    g_error("gpointer area::secure_node_get_visualplug_buildprocedure(glong node,glong DLid,gchar* Password) UNIMPLEMENTED");

    return NULL;
}

glong area::secure_data_get_id(glong Data,glong DLid,gchar* Password) {
    /* Returns id of data */
    /* Authorization */
    if (!authorized_dl_data(DLid,Password,Data)) {
        return FALSE;
    }

    return dataref_to_data(Data)->ID;
}

GString* area::secure_data_get_datadescription(glong Data,glong DLid,gchar* Password) {
    /* Returns datadescription of data */
    /* Authorization */
    if (!authorized_dl_data(DLid,Password,Data)) {
        return FALSE;
    }

    return dataref_to_data(Data)->datadescription;
}

gchar* area::secure_data_get_rawdata(glong Data,glong DLid,gchar* Password) {
    /* Returns rawdata of data */
    /* Authorization */
    if (!authorized_dl_data(DLid,Password,Data)) {
        return FALSE;
    }

    return dataref_to_data(Data)->rawdata;
}

glong area::secure_data_get_rawlen(glong data,glong DLid,gchar* Password) {
    /* Returns rawlen of data */
    /* Authorization */
    if (!authorized_dl_data(DLid,Password,data)) {
        return FALSE;
    }

    /* TODO: Implement */
    g_error("secure_data_get_rawlen UNIMPLEMENTED");

    return -1;
}

GString* area::secure_data_get_string(glong data,glong DLid,gchar* Password) {
    /* Returns string of data */
    /* Authorization */
    if (!authorized_dl_data(DLid,Password,data)) {
        return FALSE;
    }

    /* TODO: Implement */
    g_error("secure_data_get_string UNIMPLEMENTED");

    return NULL;
}

gint area::secure_data_get_integer(glong data,glong DLid,gchar* Password) {
    /* Returns integer of data */
    /* Authorization */
    if (!authorized_dl_data(DLid,Password,data)) {
        return FALSE;
    }

    /* TODO: Implement */
    g_error("secure_data_get_integer UNIMPLEMENTED");

    return -1;
}

gint area::secure_data_get_percentage(glong data,glong DLid,gchar* Password) {
    /* Returns percentage of data */
    /* Authorization */
    if (!authorized_dl_data(DLid,Password,data)) {
        return FALSE;
    }

    /* TODO: Implement */
    g_error("secure_data_get_percentage UNIMPLEMENTED");

    return -1;
}

gboolean area::secure_data_get_boolean(glong data,glong DLid,gchar* Password) {
    /* Returns boolean of data */
    /* Authorization */
    if (!authorized_dl_data(DLid,Password,data)) {
        return FALSE;
    }

    /* TODO: Implement */
    g_error("secure_data_get_boolean UNIMPLEMENTED");

    return FALSE;
}

gboolean area::secure_node_set_dname(glong node,gchar* dname,glong DLid,gchar* Password) {
    /* Sets dname of node */
    /* Authorization */
    if (!authorized_dl_node(DLid,Password,node)) {
        return FALSE;
    }

    /* TODO: Implement */
    g_error("secure_node_set_dname UNIMPLEMENTED");

    return FALSE;
}

gboolean area::secure_node_set_status(glong node,statusType_t status,glong DLid,gchar* Password) {
    /* Sets status of node */

    /* TODO: Implement */
    g_error("secure_node_set_status UNIMPLEMENTED");

    return FALSE;
}

gboolean area::secure_node_set_description(glong node,gchar* text,glong DLid,gchar* Password) {
    /* Sets description of node */
    /* Authorization */
    if (!authorized_dl_node(DLid,Password,node)) {
        return FALSE;
    }


    /* TODO: Implement */
    g_error("secure_node_set_description UNIMPLEMENTED");

    return FALSE;
}

gboolean area::secure_node_set_datadescription(glong Node,gchar* text,glong DLid,gchar* Password) {
    /* Sets datadescription of node */
    /* Authorization */
    if (!authorized_dl_node(DLid,Password,Node)) {
        return FALSE;
    }

    node* Node_obj = noderef_to_node(Node);
    if (Node_obj == NULL) {
        return FALSE;
    } else {
        g_string_sprintf(Node_obj->dataDescription,"%s",text);
        return TRUE;
    }
}

gboolean area::secure_node_set_nodetype(glong node,nodeType_t motype,glong DLid,gchar* Password) {
    /* Sets nodetype of node */
    /* Authorization */
    if (!authorized_dl_node(DLid,Password,node)) {
        return FALSE;
    }


    /* TODO: Implement */
    g_error("secure_node_set_nodetype UNIMPLEMENTED");

    return FALSE;
}

gboolean area::secure_node_set_procedure_handled(glong Node,gboolean handled,glong DLid,gchar* Password) {
    /* Sets handled status of procedure of node */
    /* Authorization */
    if (!authorized_dl_node(DLid,Password,Node)) {
        return FALSE;
    }

    /* Set procedure_handled status */
    noderef_to_node(Node)->procedure_handled = handled;

    return TRUE;
}

gboolean area::secure_node_set_procedure_running(glong Node,gboolean running,glong DLid,gchar* Password) {
    /* Sets running status of procedure of node */
    /* Authorization */
    if (!authorized_dl_node(DLid,Password,Node)) {
        return FALSE;
    }

    /* Set procedure_running status */
    noderef_to_node(Node)->procedure_running = running;

    return FALSE;
}

gboolean area::secure_node_set_datapassedover(glong Node,gboolean passed,glong DLid,gchar* Password) {
    /* Sets status of datapassedover of node */

    /* Authorization */
    if (!authorized_dl_node(DLid,Password,Node)) {
        return FALSE;
    }


    node* Node_obj = noderef_to_node(Node);

    if (Node_obj == NULL) {
        return FALSE;
    } else {
        Node_obj->dataPassedOver = passed;
        return TRUE;
    }
}

gboolean area::secure_node_set_uisetup(glong node,gboolean uisetup,glong DLid,gchar* Password) {
    /* Sets status of uisetup of node */
    /* Authorization */
    if (!authorized_dl_node(DLid,Password,node)) {
        return FALSE;
    }


    /* TODO: Implement */
    g_error("secure_node_set_uisetup UNIMPLEMENTED");

    return FALSE;
}

gboolean area::register_node_check_regulair(node *Node) {
    /* Checks regulair member fields for valid data */
    /* TODO Add procedure field, check if plugin has added needed functionality to node */
    if (Node->dname->len == 0) {
        g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_WARNING,"Node %i doesn't has a valid dname, quitting registration..",node_to_noderef(Node));
        return FALSE;
    }

    if (Node->uri->subnetID < 1) {
        g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_WARNING,"Node %s doesn't has a valid subnetID, quitting registration..",Node->dname->str);
        return FALSE;
    }

    if (Node->Data == NULL) {
        g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_WARNING,"Node %s doesn't has valid data, quitting registration..",Node->dname->str);
        return FALSE;
    }

    if (Node->dDescription->len == 0) {
        g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_WARNING,"Node %s doesn't has a valid ddescription, quitting registration..",Node->dname->str);
        return FALSE;
    }

    return TRUE;
}

gboolean area::register_node_check_refs(node *Node) {
    /* Checks member ref fields for valid data */
    if (Node->init_ref == NULL) {
        g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_WARNING,"Node %s doesn't has a valid init_ref, quitting registration..",Node->dname->str);
        return FALSE;
    }

    if (Node->procedure_ref == NULL) {
        g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_WARNING,"Node %s doesn't has a valid procedure_ref, quitting registration..",Node->dname->str);
        return FALSE;
    }

    if (Node->cleanup_ref == NULL) {
        g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_WARNING,"Node %s doesn't has a valid cleanup_ref, quitting registration..",Node->dname->str);
        return FALSE;
    }

    if (Node->activate_ref == NULL) {
        g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_WARNING,"Node %s doesn't has a valid activate_ref, quitting registration..",Node->dname->str);
        return FALSE;
    }

    return TRUE;
}

gboolean area::secure_register_node(glong Node,glong DLid, gchar* Password) {
    /* Registers a Node. NOTE that node needs to be fully initialized, use resouce::node_initialize() for this purpose */

    /* Check node */
    if (Node<0) {
       g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_ERROR,"Can not register Noderef %i",Node);
       return FALSE;
    }
    /* Authorization */
    if (!authorized_dl(DLid,Password)) {
        return FALSE;
    }

    /* 
     * ... Secured from here ...
     */
 
    node* Node_obj = noderef_to_node(Node);
    if (Node_obj == NULL) {
        g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_WARNING,"Cant register a NULL node");
        return FALSE;
    }

    /* Check normal node fields */
    if (!register_node_check_regulair(Node_obj)) {
        return FALSE;
    }

    /* Ref fields checking */
    if ((Node_obj->nodeType == PipeLineNode) || (Node_obj->nodeType == VisualCentralNode) || (Node_obj->nodeType == TemporaryNode)) {
        /* No ref checking */
    } else {
        if (!register_node_check_refs(Node_obj)) {
            return FALSE;
        }
    }

    /* Register node */
    if (!register_node(DLid,Node_obj,Engine)) {
        return FALSE; 
    }

    //TODO register visuals to VISUAL_NODE and sensors to PIPELINE_NODE automatically
    return TRUE;
}

gboolean area::secure_idle(glong delay,glong DLid, gchar* Password) {

    /* Authorization */
    if (!authorized_dl(DLid,Password)) {
        return FALSE;
    }

    idle(delay);
    return TRUE;
}

gboolean area::secure_node_add_blockedlist(glong Node,glong Data,glong DLid,gchar* Password) {
    /* Add Data to begin of DataBlockedList of Node */
 
    /* Authorization */
    if (!authorized_dl_node(DLid,Password,Node)) {
        return FALSE;
    }

    /* Refs to objects */
    data* Data_obj = dataref_to_data(Data);
    node* Node_obj = noderef_to_node(Node);

    /* Data same area as Node? */
    if (Node_obj->uri->subnetID != Data_obj->ID) {
        return FALSE;
    }

    /* Add Data to Node's blockedlist */
    return node_add_blockedlist(Node_obj,Data_obj);
}

void area::secure_lock(const gchar *text,glong DLid,gchar* Password) {
    /* Authorization */
    if (!authorized_dl(DLid,Password)) {
        return;
    }

    BL_lock(text);
    return;
}

void area::secure_unlock(const gchar *text,glong DLid,gchar* Password) {
    /* Authorization */
    if (!authorized_dl(DLid,Password)) {
        return;
    }

    BL_unlock(text);
    return;
}

void area::secure_locknunlock(const gchar *text,glong DLid,gchar* Password) {
    /* Authorization */
    if (!authorized_dl(DLid,Password)) {
        return;
    }

    BL_locknunlock(text);
    return;
}

glong area::secure_node_new(gchar *dname,gchar *ddescription,gchar *allowedfilter,nodeType_t type,gpointer init_ref,gpointer procedure_ref,gpointer cleanup_ref, gpointer activate_ref,glong DLid,gchar* Password) {
    /* Allocates new node in DL's area */
    node* Node;

    /* Authorization */
    if (!authorized_dl(DLid,Password)) {
        return -1;
    }

    /* Allocate new node */
    Node = node_new(dname,ddescription,allowedfilter,type,DLid,init_ref,procedure_ref,cleanup_ref,activate_ref);

    return node_to_noderef(Node);
}

gboolean area::secure_node_delete(const glong node,glong DLid,gchar* Password) {
    /* delete node based on node referance */

    /* Authorization */
    if (!authorized_dl_node(DLid,Password,node)) {
        return FALSE;
    }

    /* delete node */
    node_delete(noderef_to_node(node));

    return TRUE;
}

/* Cloons a Node including all the lists and history */
glong area::secure_node_cloon(glong Node,glong DLid,gchar* Password) {
    node* NodeObj;
    node* NodeObjNew;

    /* Authorization */
    if (!authorized_dl_node(DLid,Password,Node)) {
        return -1;
    }

    /* Cloon Node */
    NodeObj = noderef_to_node(Node);
    NodeObjNew = node_new( NodeObj->dname->str, NodeObj->dDescription->str, NodeObj->allowedFilter->str, NodeObj->nodeType, DLid, (void*)NodeObj->init_ref, (void*)NodeObj->procedure_ref, (void*)NodeObj->cleanup_ref, (void*)NodeObj->activate_ref);
    NodeObjNew->addresSpaceList = g_list_copy(NodeObj->addresSpaceList);
    NodeObj->Data->data_copy(NodeObjNew->Data,NodeObj->Data);
    NodeObjNew->dataSequence = NodeObj->dataSequence;
    NodeObjNew->dataPassedOver = NodeObj->dataPassedOver;
    NodeObjNew->dataBlockedList = g_slist_copy(NodeObj->dataBlockedList);
    g_string_sprintf(NodeObjNew->historyFilename,"%s",NodeObj->historyFilename);
    NodeObjNew->historyIO = g_io_channel_unix_new(g_io_channel_unix_get_fd(NodeObj->historyIO));
    NodeObjNew->relationList = g_list_copy(NodeObj->relationList);
    for (gint i=0;i<g_list_length(NodeObjNew->relationList);i++) {
        noderef_to_node(((relation*)(g_list_nth_data(NodeObjNew->relationList,i)))->Node)->externalRelations++;
    }
    NodeObjNew->externalRelations = NodeObj->externalRelations;
    NodeObjNew->procedure_handled = NodeObj->procedure_handled;
    NodeObjNew->procedure_running = NodeObj->procedure_running;

    return node_to_noderef(NodeObjNew);
}

gboolean area::secure_node_add_relation(glong node,glong relation,glong DLid,gchar* Password) {
    /* Adds relation to node */

    /* Authorization */
    if (!authorized_dl_node(DLid,Password,node)) {
        return FALSE;
    }

    /* add relation */
    relate_node_to_node(noderef_to_node(node),noderef_to_node(relation));

    return TRUE;
}

gboolean area::secure_node_set_allowedfilter(glong Node,gchar* filter,glong DLid,gchar* Password) {
    /* sets allowedfilter field of node */

    /* Authorization */
    if (!authorized_dl_node(DLid,Password,Node)) {
        return FALSE;
    }

    /* Set allowed filter of node */
    node* Node_obj = noderef_to_node(Node);

    if (Node_obj != NULL) {
        g_string_sprintf(Node_obj->allowedFilter,"%s",filter);
    } else {
        return FALSE;
    }

    return TRUE;
}

gboolean area::secure_node_set_configinit(glong Node,gchar* config,glong DLid,gchar* Password) {
    /* Set default configuration of node */

    /* Authorization */
    if (!authorized_dl_node(DLid,Password,Node)) {
        return FALSE;
    }

    /* TODO: IMPLEMENT */
    g_error("area::secure_node_set_configinit UNIMPLEMENTED");

    return FALSE;

}

gboolean area::secure_node_set_configruntime(glong Node,gchar* config,glong DLid,gchar* Password) {
    /* Set runtime configuration of node */

    /* Authorization */
    if (!authorized_dl_node(DLid,Password,Node)) {
        return FALSE;
    }

    /* Set configruntime of node */
    node* Node_obj = noderef_to_node(Node);

    if (Node_obj != NULL) {
        g_string_sprintf(Node_obj->configRuntime,"%s",config);
    } else {
        return FALSE;
    }

    return TRUE;
}

gboolean area::secure_data_set_originatorID(glong Data,glong ID,glong DLid,gchar* Password) {
    /* Set originator ID of data block */

    /* Authorization */
    if (!authorized_dl_data(DLid,Password,Data)) {
        return FALSE;
    }

    /* Set configruntime of node */
    data* Data_obj = dataref_to_data(Data);

    if (Data_obj != NULL) {
        Data_obj->originatorID = ID;
    } else {
        return FALSE;
    }

    return TRUE;
}


glong area::secure_historyIO_write(glong Node,glong DLid, gchar* Password) {
    /* Writes data of node to history file */
    node* Node_obj;
    gint bytes_written;

    /* Authorization */
    if (!authorized_dl_node(DLid,Password,Node)) {
        return -1;
    }

    Node_obj = noderef_to_node(Node);

    bytes_written = historyIO_write(Node_obj);

    return bytes_written;
}

glong area::secure_data_new(glong Data,glong rawmemlen,glong DLid,gchar* Password) {
    /* Allocates a new data in the area to which DLid belongs */

    /* Authorization */
    if (!authorized_dl_data(DLid,Password,Data)) {
        return -1;
    }

    data* Data_obj = data_new(dataref_to_data(Data)->datadescription->str,rawmemlen);

    /* return referance to data */
    return data_to_dataref(Data_obj);
}

glong area::secure_data_new(glong Data,glong DLid,gchar* Password) {
    /* Aloocates a new data in the area to which DLid belongs */

    /* Authorization */
    if (!authorized_dl_data(DLid,Password,Data)) {
        return -1;
    }

    data* Data_obj = data_new("No description",DATA_RAWLEN);

    /* return referance to data */
    return data_to_dataref(Data_obj);
}

gboolean area::secure_data_delete(const glong data,glong DLid,gchar* Password) {
    /* Delete a data based on data referance */

    /* Authorization */
    if (!authorized_dl_data(DLid,Password,data)) {
        return FALSE;
    }

    data_delete(dataref_to_data(data));

    return TRUE;
}

gboolean area::secure_data_copy(glong Data,glong source,glong DLid,gchar* Password) {
    /* Copies source data into data */
    data *DataObj;
    data *sourceObj;

    /* Authorization */
    if (!authorized_dl_data(DLid,Password,source)) {
        return FALSE;
    }

    if (!authorized_dl_node(DLid,Password,Data)) {
        return FALSE;
    }

    if (Data == source) {
        g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_WARNING,"Source data block is same as destination (ID=%li), data copy canceled",Data);
        return FALSE;
    }

    BL_lock("data_copy");
    /* Check if data blocks are not NULL */
    DataObj = dataref_to_data(Data);
    sourceObj = dataref_to_data(source);
    if ((DataObj == NULL) || (sourceObj == NULL)) {
        g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_WARNING,"Cant copy data blocks from\to NULL blocks");
        return FALSE;
    }

    /* Copy data block */
    DataObj->data_copy(DataObj,sourceObj);
    g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_DEBUG,"Copied data block %li to block %li",source,Data);
  
    /* Clean & Exit */
    BL_unlock("data copy");
    return TRUE;
}

gboolean area::secure_data_clear(glong data,glong DLid,gchar* Password) {
    /* Clears data */

    /* Authorization */
    if (!authorized_dl_data(DLid,Password,data)) {
        return FALSE;
    }

    /* Clear */
    
}

gboolean area::secure_node_execute_procedure(glong Node,glong DLid,gchar* Password) {
    /* Execute procedure of node */
    node* NodeObj = noderef_to_node(Node);
    gboolean result;

    /* Authorization */
    if (!authorized_dl_node(DLid,Password,Node)) {
        return FALSE;
    }

    /* Execute procedure */
    /* TODO: Hook segfault here temporary */
    result = (NodeObj->procedure_ref)();
    /* TODO: Unhook segfaults handler */
    
    return result;
}

gboolean area::secure_node_execute_visualplug_buildprocedure(glong Node,glong XID,glong DLid,gchar* Password) {
    /* Execute visualplug buildprocedure */

    /* Authorization */
    if (!authorized_dl_node(DLid,Password,Node)) {
        return FALSE;
    }

    /* TODO: IMPLEMENT */
    g_error("secure_node_execute_visualplug_buildprocedure UNIMPLEMENTED");

    return FALSE;
}

gboolean area::secure_data_set_datadescription(glong data,gchar* datadescription,glong DLid,gchar* Password) {
    /* Sets datadescription of data */

    /* Authorization */
    if (!authorized_dl_data(DLid,Password,data)) {
        return FALSE;
    }

    /* Set datadescription */
    g_string_sprintf(dataref_to_data(data)->datadescription,"%s",datadescription);

    return TRUE;
}

gboolean area::secure_data_set_rawdata(glong data,gchar* rawdata,gint len,glong DLid,gchar* Password) {
    /* Sets rawdata (and rawlen) of data */

    /* Authorization */
    if (!authorized_dl_data(DLid,Password,data)) {
        return FALSE;
    }

    /* Check len */
    if ((len == 0) || (len>DATA_RAWLEN)) {
        return FALSE;
    }

    /* Set rawdata */
    g_memmove(dataref_to_data(data)->rawdata,rawdata,len);

    return TRUE;
}

gboolean area::secure_data_clean_rawdata(glong Data,glong DLid,gchar* Password) {
    /* Fill rawdata with 0's */
    data* Data_obj;

    /* Authorization */
    if (!authorized_dl_data(DLid,Password,Data)) {
        return FALSE;
    }

    /* Check data object */
    Data_obj = dataref_to_data(Data);
    if (Data_obj == NULL) {
        return FALSE;
    }
    if ((Data_obj->rawdatalen == 0) || (Data_obj->rawdatalen > DATA_RAWLEN)) {
        return FALSE;
    }

    /* Set rawdata */
    Data_obj->rawdata = (gchar*)memset(Data_obj->rawdata,0,Data_obj->rawdatalen);

    return TRUE;
}

gboolean area::secure_deadend(glong Node,glong DLid, gchar* Password) {
    /* Handles the deadend relation, when a Node can not pass it's data */
    /* TODO: re-implement this funtion */

    //g_message("done deadend..");
    return FALSE;
}

gboolean area::secure_conflictportal(glong child_node,glong parent_node,rType_t relation,glong DLid, gchar* Password) {
    /* logic\engine errors etc will be handled here */

    /* Authorization */
    if (!authorized_dl_node(DLid,Password,child_node)) {
        return FALSE;
    }

    if (!authorized_dl_node(DLid,Password,parent_node)) {
        return FALSE;
    }
    
    /* Handles conflict relations */
    switch (relation) {
        case SameRT:g_print("SAME conflict, Node tries to (re)define itself.\n");
                break;
        case RedefineRT:g_print("REDEFINE conflict, an existing relation is redefined.\n");
                break;
        case ChainEnd_StructRT:
                g_print("DEADEND conflict, %s has no child to offer data to.\n",secure_node_get_dname(parent_node,DLid,Password)->str);
                /* call deadend handler */
                secure_deadend(child_node,DLid,Password);
        break;
        default:g_print("Unknown conflict.\n");
    }

    /*
     * TODO: handle more ... 
     */

    /* Conflics handled OK */
    return TRUE;
}

