/*
 * messaging database or DFP (DataFlow Processor) , 
 * begin 20-6-1999, BastardAI
 *		 10-7-1999, B., started sensor-pipeline-collector-processors-visuals
 *						implementation. 
 *						- The pipeLineMO will be a db member all
 *						sensors and collectors will be communicating thru. 
 *						- collector puts a stream into the pipelinei by
 *						by calling messaging_db_pipeline_in(messaging_object *MO)
 *						NOTE this defines all sensors
 *						sensor(NEUTRAL)-pipeLine(ALLOWING). They are not central
 *						connected to a MO, but the pipeLineMO accept all input.
 *						- The data on the pipeline is offered to all collectors
 *						Collectors are connected to pipeLineMO and have a
 *						pipeLine(NEUTRAL)-collector(ALLOWING) relation.
 *						One, or more collectors proces the data, or else the
 *						non-processed stream will end up in the
 *						messaging_db_visual_error(messaging_object *MO) void.
 *						- Processors are like collectors, but they do not have
 *						pipeline access. Processors are not central connected.
 *						- Visuals receive streams and "display" them, like
 *						the messaging_db_visual_logfile_all() function. Visuals
 *						are connected to visualMO.
 *		21-7-1999, B., added pcre library to filter-code. It has perl5 like
 *						regular expression functions.
 *      23-7-1999, Ba.-AI, first prototype finished. The DB is 90% operational,
 *    					only the filter-ckecking code always agrees.
 *		28-7-1999, B.,	Done db filtering, user configuration of dead end streams,
 *						lots of enhancements. DB core done 100%.
 *      28-9-2000, Bstard, Ported GMS to C++ and became the BL in the Piper project
 *
 * -> continued into the changeLog.txt
 */

/*
 * INCLUDES
 */
#include "messaging_config.h"
#include "messaging_engine.h"
#include <stdlib.h>
#include <fstream.h>
#include <string.h>
#include <stdio.h>
#include <time.h>
#include <sys/time.h>
#include <unistd.h>

#include <glib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stddef.h>
#include <regex.h>
#include <fnmatch.h>
#include <gmodule.h>
#include <dirent.h>
#include <signal.h>

#ifdef PIPER_ENABLE_THREADS
	#include <pthread.h>
#endif


/*
 * FUNCTIONS
 */

gboolean engine::build_Piper_Paths(glong DLid, gchar* Password) {
    /* This function builds the relations between plugins that */
    /* are needed for GMS to functions as a BL */
    gboolean result = TRUE;
    glong Node;

    /* Link PL sensor as child to DL visual */
    result = result && secure_relate_dname_to_dname("DL plugin visual","PL plugin sensor",DLid,Password);
    /* Link PL visual as child to PL collector */
    result = result && secure_relate_dname_to_dname("PL plugin sensor Collector","PL plugin visual",DLid,Password);
    /* Link DL sensor as child to PL visual */
    result = result && secure_relate_dname_to_dname("PL plugin visual","DL plugin sensor",DLid,Password);
    /* Link DL visual as child to DL collector */
    result = result && secure_relate_dname_to_dname("DL plugin sensor Collector","DL plugin visual",DLid,Password);

    return result;
}


gboolean engine::rules(gint mode,glong DLid,gchar* Password) {
    /* Main function for adding hardcoded node logic */
    gboolean result = FALSE;

    switch (mode) {
        case 1:         result = build_Piper_Paths(DLid,Password);
                        break;
        default:        break;
    }
    if (result==FALSE) {
        g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_CRITICAL,"Applying rules failed, could not setup the configuration for Piper Mode");
    } else {
        g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_INFO,"Piper mode setup done");
    }

    return result;
}


gpointer engine::gtkplugdestroy(GtkWidget *plug,gpointer user_data) {
    gtk_widget_unrealize((GtkWidget*)user_data);
    return NULL;
}


gpointer engine::gtkplugdestroyevent(GtkWidget *plug,gpointer user_data) {
    return NULL;
}


GtkWidget* engine::gtkplugnew(gint XID,const gchar *text) {
    /* Allocate new gtkplug */
    GtkWidget *plug;

    if (XID==0) 
        return NULL;
    plug = gtk_plug_new(XID);
    //gtk_signal_connect(GTK_OBJECT(plug),"destroy",GTK_SIGNAL_FUNC(gtkplugdestroy),plug);
    //gtk_signal_connect(GTK_OBJECT(plug),"destroy-event",GTK_SIGNAL_FUNC(gtkplugdestroyevent),plug);
    return plug;
}


gint engine::regexpmatch(gchar *string, gchar *filter) {
    /* (char*) filter is a Perl5.005 regular expression 
     * UPDATE: killall -9 libpcre, it behaved like smegma.
     *         glibc is temporary used, it's very limited however, I
     *		   hope to find some Perl-functionality sometime... 
     * TODO: implement gnome-regexp stuff 
     * SO: 	returns TRUE if {filter} = substring of {string} 
     */
    regex_t preg;
    regmatch_t *pmatch;
    gint result;

    pmatch = (regmatch_t*)g_malloc(sizeof(regmatch_t));
    result = regcomp(&preg,filter,0);
    result = !regexec(&preg,string,1,pmatch,0);

    regfree(&preg);
    g_free(pmatch);
    return result;
}


gint engine::alarmGETsocketXID(void) {
    return dfpadmref->alarmGETsocketXID();
}


gint engine::statusbarGETsocketXID(void) {
    return dfpadmref->statusbarGETsocketXID();
}


int dirfilter_shared(const struct dirent *entry) {
    return 1;
}

void engine::processor(gpointer Nodeparentchild,glong DLid,gchar* Password) {
    /*
     * This function is called internally after an adjustment to a DBMO
     * It will process all the DBMO's, it will start with the pipeLineNode,
     * the one that receives all plugin streams. First all Collectors are
     * processed, 2ndly their 1st generation childs, etc.. until all
     * streams ended into a Visual. NOTE that plugins can register 
     * visuals, so they can remove streams outoff the engine. Regular
     * operation is having plugins only register a sensor and a collector
     * and let the user configuration handle visuals.
     *
     */
    glong parent_node = ((MOpnc*)Nodeparentchild)->parent;
    glong child_node = ((MOpnc*)Nodeparentchild)->child;
    glong mdo;
    gint i;
    gint currentRelationsNumber = secure_node_get_childs(child_node,DLid,Password);
    MOpnc *mopnc[currentRelationsNumber];

    /* Display debug line */
    g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_DEBUG,"Processor, parent = %s, child = %s",secure_node_get_dname(parent_node,DLid,Password)->str,secure_node_get_dname(child_node,DLid,Password)->str);

    /* Is the BL shutting down? */
    if (shuttingdown) {
        g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_WARNING,"System is shutting down, processing canceled");
        secure_unlock("messaging_db_processor",DLid,Password);
        return;
    }

    /* Is parent a sensor? If yes, jump to pipeline_in() */
    if (secure_node_get_nodetype(parent_node,DLid,Password) == SensorNode) {
        g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_DEBUG,"Parent is Pipeline, quitting processor");
        pipeline_in(parent_node,DLid,Password);
        return; // and exit this branch ;) 
    }

    /* Check if parent is allowed to access child */
    if (secure_node_allowes_data(child_node,secure_node_get_data(parent_node,DLid,Password),DLid,Password)==FALSE) {
       return;
    }

    /* wait for plugin to become available, cq. not being in use */
    /* wait for MOC->procedure to be fully handled */
    /* wait 4 procedure to end end dataPassedOver to be set */
    if (secure_node_get_procedure_handled(child_node,DLid,Password)==FALSE) {
        g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_DEBUG,"Waiting for Node %s to become available",secure_node_get_dname(child_node,DLid,Password)->str);
        while (secure_node_get_procedure_handled(child_node,DLid,Password)==FALSE) {
           secure_idle(50,DLid,Password);
        }
        g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_DEBUG,"Done waiting for Node %s to become available",secure_node_get_dname(child_node,DLid,Password)->str);
    }

    /* ok, it's free now, claim it ours, data is not passedover by default */
    secure_node_set_procedure_handled(child_node,FALSE,DLid,Password);
    secure_node_set_datapassedover(child_node,FALSE,DLid,Password);

    /* write data to historyIO */
    if (!secure_historyIO_write(child_node,DLid,Password)) {
        // return; ?? exit ??
    }

    /* Is the MOP blocking datastreams? , Accepts MOC from MOP? OR: does the filter allow processing? */
    if ((secure_node_get_status(child_node,DLid,Password)==Blocking) || ((!regexpmatch(secure_data_get_datadescription(secure_node_get_data(parent_node,DLid,Password),DLid,Password)->str,secure_node_get_allowedfilter(child_node,DLid,Password)->str)))) {
        /* not, terminate this branch */
        secure_node_set_procedure_handled(child_node,TRUE,DLid,Password);
        secure_node_set_datapassedover(child_node,TRUE,DLid,Password);
        g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_WARNING,"Child %s didn't accept data from %s",secure_node_get_dname(child_node,DLid,Password)->str,secure_node_get_dname(child_node,DLid,Password)->str);
        return;
    } else {
        /* else copy MOP's data into MOC and set dataPassedOver */
        #ifdef MO_CACHE
        /* cache data. Last 32 datablocks are cached, this will cost a lot of RAM but the to-be-implemented KB will depend on it */
        mdo = secure_data_new(child_node,DLid,Password);
        /* update history */
        secure_data_copy(mdo,secure_node_get_data(child_node,DLid,Password),DLid,Password);
        /* add current MDO at front of list */
H
        secure_node_add_blockedlist(child_node,mdo,DLid,Password);
        #endif

        secure_data_copy(secure_node_get_data(child_node,DLid,Password),secure_node_get_data(parent_node,DLid,Password),DLid,Password);
        /* define dataDescription */
        secure_node_set_datadescription(child_node,secure_node_get_datadescription(parent_node,DLid,Password)->str,DLid,Password);
    }

    /* does the member has any operation to do? */
    if (secure_node_get_procedure(child_node,DLid,Password) != NULL) {
        /* TODO: mutext this flag */
        secure_node_set_procedure_running(child_node,TRUE,DLid,Password);
        /* execute procedure */
        //res = pthread_create(&tree,NULL,MOC->procedure,MOC);
        secure_node_execute_procedure(child_node,DLid,Password);
        secure_node_set_procedure_handled(child_node,FALSE,DLid,Password);
    }

    /* wait 4 procedure to end */
    /* NOTE: plugin->procedure MUST set the procedure_running field to FALSE when it's finished */
    if (secure_node_get_procedure(child_node,DLid,Password) != NULL) { /* if a procedure is available */
        g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_DEBUG,"Waiting for Node %s to become available",secure_node_get_dname(child_node,DLid,Password)->str);
        while (secure_node_get_procedure_running(child_node,DLid,Password) == TRUE) { /* wait 4 flag to become down again */
            secure_idle(50,DLid,Password);
        }
        g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_DEBUG,"Done waiting for Node %s to become available",secure_node_get_dname(child_node,DLid,Password)->str);
    }
    /* process each collector if they accept the offered data */
    while (currentRelationsNumber-->0) {
        /* pass current MOC as the new MOP */
        /* TODO: processing depends upon relation type */
        /* allocate parent\child set */
        mopnc[currentRelationsNumber] = (MOpnc*)g_malloc(sizeof(MOpnc));
        /* NOTE: MOC becomes PARENT in next interation! */
        mopnc[currentRelationsNumber]->parent = child_node;
        mopnc[currentRelationsNumber]->child = secure_node_get_relation(child_node,currentRelationsNumber,DLid,Password);
        mopnc[currentRelationsNumber]->threaded = FALSE;

        g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_DEBUG,"Handling %i th child named %s, child of %s",currentRelationsNumber,secure_node_get_dname(mopnc[currentRelationsNumber]->child,DLid,Password)->str,secure_node_get_dname(mopnc[currentRelationsNumber]->parent,DLid,Password)->str);
        //do thread this? not a.t.m.; pthread_create(&tree[i],NULL,messaging_db_processor,mopnc[currentRelationsNumber]);
        /* set dataPassedOver */
        secure_node_set_datapassedover(parent_node,TRUE,DLid,Password);

        /*
         * Execute processor for this child
         */
        processor(mopnc[currentRelationsNumber],DLid,Password);
    }

    /* handle dead end streams (visuals are deadend by definition, so treat them special) */
    /* NOTE: visuals should set the dataPassedOver flag, but this logic can handle ugly visuals */
    if ((secure_node_get_datapassedover(child_node,DLid,Password)==FALSE)&&(secure_node_get_nodetype(child_node,DLid,Password))) {
        g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_MESSAGE,
              "Handle dead end streams; dataPassedOver = %i  Node = %s",
              secure_node_get_datapassedover(child_node,DLid,Password),
              secure_node_get_dname(child_node,DLid,Password)->str);
        secure_conflictportal(child_node,-1,ChainEnd_StructRT,DLid,Password);
    }
    /* reset so plugin can be used again */
    secure_node_set_datapassedover(child_node,TRUE,DLid,Password);
    secure_node_set_procedure_handled(child_node,TRUE,DLid,Password);

    /* exit, methode based on entry type */
    if (((MOpnc*)Nodeparentchild)->threaded == TRUE) {
        g_free(Nodeparentchild);
        #ifdef PIPER_THREADS_ENABLED
        /* exit thread */
        pthread_exit(0);
        #endif
    } else {
        g_free(Nodeparentchild);
        return;
    }

    /* for smartass compilers */
    return;
}

gboolean engine::pipeline_in(glong Node,glong DLid,gchar* Password) {
    /* 
     * Sensors call this function to have their data put into the db
     */
    gint i;
    guint bytes_written;
    glong node_data = secure_node_get_data(Node,DLid,Password);
    glong parent_data;
    glong pipeLineCloon;
    /* TODO: make LList */ MOpnc *Nodeparentchild[256];
    #ifdef PIPER_THREADS_ENABLED
    /* TODO: make LList */ pthread_t tree[256];
    gint res[256];
    #endif

    /* Does the Sensor node has a procedure operation to do? */
    if (secure_node_get_procedure(Node,DLid,Password) != NULL) {
        /* TODO: mutext this flag */
        secure_node_set_procedure_running(Node,TRUE,DLid,Password);
        /* execute procedure */
        secure_node_execute_procedure(Node,DLid,Password);
        secure_node_set_procedure_handled(Node,FALSE,DLid,Password);
    }

    /* Mark data block's originator fields (originatorID and datadescription) */
    secure_data_set_originatorID(node_data,Node,DLid,Password);
    secure_data_set_datadescription(node_data,secure_node_get_datadescription(Node,DLid,Password)->str,DLid,Password);

    /* Create a temporary Pipeline for this run */
    pipeLineCloon = secure_node_cloon(pipeLine,DLid,Password);
    secure_node_set_datapassedover(pipeLineCloon,FALSE,DLid,Password);

    /* has the sensor access to the local BL instance? 
       If not, something is very wrong, all sensors 
       should have pipeline access per say */
    if (!secure_node_allowes_data(pipeLineCloon,node_data,DLid,Password)) {
        g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_ERROR,"Node %s has no access to the Pipeline. Something is very wrong, ALL sensors should be able to access the pipeline",secure_node_get_dname(Node,DLid,Password)->str);
        return FALSE;
    }

    /* is sensor blocked? ifso do not process */
    if (secure_node_get_status(Node,DLid,Password) == Blocking) {
        g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_INFO,"Node %s is Blocked, pipeline_in canceled",secure_node_get_dname(Node,DLid,Password)->str);
        return FALSE;
    }

    /* is pipeline accepting data? */
    if (shuttingdown) {
       g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_INFO,"System is shutting down, pipeline_in canceled");
       return FALSE;
    }

    /* is the dataDescription of the data block empty? */
    GString *datadescription = secure_node_get_datadescription(Node,DLid,Password);
    if (datadescription == NULL) {
        g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_WARNING,"Could not retrieve datadescription of node, pipeline in has been canceled");
        return FALSE;
    }
        
    /* ?? Clashes with check data->description == node->datadescription ?? */
    if (datadescription->len == 0) {
        g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_INFO,"Filled data block description with node description");
        secure_data_set_datadescription(node_data,secure_node_get_datadescription(Node,DLid,Password)->str,DLid,Password);
    }

    /* does the dataDescription of the sensor node match it's data block? */
    if (!regexpmatch(datadescription->str,secure_data_get_datadescription(node_data,DLid,Password)->str)) {
        g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_INFO,"Consistency mismatch: Sensor node %s has description %s, which does not equal description %s of data block %i, transaction canceled",secure_node_get_dname(Node,DLid,Password)->str,secure_node_get_datadescription(Node,DLid,Password)->str,secure_data_get_datadescription(node_data,DLid,Password)->str,secure_data_get_id(node_data,DLid,Password));
        return FALSE;
    }


    /* --- PIPELINE ACCEPTS DATA BLOCK --- */
    g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_MESSAGE,"%s passes data to pipeline",secure_node_get_dname(Node,DLid,Password)->str);

    /* copy data into pipeline AND clean sensor data block */
    secure_data_copy(secure_node_get_data(pipeLineCloon,DLid,Password),node_data,DLid,Password);
    secure_data_clear(node_data,DLid,Password); /* clean input memory */
    secure_node_set_procedure_handled(Node,TRUE,DLid,Password); /* Unlock Sensor */
    
    secure_typecast(pipeLineCloon,DLid,Password);

    /* write data to historyIO */
    /* TODO: only write 1st non-cached MDO */
    /* ??? Node always is pipeline ??? */
    if ((Node != pipeLineCloon) && (Node != visual_central)) {
        secure_historyIO_write(Node,DLid,Password);
    }

    /* process each collector if they accept the offered data */
    /* NOTE: messaging_db_processor does do a complete tree */
    /* NOTE2: trees after the pipeline MO are threaded */
    g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_DEBUG,"Pipeline has %i childs, offering description %s",secure_node_get_childs(pipeLineCloon,DLid,Password),secure_data_get_datadescription(secure_node_get_data(pipeLineCloon,DLid,Password),DLid,Password)->str);
    for (i = secure_node_get_childs(pipeLineCloon,DLid,Password)-1;i>-1;i--) {
        Nodeparentchild[i] = (MOpnc*)g_malloc(sizeof(MOpnc));
        Nodeparentchild[i]->parent = pipeLineCloon;
        Nodeparentchild[i]->child = secure_node_get_relation(pipeLineCloon,i,DLid,Password);
        g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_DEBUG,"Handling %i th child %s with allow filter %s",i,secure_node_get_dname(Nodeparentchild[i]->child,DLid,Password)->str,secure_node_get_allowedfilter(Nodeparentchild[i]->child,DLid,Password)->str);
        parent_data = secure_node_get_data(Nodeparentchild[i]->parent,DLid,Password);
        // Check if Collector accepts MDO:
        if (parent_data < 0 ) {
            //Not ok
        } else 
        if (regexpmatch(secure_data_get_datadescription(parent_data,DLid,Password)->str,secure_node_get_allowedfilter(Nodeparentchild[i]->child,DLid,Password)->str)) {
            secure_node_set_datapassedover(pipeLineCloon,TRUE,DLid,Password);
            #ifdef PIPER_THREADS_ENABLED
            Nodeparentchild[i]->threaded = TRUE;
            res[i] = pthread_create(&tree[i],NULL,messaging_db_processor,Nodeparentchild[i]);
            #else
            Nodeparentchild[i]->threaded = FALSE;
            /* Jump to main processing code */
            processor(Nodeparentchild[i],DLid,Password);
            #endif
        } else {
            //nop
        }
    }

    /* clean sensor data block */
    secure_data_clear(secure_node_get_data(Node,DLid,Password),DLid,Password);

    /* handle dead end streams, when no collector has accepted data block  */
    if (secure_node_get_datapassedover(pipeLineCloon,DLid,Password)==FALSE) {
        g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_MESSAGE,
              "Handle dead end streams; dataPassedOver = %i  Node = %s",
              secure_node_get_datapassedover(pipeLineCloon,DLid,Password),
              secure_node_get_dname(pipeLineCloon,DLid,Password)->str);
        secure_conflictportal(pipeLineCloon,-1,ChainEnd_StructRT,DLid,Password);
    } else 
        g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_MESSAGE,"Pipeline processing done");


    /* Clean & Exit */
    secure_node_delete(pipeLineCloon,DLid,Password);
    return TRUE;
}

GtkWidget *visual_MO_init_configPanel() {
	/* The init_configPanel() service of the visual_MO */
	GtkWidget *label;

	label = gtk_label_new ("This is a system component\nand can not be configured");
	gtk_widget_set_name (label, "label");
	if (label == NULL)
		g_print("Visual_MO_init_configPanel ERROR\n");
	else
		g_print("Visual_MO_init_configPanel OK\n");
	return label;
}   

gboolean engine::registerPLUGIN2DL(glong Node,glong DLid,gchar* Password) {
    gint XID = 0;
    if (Node > 0) {
        if ((secure_node_get_nodetype(Node,DLid,Password) == VisualNode) && (secure_node_get_visualplug_buildprocedure(Node,DLid,Password)!=NULL)) {
            /* Setup visualSocket; request the ADM for a socket and store it in the MO-struct */
            XID = dfpadmref->visualGETsocketXID(secure_node_get_name(Node,DLid,Password));
            if (XID>0) {
                secure_node_execute_visualplug_buildprocedure(Node,XID,DLid,Password);
                secure_node_set_uisetup(Node,TRUE,DLid,Password);
                secure_node_set_status(Node,Passing,DLid,Password);
                return TRUE;
            }
        } else {
            /* no visual socket requered, set to passing */
            secure_node_set_uisetup(Node,TRUE,DLid,Password);
            secure_node_set_status(Node,Passing,DLid,Password);
        }
    }
    return FALSE;
}

void* engine::runADMclientORB(gpointer data) {
	g_print("Running the BL_admin client ORB\n");
	DFPADM_corba_orb->run();
	g_print("Destroying the BL_admin client ORB\n");
	DFPADM_corba_orb->destroy();
	return 0;
}

// gboolean engine::bind2DL() {
// 	/*
// 	 * Binds as client to GMS UI\ADM
// 	 */
// 	FILE *ifp;
// 	gchar *ior;
// 	gchar filebuffer[1024];
// 	GString *iorFile;
// 	GList *element;
// 	node *NODE_LIST;
//	gint argc = 1;
//	gchar *argv[1];
//	/* build dummy args */
//    argc = 1;
//    argv[0] = (gchar*)"BL";
//	gint ADMclient_ORB_thread;
//	pthread_t ADMclient_ORB_tree;
//
//	g_print("Trying to connect to BL_admin...\n");
//	    /* define IOR file location */
//    iorFile = g_string_new(getenv("HOME"));
//    iorFile = g_string_append(iorFile, "/piper_info");
//    mkdir(iorFile->str,S_IRWXU);
//    iorFile = g_string_append(iorFile, "/ior");
//    mkdir(iorFile->str,S_IRWXU);
//    iorFile = g_string_append(iorFile, "/bl2admin.ior");
//
//	/* read ior file */
//	ifp = fopen(iorFile->str,"r");
//	if( ifp == NULL )
//	{
//		g_print("No IOR file found..\n");
//		g_string_free(iorFile,TRUE);
//		return FALSE;
//	}
//	fgets(filebuffer,1024,ifp);
//	ior = g_strdup(filebuffer);
//	//g_print("IOR = %s\n",ior);
//	fclose(ifp);
//	/* corba bind */
//      try {
//		DFPADM_corba_orb = CORBA::ORB_init(argc, argv, "omniORB3");
//        DFPADM_corba_object = DFPADM_corba_orb->string_to_object(ior);
//        dfpadmref = DFPADM::_narrow(DFPADM_corba_object);
//		if( CORBA::is_nil(dfpadmref) ) {
//			cerr << "Can't narrow reference to type DFPADM (or it was nil)." << endl;
//			bound2DL = FALSE;
//			return 1;
//        }
//		ADMclient_ORB_thread = pthread_create(&ADMclient_ORB_tree,NULL,(void* (*)(void*))(runADMclientORB(NULL)),NULL);
//      }
//      catch(CORBA::COMM_FAILURE& ex) {
//        cerr << "Caught system exception COMM_FAILURE -- unable to contact the "             << "object." << endl;
//      }
//      catch(CORBA::SystemException&) {
//        cerr << "Caught a CORBA::SystemException." << endl;
//      }
//      catch(CORBA::Exception&) {
//        cerr << "Caught CORBA::Exception." << endl;
//      }
//      catch(omniORB::fatalException& fe) {
//        cerr << "Caught omniORB::fatalException:" << endl;
//        cerr << "  file: " << fe.file() << endl;
//        cerr << "  line: " << fe.line() << endl;
//        cerr << "  mesg: " << fe.errmsg() << endl;
//      }
//      catch(...) {
//        cerr << "Caught unknown exception." << endl;
//      }
//
//		if (!dfpadmref) {
//			g_message("DFP cannot bind to %s\n", ior);
//		} else {
//			//g_print("Bound to ADM  %i\n",dfpadmref->visualGETconfigXID());
//			/* request output panels on ADM for the visuals */
//
//			//element = NODE_LLIST;
//            j = secure_node_get_nodes(DLid,Password);
//            for (i=0;i<j;i++) {
//			//while (element->next != NULL) {
//				NODE_LIST = (node*)element->data;
//				if (NODE_LIST->moType == VisualMO) {
//					if (strcmp(NODE_LIST->dname->str,"visual APPLET MO")==0) {
//						/* dock special visual alarm into ADM */
//						NODE_LIST->visualPlug_buildProcedure(0);
//						NODE_LIST->UIsetup = TRUE;
//						NODE_LIST->status = Passing;
//					} else 
//						if (strcmp(NODE_LIST->dname->str,"visual STATUSBAR MO")==0) {
//						   /* dock special visual statusbar into ADM */
//							NODE_LIST->visualPlug_buildProcedure(0);
//							NODE_LIST->UIsetup = TRUE;
//							NODE_LIST->status = Passing;
//						} else 
//							registerPLUGIN2DL(NODE_LIST);
//				}
//				element = g_list_next(element);
//			}
//			bound2DL = TRUE;
//			shuttingdown = FALSE;
//		}
//	g_string_free(iorFile,TRUE);
//	secure_unlock("messaging_db_bind2DL",DLid,Password);
//	return bound2DL;
//}
//
//gboolean engine::is_bound2DL() {
//	/* ADM corba calls must depend on returned value */ 
//	return bound2DL;
//}
//
//gint engine_bind2DL_timeout(gpointer data) {
//	/* when DFP isn't bound to ADM try every second to bind */
//    engine *Engine = (engine*)data;
//	if (Engine->bound2DL==TRUE) {
//		//g_message("Already bound, releasing");
//	    Engine->bound2DL = FALSE;
//		/* unlock pipeline */
//		Engine->shuttingdown = FALSE;
//	    /* NOTE: the corba-client is finished after the client called REGISTERadm */
//		Engine->secure_unlock("messaging_db_bind2DL_timeout",ENGINE_DL_ID,ENGINE_DL_PASSWORD);
//		/* terminate timer */
//		return FALSE;
//	}
//	return (!Engine->bind2DL());
//}

void* messaging_db_startgtk(gpointer data) {
	gtk_main();
	return NULL;
}

gint messaging_db_loop() {
	/* idle loop - unused a.t.m. */
	g_print("Entered idle loop\n");
	g_main_run(g_main_new(FALSE));
	return 0;
}

void messaging_keep_compiler_happy(void) {
	/* gcc 2.95 with -Wall complains about unused functions */
	#ifdef VSHMODE
	impl_vsh_dl2bl_Representation__create(NULL,NULL);
	impl_vsh_dl2bl_Obtain__create(NULL,NULL);
	#endif
	return;
}
