#include "messaging_resource.h"
#include "messaging_config.h"
#include "messaging_engine.h"

/*
 * Register\Login a DL
 * still very basic func
 */
gboolean resource::register_dl(glong dlID,gchar *password,glong mdlID,gchar *password_area) {
    /* TODO Setup dlId list if not done already */

    /* TODO Add a valid-clientID lookup list */

    if (dlID>0 && dlID<MAXDLID) {
        if (DLS[dlID]==NULL) {
            DLS[dlID] = (messaging_security_DL*)g_malloc(sizeof(messaging_security_DL));            DLS[dlID]->DLid = dlID;
            DLS[dlID]->password = g_string_new(g_strdup(password));
            DLS[dlID]->MasterDLid = mdlID;
            DLS[dlID]->nodes = NULL;
            g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_INFO,"Accepted DL for Area %i",dlID);
        } else {
            g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_INFO,"DL %i was not accepted because the ID is already taken",dlID);
            return FALSE;
        }
    } else {
        g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_INFO,"DL %i was not accepted because the ID is not within the valid range",dlID);
        return FALSE;
    }

    /* 1st DL registered? */
    if (dlID == 1) {
        /* we've got a DL */
        any_dl = TRUE;
    }

    return TRUE;
}

gboolean resource::setup_nodes(gpointer Engine) {
    /* load all availeble plugins */
    load_plugins(Engine);
    /* load previous situation */
    //DISABLED, UNSTABLE: load_state(Engine);
    /* activate all loaded plugins */
    activate_nodes();
    /* save current situation */
    save_state();

    return TRUE;
}

/*
 * unregister DL (logout)
 */
gboolean resource::unregister_dl(glong dlID,gchar *password) {
    /* basic stuff */
    if (dlID>0 && dlID<MAXDLID) {
        if (DLS[dlID] != NULL) {
            g_free(DLS[dlID]);
            DLS[dlID] = NULL;
            g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_INFO,"Released DL %i",dlID);
            return TRUE;
        } else {
            g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_INFO,"DL %i was not released because it was never accepted",dlID);
            return FALSE;
        }
    } else {
        g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_INFO,"DL %i was not released because the ID is outside the valid range",dlID);
        return FALSE;
    }
    return FALSE;
}

gboolean resource::data_id_lock(data* Data) {
    glong id = -1;
    glong i = 1;

    BL_lock("resource::data_id_lock");
    while (i<DATA_MAXIMUM_OBJECTS && id==-1) {
        if (DATA_LOCKED_LIST[i]==NULL) {
            DATA_LOCKED_LIST[i] = Data;
            id = i;
        }
        i++;
    }
    if (id==-1) {
        // oops, run out of id's
        g_error("oops, run out of data id's");
    }

    /* Set data id */
    Data->ID = id;

    /* Add data to total data list */
    DATAS = g_list_append(DATAS,Data);

    BL_unlock("resource::data_id_lock");
    return TRUE;
}

gboolean resource::data_id_unlock(data* Data) {
    glong id = Data->ID;

    BL_lock("resource::data_id_unlock");
    /* id valid value? */
    if (id<1 || id>DATA_MAXIMUM_OBJECTS) {
        g_warning("dwoink, wont unlock invalid data id");
        return FALSE;
    }
    /* was id previously locked? */
    if (DATA_LOCKED_LIST[id]==NULL) {
        g_warning("cant unlock data id %i because it never was locked");
        return FALSE;
    }

    /* Remove data from total data list */
    DATAS = g_list_remove(DATAS,Data);
    /* unlock id */
    DATA_LOCKED_LIST[id] = NULL;

    BL_unlock("resource::data_id_unlock"); 
    return TRUE;
}

gboolean resource::node_id_lock(node* Node) {
    glong id = -1;
    glong i = 0;
    gchar* Name = Node->dname->str;
    gchar* AllowedFilter = Node->allowedFilter->str;
 
    BL_lock("resource::node_id_lock");
    /* Find free id */
    while (i<NODE_MAXIMUM_OBJECTS && id==-1) {
        if (NODE_LOCKED_LIST[i]==NULL) {
            NODE_LOCKED_LIST[i] = Node;
            id = i;
        }
        i++;
    }

    /* No free id left? */
    if (id==-1) {
        // TODO: remove this limitation
        g_error("OOPS! Ran out of node id's. This isn't a production version of the BL, check for upgrades");
    }
    
    /* Add node to total node list */
    NODES = g_list_append(NODES,Node);

    /* Update total nodes */
    Nodes++;
    /* Set node id */
    Node->uri->nodeID = id;

    BL_unlock("resource::node_id_lock");
    return TRUE;
}

gboolean resource::node_id_unlock(node* Node) {
    glong id = Node->uri->nodeID;

    BL_lock("resource::node_id_unlock");
    /* id valid value? */
    if (id<0 || id>NODE_MAXIMUM_OBJECTS) {
        g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_WARNING,"Wont unlock invalid node id %i",id);
        return FALSE;
    }
    /* was id previously locked? */
    if (NODE_LOCKED_LIST[id]==NULL) {
        g_warning("cant unlock node id %i because it never was locked");
        return FALSE;
    }

    /* Remove node from total node list */
    NODES = g_list_remove(NODES,Node);
    /* unlock id */
    NODE_LOCKED_LIST[id] = NULL;
    /* update hash tables */
    g_hash_table_remove(NODE_DNAME_HASH,Node->dname->str);
    g_hash_table_remove(NODE_FILTER_HASH,Node->allowedFilter->str);
    /* Update total nodes */
    Nodes--;
    /* Check total nodes */
    if (Nodes<0) {
        g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_CRITICAL,"Total number of nodes seems to be %i",Nodes);
    }
    g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_DEBUG,"Unlocked node %i",Node->uri->nodeID);

    BL_unlock("resource::node_id_unlock");
    return TRUE;
}

node* resource::node_lookup_by_dname(gchar* Dname) {
    glong *Node;
    Node = (glong*)g_hash_table_lookup(NODE_DNAME_HASH,Dname);

    if (Node == NULL) {
        g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_WARNING,"Node with dname %s is not available",Dname);
        return NULL;
    } else {
        return noderef_to_node(*Node);
    }
}

node* resource::node_lookup_by_filter(gchar* Filter) {
    glong *Node;
    Node = (glong*)g_hash_table_lookup(NODE_FILTER_HASH,Filter);

    if (Node == NULL) {
        g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_WARNING,"Node with filter %s is not available",Filter);
        return NULL;
    } else {
        return noderef_to_node(*Node);
    }
}

gboolean resource::register_node(gint DLid,node* Node,gpointer Engine) {
    /* Add node to node pool and activate it */
    gboolean result = TRUE;
    messaging_security_DL *MSDL;
    
    /* Get node list of this area */
    if (DLid<0 || DLid>MAXDLID) {
        g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_WARNING,"Cant register node to non exsisting DL");
        return FALSE;
    } else {
       MSDL = DLS[DLid];
    }

    if (MSDL==NULL) {
        g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_ERROR,"Cant register to area %i because it doesn't exsists",DLid);
        result =  FALSE;
    } else {
        /* Add noderef to area list */
        MSDL->nodes = g_list_append(MSDL->nodes,Node);
        g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_INFO,"Registered node %s to area %i",Node->dname->str,DLid);
        /* Update lookup lists */
        g_hash_table_insert(NODE_DNAME_HASH,Node->dname->str,&(Node->uri->nodeID));
        g_hash_table_insert(NODE_FILTER_HASH,Node->allowedFilter->str,&(Node->uri->nodeID));
        /* open history IO */
        if (!historyIO_open(Node)) {
            return FALSE;
        }
        /* init node */
        if ( Node->init_ref == NULL || Node->init_ref(node_to_noderef(Node),(engine*)Engine) != TRUE ) {
            g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_WARNING,"Node %s failed to initialise",Node->dname->str);
            result = FALSE;
        } else {
            g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_MESSAGE,"Node %s initialised",Node->dname->str);
        }
    }

    return result;
}


gboolean resource::unregister_node(node* Node) {
    /* unregister a sensor */
    /* TODO: add auth checking */
    /* NOTE : Do not destruct MOS here, just deactivate */
    /* NOTE2: Destructing a MO will call this function too */
    historyIO_close(Node);
    //node_delete(Node);

    return TRUE;
}

node* resource::noderef_to_node(glong Node) {
    /* Returns node based on node referance */

    /* Check Node */
    if ((Node<-1) || (Node>NODE_MAXIMUM_OBJECTS)) {
        g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_WARNING,"Could not find node for nodeID %i",Node);
        return NULL;
    }

    return NODE_LOCKED_LIST[Node];
}

glong resource::node_to_noderef(node* Node) {
    /* Returns referance to node */

    /* Check Node */
    if (Node == NULL) {
        g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_WARNING,"Can not get referance for a NULL node");
        return -1;
    }
    
    return Node->uri->nodeID;
}

node* resource::node_get_relation(node* Node,glong relation_nr) {
    /* returns the (relation)th child of node */
    node *relation_node;

    /* Check */
    if (Node == NULL) {
        g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_WARNING,"Cant work with NULL nodes");
        return NULL;
    }
    if ( (relation_nr<0) || (relation_nr>g_list_length(Node->relationList)) ) {
        g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_WARNING,"Cant get %lith relation of node %s",relation_nr,Node->dname->str);
        return NULL;
    }


    /* get (relation)th child of node */
    relation *Relation = (relation*)g_list_nth_data(Node->relationList,relation_nr);
    relation_node = noderef_to_node(Relation->Node);

    return relation_node;
}

glong resource::node_get_currentrelations(node* node) {
    /* returns number of nodes that have node as their child,
       in other words: how many nodes have a relation to this node */

    return node->externalRelations;
}

glong resource::node_get_childs(node* node) {
    /* returns number of childs of node */

    return g_list_length(node->relationList);
}

gpointer resource::node_get_activate(node* node) {
    /* Returns activate of node */
    return (gpointer)node->activate_ref;
}

GString* resource::node_get_datadescription(node* node) {
    /* Returns datadescription of node */
    return node->dataDescription;
}

glong resource::nodes(void) {
    /* Returns number of active nodes in node pool */
    /* TODO: just return nodes in a sigle zone?
    /* NOTE: node_id_lock and node_id_unlock keep track of Nodes */
    return Nodes;
}

data* resource::dataref_to_data(glong Data) {
    /* Returns data object based on data referance */
    data* DataObj;

    /* Check Data */
    if ((Data<-1) || (Data>DATA_MAXIMUM_OBJECTS)) {
        g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_WARNING,"Can not return data for dataref %i",Data);
        return NULL;
    }

    DataObj = DATA_LOCKED_LIST[Data];
    return DataObj;
}

glong resource::data_to_dataref(data* Data) {
    /* Returns dataref for data object */

    if (Data == NULL) {
       g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_WARNING,"Can not get referance for a NULL data");
       return -1;
    } else {
       return Data->ID;
    }
}
 
gboolean resource::node_add_blockedlist(node* Node,data* Data) {
    /* Adds Data to begin of datablockedlist of Node */
    GSList *element1;

    /* Logic check */
    #ifndef MO_CACHE
    g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_WARNING,"MO CACHE is disbaled, resource::node_add_blockedlist shouldn't be called");
    return FALSE;
    #else
    /* remove elements above MO_CACHE_ELEMENTS */
    while (g_slist_length(Node->dataBlockedList) >= MO_CACHE_ELEMENTS) {
        element1 = g_slist_last(Node->dataBlockedList);
        Data = (data*)element1->data;
        Node->dataBlockedList = g_slist_remove(Node->dataBlockedList,element1->data);
        data_delete(Data);
    }

    return TRUE;
    #endif
}

gboolean resource::relate_node_to_node(node* parent,node* child) {
    /* Relate two NODE's */
    gint i;
    relation *Relation = new relation();

    /* Check */
    if (parent == NULL || child == NULL) {
        g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_WARNING,"Cant relate undefined nodes");
        return FALSE;
    }

    /* Parent and Child are the same node? */
    if (parent->uri->nodeID == child->uri->nodeID) {
        g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_WARNING,"Parent and Child node are the same (ID=%i, dname=%s), canceled adding of relation",parent->uri->nodeID,parent->dname->str);
        return FALSE;
    }

    for (i=node_get_currentrelations(parent);i>0;i--) {
        if (node_get_relation(parent,i-1) == child) {
        /* The two specified members already have a definition here, the
         * db code should somehow correct the data situation. We'll
         * redirect the conflicting definitions to a KDB
         */
        conflictportal(parent,child,RedefineRT);
        return FALSE;
        }
    }

 
    /* Create relation, make child of parent */
    Relation->Node = node_to_noderef(child);
    Relation->rType = Sequential_StructRT;   /* TODO!!!!!! */
    parent->relationList = g_list_append(parent->relationList,Relation);
    parent->externalRelations++;
    g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_DEBUG,"parent = %s, parent->relationList = %i,  child = %s, child->externalRelations = %i",parent->dname->str,g_list_length(parent->relationList),child->dname->str,child->externalRelations);

    return TRUE;
}

gboolean resource::relate_node_to_filter(gchar *parent_filter,node* Node) {
    /* relate based on filter */
    node* parent_node;

    parent_node = node_lookup_by_filter((gchar*)parent_filter);

    if (parent_node == NULL) { /* found nothing? */
        g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_WARNING,"%s requested link based on filter to %s but none available - Request Cancelled",Node->dname->str,parent_filter);
        return FALSE;
    }

    /* link child to parent */
    relate_node_to_node(parent_node,Node);

    /* build some debug info */
    g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_INFO,"%s linked to %s",Node->dname->str,parent_node->dname->str);

    return TRUE;
}

gboolean resource::relate_node_to_dname(gchar *parent_name,node* Node) {
    node* parent_node;

    parent_node = node_lookup_by_dname((gchar*)parent_name);

    /* no relation found? -> only debug display */
    if (parent_node == NULL) {
        g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_WARNING,"%s requested link based on name to %s but none available - Request Cancelled",Node->dname->str,parent_name);
        return FALSE;
    }

    /* Create relation */
    return relate_node_to_node(parent_node,Node);
}

gboolean resource::relate_dname_to_node(node* Node,gchar* child_name) {
    node* child_node;

    child_node = node_lookup_by_dname((gchar*)child_name);

    /* no relation found? -> only debug display */
    if (child_node == NULL) {
        g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_WARNING,"%s requested link based on name to %s but none available - Request Cancelled",child_name,Node->dname->str);
        return FALSE;
    }

    /* Create relation */
    return relate_node_to_node(Node,child_node);
}

gboolean resource::relate_dname_to_dname(gchar *parent_name,gchar *child_name) {
    /* relate child node based on dname to parent node based on dname */
    node* parent_node;
    node* child_node;

    parent_node = node_lookup_by_dname((gchar*)parent_name);
    child_node = node_lookup_by_dname((gchar*)parent_name);

    /* no relation found? -> only debug display */
    if (parent_node == NULL) {
        g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_WARNING,"A link was requested based on name to %s but none available - Request Cancelled",parent_name);
        return FALSE;
    }
    if (parent_node == NULL) {
        g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_WARNING,"A link was requested based on name to %s but none available - Request Cancelled",child_name);
        return FALSE;
    }

    return relate_node_to_node(parent_node,child_node);
}

gboolean resource::unrelate_node_to_dname(gchar *relation_name,node* Node) {
    /* remove a relation by name */
    node* relation_node;

    relation_node = node_lookup_by_dname((gchar*)relation_name);

    /* no relation found? -> only debug display */
    if (relation_node == NULL) {
        g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_WARNING,"%s requested removal of relation based on name to %s but none available - Request Cancelled",Node->dname->str,relation_name);
        return FALSE;
    }

    /* break relation */
    return unrelate_node_to_node(Node,relation_node);
}

gboolean resource::unrelate_node_to_node(node* parent,node* child) {
    /* remove child node as relation of parent */

    /* TODO: Implement */

    g_error("gboolean resource::unrelate_node_to_node(node* parent,node* child) UNIMPLEMENTED");

    return FALSE;
}

gint resource::load_state(gpointer Engine) {
    /* Loads and sets up a node situation from file */

    GIOChannel *savefile;
    gint fd,i,r,n,records,CR;
    guint bytesread;
    gpointer databuf;
    gint databufMaxlen = 32000;
    GString *tempstring;
    node *temp_node;
    node *buf_node;
    GString *dir;
    rType_t rType;

    /* Define source location */
    dir = g_string_new(getenv("HOME"));
    g_string_append(dir,"/piper_info/save/");
    g_string_append(dir,"BL_status.save");

    /* Create temporary MO for load buffering and db registration */
    databuf = g_malloc(databufMaxlen);
    buf_node = node_new((const gchar*)"temporary loadbuffer mo",(const gchar*)"undefined","",TemporaryNode,0,NULL,NULL,NULL,NULL);
    //register_node(1,buf_node,Engine);
    tempstring = g_string_new("");

    /* Check save */
    fd = open(dir->str,O_RDWR);
    if (fd < 0 ) {
        /* New installation or config deleted */
        g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_INFO,"Nothing to load, setting up default...");
        close(fd);
        node_delete(buf_node);
        g_string_free(tempstring,TRUE);
        g_free(databuf);
        g_string_free(dir,TRUE);
        return FALSE; /* No previous config, quit */
    } else {
        g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_INFO,"Loading configuration");
        savefile = g_io_channel_unix_new(fd);
    }

    /* Read header cq #records */
    /* TODO: Add versioning */
    bytesread = load_record(savefile,(gchar*)&records,sizeof(gint),"Header");

    /* loop load\check\fill\kick plugins */
    /* 0: read saved data and rebuild MO's without doing their relations */
    /* 1: fill MO's relations to excisting MO's, remove referances to non-excistant MO's */
    for (i=0;i<records;i++) {

        /* define MO_TEMP, so search MO by dname for excistance and do MO_TEMP = found_MO */
        memset(databuf,0,databufMaxlen);
        bytesread = load_record(savefile,(gchar*)databuf,databufMaxlen,"Dname");

        /* Link node based on loaded name to buffer node */
        n = relate_dname_to_node(buf_node,(gchar*)databuf);
        if (n==TRUE) {
            /* Linking done, refer temporary node to the linked node */
            temp_node = noderef_to_node(((relation*)(g_list_last(buf_node->relationList)->data))->Node);
        } else {
            node_delete(buf_node);
            buf_node = node_new((const gchar*)"temporary loadbuffer mo",(const gchar*)"undefined","",TemporaryNode,0,NULL,NULL,NULL,NULL);
            temp_node = buf_node; /* This data will be read , but overwritten next iteration */
        }

        /* read into MO_TEMP */
        bytesread = load_record(savefile,(gchar*)temp_node->Data->rawdata,DATA_RAWLEN,"Raw data");
        bytesread = load_record(savefile,(gchar*)&temp_node->Data->rawdatalen,sizeof(gint),"Raw data len");
        bytesread = load_record(savefile,(gchar*)databuf,databufMaxlen,"Data string");
        g_string_sprintf(temp_node->Data->string,"%s",(gchar*)databuf);
        bytesread = load_record(savefile,(gchar*)&temp_node->Data->integer,sizeof(gint),"Data integer");
        bytesread = load_record(savefile,(gchar*)&temp_node->Data->percentage,sizeof(gint),"Data percentage");
        bytesread = load_record(savefile,(gchar*)&temp_node->Data->boolean,sizeof(gint),"Data boolean");

        /* done MDO */
        bytesread = load_record(savefile,(gchar*)&(temp_node->dataSequence),sizeof(gint),"dataSequence");
        memset(databuf,0,databufMaxlen);
        bytesread = load_record(savefile,(gchar*)databuf,databufMaxlen,"dataDescription");
        g_string_sprintf(temp_node->dataDescription,"%s",(gchar*)databuf);
        bytesread = load_record(savefile,(gchar*)&(temp_node->dataPassedOver),sizeof(gint),"dataPassedOver");
        #ifdef MO_CACHE
        bytesread = load_record(savefile,(gchar*)&(n),sizeof(gint),"");
        while (n>0) {
            /* TODO: read into cache, not into primary MDO */
            bytesread = load_record(savefile,(gchar*)temp_node->Data->rawdata,DATA_RAWLEN,"");
            bytesread = load_record(savefile,(gchar*)&temp_node->Data->rawdatalen,sizeof(gint),"");
            bytesread = load_record(savefile,(gchar*)databuf,databufMaxlen,"");
            g_string_sprintf(temp_node->Data->string,"%s",(gchar*)databuf);
            bytesread = load_record(savefile,(gchar*)&temp_node->Data->integer,sizeof(gint),"");
            bytesread = load_record(savefile,(gchar*)&temp_node->Data->percentage,sizeof(gint),"");
            bytesread = load_record(savefile,(gchar*)&temp_node->Data->boolean,sizeof(gint),"");
            n--;
        }
        #endif
        memset(databuf,0,databufMaxlen);
        bytesread = load_record(savefile,(gchar*)databuf,databufMaxlen,"historyFilename");
        g_string_sprintf(temp_node->historyFilename,"%s",(gchar*)databuf);
        /* no historyIO, this will be read during runtime */
        memset(databuf,0,databufMaxlen);
        bytesread = load_record(savefile,(gchar*)databuf,databufMaxlen,"dDescription");
        g_string_sprintf(temp_node->dDescription,"%s",(gchar*)databuf);
        bytesread = load_record(savefile,(gchar*)&(temp_node->nodeType),sizeof(nodeType_t),"nodeType");
        g_string_sprintf(tempstring,"%s",temp_node->nodetype_to_char(temp_node->nodeType));
        bytesread = load_record(savefile,(gchar*)&(CR),sizeof(gint),"number of relations");
        for (r=0;r<CR;r++) {
            /* Fill relations list, do not relate automatically, 
             * but let have relate_by_node do it */
            memset(databuf,0,databufMaxlen);
            bytesread = load_record(savefile,(gchar*)databuf,databufMaxlen,"relation dname");
            n = relate_node_to_dname((gchar*)databuf,temp_node);
            if (n==TRUE)
                bytesread = load_record(savefile,(gchar*)&((relation*)g_list_nth_data(temp_node->relationList,r))->rType,sizeof(rType_t),"relation type");
            else
                bytesread = load_record(savefile,(gchar*)&rType,sizeof(rType_t),"relation type");
        }
        memset(databuf,0,databufMaxlen);
        bytesread = load_record(savefile,(gchar*)databuf,databufMaxlen,"allowedFilter");
        g_string_sprintf(temp_node->allowedFilter,"%s",(gchar*)databuf);

        bytesread = load_record(savefile,(gchar*)&temp_node->status,sizeof(statusType_t),"status");
        memset(databuf,0,databufMaxlen);
        bytesread = load_record(savefile,(gchar*)databuf,databufMaxlen,"configRuntime");
        g_string_sprintf(temp_node->configRuntime,"%s",databuf);
    }

    /* and remove plugins that were not used last time */
    /* and remove all referances to non-excistent plugins */
    /* TODO a lot :) */
    close(fd);
    g_string_free(tempstring,TRUE);
    node_delete(buf_node);
    g_free(databuf);
    g_string_free(dir,TRUE);

    return TRUE;
}

gboolean resource::save_state(void) {
    /* Saves the current db situation */
    GIOChannel *savefile;
    gint fd,r,i;
    guint bytessaved;
    GString *dir;
    relation *Relation;
    GList *MOelement;
    node *NODE_LIST;
    #ifdef MO_CACHE
    GSList *element;
    #endif

    dir = g_string_new(getenv("HOME"));
    g_string_append(dir,"/piper_info/save/");
    g_string_append(dir,"BL_status.save");
    fd = open(dir->str,O_WRONLY);
    if (fd < 0 ) {
        close(fd);
        fd = open(dir->str,O_WRONLY|O_CREAT|O_APPEND,S_IRWXU);
    }

    savefile = g_io_channel_unix_new(fd);
    /* save header record */
    r = nodes(); //g_list_length(NODE_LLIST)-1;
    bytessaved = save_record(savefile, (gchar*)&r,sizeof(gint));

    /* Check NODES */
    if (nodes() == 0) {
        /* No nodes allocated */
        g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_DEBUG,"No nodes to save");
        return TRUE;
    } else {
        g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_DEBUG,"Saving %i nodes",nodes());
    }

    /* save NODE_LIST */
    MOelement = NODES;
    i = 0;
    do {
        NODE_LIST = (node*)(g_list_nth_data(NODES,i));
        g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_DEBUG,"Saving node %s",NODE_LIST->dname->str);
        if (NODE_LIST != NULL) {
            bytessaved = save_record(savefile, NODE_LIST->dname->str,NODE_LIST->dname->len);
            /* - */
            bytessaved = save_record(savefile, NODE_LIST->Data->rawdata,DATA_RAWLEN);
            bytessaved = save_record(savefile,&NODE_LIST->Data->rawdatalen,sizeof(gint));
            bytessaved = save_record(savefile, NODE_LIST->Data->string->str,NODE_LIST->Data->string->len);
            bytessaved = save_record(savefile,&NODE_LIST->Data->integer,sizeof(gint));
            bytessaved = save_record(savefile,&NODE_LIST->Data->percentage,sizeof(gint));
            bytessaved = save_record(savefile,&NODE_LIST->Data->boolean,sizeof(gint));
            /* - */
            bytessaved = save_record(savefile,&NODE_LIST->dataSequence,sizeof(gint));
            bytessaved = save_record(savefile, NODE_LIST->dataDescription->str,NODE_LIST->dataDescription->len);
            bytessaved = save_record(savefile,&NODE_LIST->dataPassedOver,sizeof(gint));
            #ifdef MO_CACHE
            if (NODE_LIST->dataBlockedList == NULL) {
                r = 0;
            } else {
                r = g_slist_length(NODE_LIST->dataBlockedList);
            }
            bytessaved = save_record(savefile,&(r),sizeof(gint));
            while (r>0) {
                element = g_slist_nth(NODE_LIST->dataBlockedList,r-1);
                bytessaved = save_record(savefile, ((data*)element->data)->rawdata,DATA_RAWLEN);
                bytessaved = save_record(savefile,&((data*)element->data)->rawdatalen,sizeof(gint));
                bytessaved = save_record(savefile, ((data*)element->data)->string->str,((data*)element->data)->string->len);
                bytessaved = save_record(savefile,&((data*)element->data)->integer,sizeof(gint));
                bytessaved = save_record(savefile,&((data*)element->data)->percentage,sizeof(gint));
                bytessaved = save_record(savefile,&((data*)element->data)->boolean,sizeof(gint));
                r--;
            }
            #endif
            bytessaved = save_record(savefile, NODE_LIST->historyFilename->str,NODE_LIST->historyFilename->len);
            /* historyIO not saved .. wonder why */
            bytessaved = save_record(savefile, NODE_LIST->dDescription->str,NODE_LIST->dDescription->len);
            bytessaved = save_record(savefile,&NODE_LIST->nodeType,sizeof(nodeType_t));
            r = g_list_length(NODE_LIST->relationList);
            bytessaved = save_record(savefile,&r,sizeof(gint));
            for (r=0;r<g_list_length(NODE_LIST->relationList);r++) {
                Relation = (relation*)g_list_nth_data(NODE_LIST->relationList,r);
                if (!Relation) {
                    /* Found NULL relation. Saving dummy record */
                    bytessaved = save_record(savefile,(gpointer)"", 0); /* save a NULL MO */
                    bytessaved = save_record(savefile, NULL, sizeof(rType_t)); /* and a NULL rType */
                } else {
                    bytessaved = save_record(savefile, noderef_to_node(Relation->Node)->dname->str,noderef_to_node(Relation->Node)->dname->len);
                    bytessaved = save_record(savefile,&Relation->rType,sizeof(rType_t));
                }
            }
            bytessaved = save_record(savefile, NODE_LIST->allowedFilter->str,NODE_LIST->allowedFilter->len);
            /* no procedure write.. */
            bytessaved = save_record(savefile,&NODE_LIST->status,sizeof(statusType_t));
            /* hmmss ... bytessaved = messaging_db_save_record(savefile, NODE_LIST[i]->config_init,1024); */
            bytessaved = save_record(savefile, (gpointer)(NODE_LIST->configRuntime->str),NODE_LIST->configRuntime->len);
            /* configPanel, configPanel_buildProcedure, init and destruct not done */
        }
        MOelement = g_list_next(MOelement);
        i++;
    } while (MOelement!=NULL);

    g_io_channel_close(savefile);
    close(fd);
    g_string_free(dir,TRUE);
    return TRUE;
}

gint dirfilter(const struct dirent *entry) {
    /* checks if a file is a plugin candidate */
    gchar *result1,*result2,*result3;
    result1 = strstr(entry->d_name,"plugin_");
    result2 = strstr(entry->d_name,".so");
    result3 = strstr(entry->d_name,".0");
    if ((result1==NULL)||(result2==NULL)||(result3!=NULL))
        return 0;
    else
        return 1;
}

gint resource::load_plugins(gpointer Engine) {
    /* Loads all availeble plugins */
    struct dirent **libsdir;
    plugin_maker* maker;
    node *Node,*Collector;
    gint i;
    GString *temp = g_string_new("");

    i = scandir (PLUGIN_DIR,&libsdir,dirfilter,alphasort);
    if (i >= 0) {
        gint cnt;
        for (cnt = 0; cnt < i; ++cnt) {
            Node = node_new("unfinished node","","",UnknownNode,1,NULL,NULL,NULL,NULL);
            maker = load_plugin(libsdir[cnt]->d_name,node_to_noderef(Node),Engine);
            if (maker != NULL) {
                /* Setup the node and fill in the defaults */
                g_string_sprintf(Node->dname,"%s",maker->name());
                g_string_sprintf(Node->dDescription,"%s",maker->description());
                g_string_sprintf(Node->dataDescription,"%s",maker->description());
                Node->nodeType = maker->nodetype();
                node_initialize(Node,(gpointer)maker->init,(gpointer)maker->procedure,(gpointer)maker->cleanup,(gpointer)maker->activate);
                
                /* Added node to system */
                register_node(1,Node,Engine);

                /* Setip default relation */
                switch (Node->nodeType) {
                    case SensorNode : /* Add pipeline as child */
//g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_DEBUG,"Node->dname = %s,  noderef_to_node(((engine*)Engine)->pipeLine)->dname = %s",Node->dname->str,noderef_to_node(((engine*)Engine)->pipeLine)->dname->str);
                                      relate_node_to_node(Node,noderef_to_node(((engine*)Engine)->pipeLine));
                                      /* Create Collector */
                                      g_string_sprintf(temp,"%s Collector",Node->dname->str);
                                      Collector = node_new(temp->str,Node->dDescription->str,Node->dataDescription->str,(nodeType_t)CollectorNode,Node->uri->subnetID,NULL,NULL,NULL,NULL);
                                      register_node(1,Collector,Engine);
                                      relate_node_to_node(noderef_to_node(((engine*)Engine)->pipeLine),Collector);
                                      break;
                    case FilterNode : /* TODO */
                                      break;
                    case VisualNode : /* Add node to VisualCentral */
                                      relate_node_to_node(noderef_to_node(((engine*)Engine)->visual_central),Node);
                                      break;
                    default         : break;
                }
            }
        }
    } else
        g_error ("Couldn't find the plugins, something is very wrong :(");


    /* Clean & Exit */
    g_string_free(temp,TRUE);
    return TRUE;
}

gboolean resource::activate_nodes() {
    GList *element;
    node *NODE_LIST;

    /* Check NODES */
    if (NODES == NULL) {
        g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_INFO,"Zero nodes to activate");
        return TRUE;
    }

    /* loop MO's and execute their activate code */
    element = NODES;
    while (element->next != NULL) {
        NODE_LIST = (node*)element->data;
        if (NODE_LIST->activate_ref!=NULL) {
            g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_DEBUG,"Activating %s",NODE_LIST->dname->str);
            NODE_LIST->activate_ref();
        } else {
            g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_DEBUG,"Activating node %s aint possible",NODE_LIST->dname->str);
        }
            
        element = g_list_next(element);
    }
    return TRUE;
}

node* resource::node_new(gchar *Dname,gchar *Ddescription,gchar *allowedfilter,nodeType_t type,glong DLid,gpointer init_ref,gpointer procedure_ref,gpointer cleanup_ref, gpointer activate_ref) {

    /* Allocate new node */
    node* Node = new node(Dname,Ddescription,type,DLid);

    /* get unique ID & lock the ID in MO_LIST[] */
    node_id_lock(Node);

    /* Setup function referances */
    node_initialize(Node,init_ref,procedure_ref,cleanup_ref,activate_ref);

    /* Fill in other fields */
    g_string_sprintf(Node->allowedFilter,"%s",allowedfilter);

    /* Allocate & lock Data */
    Node->Data = data_new(Ddescription,DATA_RAWLEN);

    return Node;
}

data* resource::data_new(gchar* DataDescription,guint RawDataLen) {
    data *Data = new data((gchar*)DataDescription,DATA_RAWLEN);
    /* Alloced & lock Data */
    data_id_lock(Data);

    return Data;
}


gboolean resource::node_delete(node *Node) {
    node_id_unlock(Node);
    while (g_slist_length(Node->dataBlockedList)>0) {
        data_delete((data*)g_slist_last(Node->dataBlockedList)->data);
        g_slist_free(g_slist_last(Node->dataBlockedList));
    }

    delete Node;

    return TRUE;
}

gboolean resource::data_delete(data *Data) {
    /* clean LOCKED LIST */
    data_id_unlock(Data);
    delete Data;

    return TRUE;
}

gboolean resource::node_initialize(node* Node,gpointer init_ref,gpointer procedure_ref,gpointer cleanup_ref, gpointer activate_ref) {
    /* Sets function pointers and does checking */
    if (Node == NULL) {
        g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_WARNING,"Can not initialize a NULL node");
        return FALSE;
    }

    if ( (Node->nodeType == SensorNode) ||
         //(Node->nodeType == CollectorNode) || TODO: Collector really no functions?
         (Node->nodeType == FilterNode) ||
         (Node->nodeType == VisualNode) ||
         (Node->nodeType == UnknownNode)
       ) {
        if ((init_ref==NULL)||(procedure_ref==NULL)||(cleanup_ref==NULL)||(activate_ref==NULL)) {
            g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_WARNING,"Can not initialize a node (%s) with NULL function(s)",Node->dname->str);
            return FALSE;
        }
    }

    /* Fill node members */
    Node->init_ref      = (gboolean(*)(glong,gpointer))init_ref;
    Node->procedure_ref = (gboolean(*)())procedure_ref;
    Node->cleanup_ref   = (gboolean(*)())cleanup_ref;
    Node->activate_ref  = (gboolean(*)())activate_ref;
    //g_log(PIPER_BL_LOG_DOMAIN,G_LOG_LEVEL_INFO,"Setup the functions of node %s   (%p, %p, %p, %p)",Node->dname->str,Node->init_ref,Node->procedure_ref,Node->cleanup_ref,Node->activate_ref);
    return TRUE;
}


