"""containerlocus.py

Classes describing the container locus type.

Classes:
    ContainerLocus
"""
# standard modules
import os
import string

# local modules
from linklocus import SimpleLinkLocus
from locus import Locus
from piperdbconnect import PiperDbConnect
from piperpasswords import PiperPass

# Overflow wrapped for python
import vflow

# configuration variables
from piperconfig import *

class ContainerLocus(SimpleLinkLocus):
    
    def __init__(self, parent_locus, base_locus):
        SimpleLinkLocus.__init__(self, parent_locus, base_locus)
        self._children_loci = None
        
        self.base_locus = base_locus
            
    def initialize(self, of_net, of_node, name):
        SimpleLinkLocus.initialize(self, of_node, name)
            
        self.of_net = of_net
        self.all_of_nodes = {}
        
        # initialize the network input and outputs
        self.of_inputs = []
        self.of_outputs = []
        net_terms = vflow.UINetwork_getTerminals(self.of_net)
        for net_term in net_terms:
            term = vflow.UINetTerminal_getTerminal(net_term)
            if vflow.UITerminal_isInputTerminal(term):
                self.of_inputs.append(term)
            else:
                self.of_outputs.append(term)
            
        # add a parameter with info about the type of widget to create
        if self._parameters is None:
            self.get_parameters()
        self._parameters['ws_widget'] = ['widget', 'ANY', 'workspace', '']
        
    def create(self, type, name, do_init = 1):
        valid_container_types = ['composite', 'container', 'iterator']
        if type not in valid_container_types:
            raise KeyError("The passed type %s is not a valid container type."
              % type)
              
        new_node = SimpleLinkLocus.create(self, type, name, 0)
        
        if type == 'composite' or type == 'container':
            new_net = self.base_locus._add_of_network(name, 0)
        elif type == 'iterator':
            new_net = self.base_locus._add_of_network(name, 1)
            
        if do_init:
            self.initialize(self, new_net, new_node, name)
            
        return new_node, new_net
                        
    # --- manipulation functions
    def remove_locus(self, locus_to_remove):        
        
        # first remove the locus from the list of loci in the workspace
        if self._children_loci is None:
            self.get_children_loci()
            
        # remove the locus from the overflow info
        vflow.UINetwork_removeNode(self.of_net, locus_to_remove.of_node)
                
        # if the locus has a network...
        network_types = ['composite', 'iterator', 'container']
        if locus_to_remove.get_type() in network_types:
            # do a removal in all of the children loci
            for inner_locus in locus_to_remove.get_children_loci():
                locus_to_remove.remove_locus(inner_locus)
            
            # remove the network from the overflow document
            vflow.UIDocument_removeNetwork(self.base_locus.of_doc, 
              locus_to_remove.of_net)
                    
        # remove the locus from the list of held loci
        del self._children_loci[locus_to_remove.of_node]
                    
        

        
        
        
        
        
