"""piperpasswords.py

Classes for setting up Piper to store passwords.
Currently this is only used to store database passwords (until we need
it for something else :).

Currently able to store and retrieve:
    - database info
    
"""
# standard modules
import os
import rotor
import shelve


# password storage file
PASS_FILE = os.path.join(os.environ['HOME'], 'piper_info', 'storage', 'private', 
  'locipasswd')

class PiperPass:
    def __init__(self, username = None, password = None):
        """Retrieve the password information for a particular user.
        
        Raises:
            KeyError - If the passed username and password are not valid.
        """ 
        # check to be sure the directory for passwords exists. If not create it.
        if (os.path.exists(os.path.dirname(PASS_FILE)) == 0):
            os.makedirs(os.path.dirname(PASS_FILE))
        if username and password:
            self.pfile = shelve.open(PASS_FILE)

            self.user_info = self.pfile[self.shelve_key]
            self.key = password
        
    def add_new_user(self, username, password):
        """Add a new username and password to the database.
        """
        self.pfile = shelve.open(PASS_FILE)
        self.pfile[self.shelve_key] = []
        self.user_info = self.pfile[self.shelve_key]
        self.key = password
        
    def close(self):
        """Close the password database.
        """
        self.pfile.close()
        
    def add_password(self, type, password, other_keys = {}):
        """Add a password to the stored file.
        
        Arguments:
            * type - The type of password to store. Current options are:
                o 'database'
            * password - The password to store.
            * other_keys - A dictionary containing other key and value
            combinations which should be added to the stored info for the
            password. These can be used to further distinguish passwords.
        """
        store_dict = {}
        store_dict['type'] = type
        rt = rotor.newrotor(self.key, 24)
        encrypt_pass = rt.encrypt(password)
        store_dict['p'] = encrypt_pass
        for n in other_keys.keys():
            store_dict[n] = other_keys[n]
            
        self.user_info.append(store_dict)
        
        self.pfile[self.shelve_key] = self.user_info
        
    def get_db_password(self, db_type, db_user):
        """Retrieve a password for a database.
        
        Arguments:
            * db_type- The database type (ex. mysql, postgresql...) to
            retrieve the password for.
            * db_user - The user name to get the password for.
        """
        password = None
        
        for n in self.user_info:
            if ((n['type'] == 'database') and (n['database'] == db_type) and
              (n['username'] == db_user)):
                rt = rotor.newrotor(self.key, 24)
                password = rt.decrypt(n['p'])
                del rt
            
        return password
        
