/* SWIG interface file for UIDocument.cc */

%module UIDocument
%{
	#include <tree.h>
	#include <parser.h>
	
	#include "UIDocument.h"
	#include "UINetwork.h"
	#include <string>
	#include <vector>

%}

%typemap(python, in) string * {		
	char* temp = PyString_AsString($source);
	*$target = string(temp);
}

%typemap(python, out) string {
	string to_convert = *$source;
	const char *temp_out = to_convert.c_str();
	$target = PyString_FromString(temp_out);
}

%typemap(python, in) UINetwork::Type * {
	UINetwork::Type type_to_pass;
	long passed_type;
	
	passed_type = PyInt_AsLong($source);

	if (passed_type == 0)
		type_to_pass = UINetwork::subnet;
	else if (passed_type == 1)
		type_to_pass = UINetwork::iterator;
	else if (passed_type == 2)
		type_to_pass = UINetwork::threaded;
	else
		type_to_pass = UINetwork::subnet;
	
	$target = type_to_pass;
}

%typemap(python, out) vector<UINetwork *> {
	vector <UINetwork*> networks = *$source;
	PyObject *to_return = PyList_New(networks.size());
	
	for (int i=0; i < networks.size(); i++)
		{
			PyObject *new_item = SWIG_NewPointerObj((void *) networks[i],
										  SWIGTYPE_UINetwork_p);
										  
			PyList_SetItem(to_return, i, new_item);
		}
	$target = to_return;
}

%typemap(python, out) vector<DocParameterDataText> {
	vector <DocParameterDataText*> textParams = *$source;
	PyObject *to_return = PyList_New(textParams.size());
		
	for (int i=0; i < textParams.size(); i++)
		{
			PyObject *new_item = SWIG_NewPointerObj((void *) textParams[i],
										  SWIGTYPE_DocParameterDataText_p);
										  
		PyList_SetItem(to_return, i, new_item);
		}
	$target = to_return;
}

%typemap(python, out) vector<string> {
	vector <string> nodeInfo = *$(source);
	PyObject *to_return = PyList_New(nodeInfo.size());
	PyObject *string_item;
	
	for (unsigned int i = 0; i < nodeInfo.size(); i++) {
		string to_convert = nodeInfo[i];
		const char *temp_out = to_convert.c_str();
		string_item = PyString_FromString(temp_out);
		PyList_SetItem(to_return, i, string_item);
	}
	
	$target = to_return;
}		

class ItemInfo {
  public:
	string name;
	string type;
	string value;
	string description;
	
	ItemInfo();
};
	
class DocParameterDataText {
  public:
	string get_name();
	string get_value();
	string get_type();
};

class UIDocument {
  public:
	UIDocument(string _name);
	~UIDocument();
	
	void save();
	UINetwork *addNetwork(string name, UINetwork::Type type); 
    virtual void setFullPath(const string &fullpath);

	void removeNetwork(UINetwork *toRemove);
	
	vector<UINetwork *> get_networks();
	vector<DocParameterDataText> get_textParams();
	
	virtual void load();
	
	static void loadNodeDefInfo(const string &path, const string &name);
   	static void loadExtDocInfo(const string &path, const string &name);
	
	static void scanDL();
	static void loadAllInfo();

	virtual void run();
	
	vector<string> getAvailableNodes();
		
};
