/* SWIG interface file for UINetwork.cc */

%module UINetwork
%{
	#include "UINetwork.h"
	#include "UIDocument.h"
	#include "UINode.h"
	#include "UITerminal.h"
	#include "UILink.h"
	#include "UINetTerminal.h"	
	#include <tree.h>
%}

%typemap(python, in) UINetTerminal::NetTermType * {
	UINetTerminal::NetTermType type_to_pass;
	long passed_type;
	
	passed_type = PyInt_AsLong($source);

	if (passed_type == 0)
		type_to_pass = UINetTerminal::INPUT;
	else if (passed_type == 1)
		type_to_pass = UINetTerminal::OUTPUT;
	else if (passed_type == 2)
		type_to_pass = UINetTerminal::CONDITION;
	else
		PyErr_SetString(PyExc_TypeError, "Unsupported NetTerminalType");
	
	$target = type_to_pass;
}

%typemap(python, out) vector<UINode *> {
	vector <UINode*> nodes = *$source;
	PyObject *to_return = PyList_New(nodes.size());
	
	for (int i=0; i < nodes.size(); i++)
	{
		PyObject *new_item = SWIG_NewPointerObj((void *) nodes[i],
										  SWIGTYPE_UINode_p);
										  
		PyList_SetItem(to_return, i, new_item);
	}
	$target = to_return;
}

%typemap(python, out) vector<UILink *> {
	vector <UILink*> links = *$source;
	PyObject *to_return = PyList_New(links.size());
	
	for (int i=0; i < links.size(); i++)
	{
		PyObject *new_item = SWIG_NewPointerObj((void *) links[i],
										  SWIGTYPE_UILink_p);
										  
		PyList_SetItem(to_return, i, new_item);
	}
	$target = to_return;
}

%typemap(python, out) vector<UINetTerminal *> {
	vector <UINetTerminal*> terminals = *$source;
	PyObject *to_return = PyList_New(terminals.size());
	
	for (int i=0; i < terminals.size(); i++)
	{
		PyObject *new_item = SWIG_NewPointerObj((void *) terminals[i],
										  SWIGTYPE_UINetTerminal_p);
										  
		PyList_SetItem(to_return, i, new_item);
	}
	$target = to_return;
}


	
class UINetwork {
  public:
	enum Type {subnet, iterator, threaded};
	
  	UINetwork(UIDocument *_doc, string _name, Type _type);
	
	virtual ~UINetwork();
	
	const string &getName();
	Type getType();
	
	void saveXML(xmlNode *root);
	
    void addTerminal(UINetTerminal *term);
    void removeTerminal(UINetTerminal *term);
	
	void addLink(UILink *link);
	void removeLink(UILink *link);
	
    void addNode(UINode *node);
	void removeNode(UINode *node);
	
//   	virtual UITerminal *newTerminal (string _name, UINode *_node, bool _isInput, 
//									double _x, double _y);
	


   	virtual UILink *newLink (UITerminal *_from, UITerminal *_to);

   	virtual UINetTerminal *newNetTerminal (UITerminal *_terminal, 
										  UINetTerminal::NetTermType _type, 
										  string _name);
	
    virtual UINode *newNode(UINetwork* _net, string _name,string _type,
							double _x, double _y, bool doInit);
	
	vector<UINode *> getNodes();
	vector<UILink *> getLinks();
	vector<UINetTerminal *> getTerminals();
	
      
	
};
