# standard modules
import os

# pyGtk
from gtk import *

# local modules
from piperconfig import *
from fileDialog import fileDialog

# --- Constants
DFLT_SAVE_DIR = os.path.join(os.environ['HOME'], 'piper_info', 
 'save')

class PiedDialog(GtkDialog):
	"""
	This class provides dialogs for Pied. Provides dialogs are a generic
	dialog, a startup dialog, a quit dialog and an about dialog.
	"""
	def __init__(self):
		"""
		Inherets from the GtkDialog class.
		"""
		GtkDialog.__init__(self)


	def generic(self, title, message, buttons, pixmap = None):
		"""
		Provides a generic dialog window. When passed a title for the 
		window, a message to display (ie. "Quit: yes or no" or something
		like that), buttons to put in the window (ie. yes or no button)
		and an optional pixmap, a dialog box will be displayed, and
		clicking a button will return the value of the button.
		"""
		self.set_title(title)
		self.set_uposition(dialog_upos_x, dialog_upos_y)
		self.connect("destroy", self.quit)
		self.connect("delete_event", self.quit)
		hbox = GtkHBox(spacing = 5)
		hbox.set_border_width(5)
		self.vbox.pack_start(hbox)
		hbox.show()
		if pixmap:
			self.realize()
		pixmap = GtkPixmap(self, pixmap)
		hbox.pack_start(pixmap, expand=FALSE)
		pixmap.show()
		label = GtkLabel(message)
		label.set_justify(JUSTIFY_LEFT)
		hbox.pack_start(label)
		label.show()

		for text in buttons:
			button = GtkButton(text)
			button.set_border_width(10)
			button.set_flags(CAN_DEFAULT)
			button.set_data("user_data", text)
			button.connect("clicked", self.click)
			self.action_area.pack_start(button, expand = 0, fill = 0)
			button.show()
		self.ret = None

		self.show()

	def quit(self, *args):
		"""
		Generic quit dialog to destroy the window and exit a Gtk loop.
		"""
		self.hide()
		self.destroy()
		mainquit()

	def click(self, button):
		"""
		When a button is clicked, grabs the value of the button
		and returns it. Used by the generic dialog.
		"""
		self.ret = button.get_data("user_data")
		print self.ret
		self.quit()

	def delete_event(self, win, event=None):
		"""
		Generic handler for deletion event--just hides the
		dialog window.
		"""
		# don't destroy window -- just leave it hidden
		win.hide()
		return TRUE
		
	def info_dlg(self, title, infoToDisplay):
		"""
		A simple dialog to display information about a program in a 
		text box. This will need to be better later so that the 
		information can be displayed in a nicer fashion.
		"""
		win = GtkDialog()
		win.set_uposition(dialog_upos_x, dialog_upos_y)
		win.connect("delete_event", self.delete_event)
		win.set_title(title)

		# pack in the info
		from front.gnome.library.modules.widgets.text_box import *
		infoBox = PiedWidget()
		infoBox.setInfo(infoToDisplay)
		infoBox.show()
		
		win.vbox.pack_start(infoBox)
		
		# pack in an ok button when they are sick of looking 
		# at the info
		button = GtkButton("Ok")
		button.connect("clicked", win.hide)
		button.set_flags(CAN_DEFAULT)
		
		win.action_area.pack_start(button)
		button.grab_default()
		button.show()

		win.show()


	def about_dlg(self, *args):
		"""
		A dialog to provide information about Pied when the user
		requests it.
		"""
		win = GtkDialog()
		win.set_uposition(dialog_upos_x, dialog_upos_y)
		win.connect("delete_event", self.delete_event)
		win.set_title("dialog")

		label = GtkLabel(nodes_title + ' ' + nodes_version + '\n' +
                         nodes_copyright + '\n' +
                         '\n' +
                         nodes_authors + '\n' +
                         '\n' +
                         nodes_brief)

		label.set_padding(10, 10)
		win.vbox.pack_start(label)
		label.show()

		button = GtkButton("Ok")
		button.connect("clicked", win.hide)
		button.set_flags(CAN_DEFAULT)
		win.action_area.pack_start(button)
		button.grab_default()
		button.show()

		win.show()
		
	def save_dlg(self, action, network):
		"""A dialog to save information about the current network."""
		win = fileDialog()
		if not(os.path.exists(DFLT_SAVE_DIR)):
			os.mkdir(DFLT_SAVE_DIR)
		win.set_filename(os.path.join(DFLT_SAVE_DIR, os.curdir)[:-1])
		win.show()
		mainloop()
		if win.selectedFile:
			network.document.save(win.selectedFile)
			
	def load_dlg(self, button, network):
		"""A dialog to load information into the network."""
		win = fileDialog()
		if not(os.path.exists(DFLT_SAVE_DIR)):
			os.mkdir(DFLT_SAVE_DIR)
		win.set_filename(os.path.join(DFLT_SAVE_DIR, os.curdir))
		win.show()
		mainloop()
		if win.selectedFile:
			network.document.loadDocument(win.selectedFile)
		


	def quit_dlg(self, action, network):
		"""
		The dialog when a user chooses to quit Pied. Prompts for a
		confirm, and if they exit, calls quitScript to clean 
		stuff up.
		"""

		win = GtkDialog()
		win.set_uposition(dialog_upos_x, dialog_upos_y)
		win.connect("delete_event", self.delete_event)
		win.set_title("dialog")

		label = GtkLabel("Do you really want \nto quit Pied?")
		label.set_padding(10, 10)
		win.vbox.pack_start(label)
		label.show()

		button = GtkButton("Yes")
		######
		# changed this to call quitScript
		button.connect("clicked", self.quitScript, network)
		######
		button.set_flags(CAN_DEFAULT)
		win.action_area.pack_start(button)
		button.grab_default()
		button.show()

		button = GtkButton("No")
		button.connect("clicked", win.hide)
		win.action_area.pack_start(button)
		button.show()

		win.show()
		

		return TRUE
		
	def quitScript(self, button, network):
		"""
		Removes the xml directory storing info about the network
		and quits Pied.
		"""
		# remove the temp/xml directory
		import shutil
		import os
		import os.path
		
		# close the connection to the middle
		network.document.close()

		# quit 
		mainquit()
		

		


