# standard modules
import sys
import os
import string

# Gtk/Gnome stuff
from gtk import *
from GDK import *
from gnome.ui import *
from GdkImlib import *

# import CORBA stubs
import modules.piper.UIL2DL

# popup menu for network
from networkmenu import PiedNetworkMenu
# popup menu for node
from nodemenu import PiedNodeMenu
# popup menu for terminal
from terminalmenu import PiedTerminalMenu
# constants
from piperconfig import *
# the node class
from node import *
# XML comments
from comment import *

class PiedWidget(GnomeCanvas):
	"""
	This class is the main network widget for Pied. It provides functions
	for adding other widgets, connecting exisiting widgets, and generally
	handling all the events that occur within Pied.
	"""

# should be in config file
	# use anti-aliased canvas
	global aa ; aa=FALSE

	#set the type of Widget
	get_type = GnomeCanvas(aa).get_type()
	
	def __init__(self, document, network, parent_node_name = ''):
		"""
		Initialize the network and set up the xml files describing the
		network.
		Arguments:
			* document - The dl implemented document (available through CORBA)
			which represents info about this running instance of the front.
			* network - The dl implemented Network (available through CORBA)
			which represents info about this particular network.
			* parent_node_name: The name of the node which will be "holding"
			the newly created node. If this is blank, then the network
			is assumed to be the main network.
		"""
		# get an instance of the GnomeCanvas
		self._o = GnomeCanvas(aa)._o

		self.parent_node = parent_node_name

		# set the network width and height from the constants file
		self.width = network_width
		self.height = network_height

		self.set_usize(self.width, self.height)
		self.set_scroll_region(0, 0, self.width, self.height)
		
		self.document = document
		self.network = network
		# the network is a node
		self.dl_node = network
		
		self.plugins = self._get_available_plugins()
		
		self.type = 'network'
		self.label = 'Network'
		
		self.pending_xy = None
		
		self.name = ''
	
		# a list with references to all of the nodes held in the network
		self.held_nodes = []

		# connect events with our own event handlers
		self.connect('event', self.network_event)
		
		# Set up an initial nodes to work with--the idea is--node0 is the
		# "network" which is also a node. So, all inputs are connected
		# to node0 unless they are specifically connected to another node

		# Pass self so that node has its network's object
		self.node0 = PiedNode(self)
		self.selected = self.node0

		# get the network colors
		self.color_white = self.get_color('white')
		self.color_black = self.get_color('black')
		self.color_red = self.get_color('red')
		self.color_yellow = self.get_color('yellow')
		self.color_green = self.get_color('green')
		self.color_blue = self.get_color('blue')
		self.wscolor_bg = self.get_style().bg[STATE_NORMAL]
		self.wscolor_fg = self.get_style().fg[STATE_NORMAL]
# should be in config file
		self.color_bg = self.get_color('black')
		self.color_fg = self.get_color('white')

		# allow the network to receive drops from containers
		networkTarget = [('WSTARGET', 0, 0)]
		self.drag_dest_set(DEST_DEFAULT_ALL, networkTarget, ACTION_MOVE)
		self.dropItem = ''
		self.dropXML = ''
		self.dropLevel = ''
		self.connect('drag_data_received', self.add_drop_nodes)
		
			
	def _get_available_plugins(self):
		"""Query the middle to get the available nodes plugins
		
		Returns a dictionary where the keys are the available 
		category names and the values are a list of all available nodes
		within those categories. 
		If the category name has a ".", then the key has a multiple
		element tuple (ie. ('unix', 'utils')) while if it doesn't have
		a dot then it is a single element tuple ('General').
		"""
		organized_plugins = {}
		# query the middle for the available nodes plugins
		available_plugins = self.document.getPlugins()
		
		# get all of the categories and put the plugins in the 
		# right categories
		for plugin in available_plugins:
			category_tuple = tuple(string.split(plugin.category, "."))
			if category_tuple not in organized_plugins.keys():
				organized_plugins[category_tuple] = []
			organized_plugins[category_tuple].append(plugin.name)
			
		return organized_plugins
		
	################################################################
	# Deal with network events: These functions are called to deal
	# with 'event' occurances in the main canvas
	################################################################
		
	def add_drop_nodes(self, w, context, x, y, data, info, time):
		"""Recieve a drop onto the network.
		
		This event handler recieves the data passed from the item 
		being dragged. This data is assumed to be in a huge string with the
		items in the string separated by stars. The string is as follows:
			node_name*item_name*level
			where:
				node_id: The id of the container node the item came from
				node_type: The type of container node the item came from. Valid
				  choices are: 'filesystem' and 'database'
				item_name: The name of the item in the container
				level: The level at which the item is located in the container.
		"""
		pass
		""" Brad -- commented this all out for now. I'll have to come back and
		implement this with the corba interface. I need to save it though so
		I can see what I was doing later and replicate this with corba stuff.
		
		comment('adding nodes')
		if data and data.format == 8:
			# get the data
			node_id, node_type, item_name, level = \
			  string.split(data.data, '*')
			# end the drop
			w.drag_finish(context, TRUE, FALSE, time)
			
			# call the middle to determine the type of item that was dropped
			di = DOMImplementation.DOMImplementation()
			dom_tree = di.createDocument(None, None, None)
			
			front_node = dom_tree.createElement('front')
	
			dom_tree.appendChild(front_node)
			
			info_node = dom_tree.createElement('get_item_info')
			front_node.appendChild(info_node)
			
			ws_node = dom_tree.createElement('network')
			ws_node.setAttribute('id', self.name)
			info_node.appendChild(ws_node)
			
			node_node = dom_tree.createElement('node')
			node_node.setAttribute('id', node_id)
			info_node.appendChild(node_node)
			
			item_node = dom_tree.createElement('item')
			item_node.setAttribute('name', item_name)
			item_node.setAttribute('level', level)
			info_node.appendChild(item_node)
				
			# now send the XML to the middle
			self.middle_connect.send_dom(dom_tree)
			
			ext.ReleaseNode(dom_tree)
			
			reply = self.middle_connect.wait_for_reply()
			reply_handler = GetItemInfoHandler(reply)
			# if the addition was successful, set the name as the unique id
			if (reply_handler.is_success() == 1):
				new_item_name, item_type, extra_info = \
				  reply_handler.get_item_info()
				
				# if the nodes contains filesystem information
				if (node_type == 'filesystem'):
					if (item_type == 'container'):
						new_node = self.add_node(None, 'container', x, y)
						new_node.gui.set_contents(new_item_name, 'filesystem')
					elif (item_type == 'document'):
						new_node = self.add_node(None, 'document', x, y)
						new_node.gui.set_current_item(new_item_name, 
						  'document')
				# if the nodes contains database information
				elif (node_type == 'database'):
					# extract the info for the connection from the extra info attribute
					connection_info = string.split(extra_info, '*')
					if (item_type == 'database'):
						new_node = self.add_node(None, 'container', x, y)
						new_node.gui.set_contents(new_item_name, 'database', 
						  connection_info)
					elif (item_type == 'table'):
						new_node = self.add_node(None, 'document', x, y)
						new_node.gui.set_current_item(new_item_name, 
						  'table', extra_info)
						# Brad-- Bad! This is a viewer--shouldn't be set here!
						# add a new widget to view tabular data
						#from front.gnome.library.modules.widgets.table_view import *
						#new_gui = PiedWidget(new_node, new_item_name, connection_info)
						#new_node.set_gui(new_gui)
		"""
				
				
	def network_event(self, widget, event):
		"""
		Catches all network events, and distrbutes them based on their 
		type to the other network event handlers.
		"""

		# Single press any mouse button
		if (event.type == GDK.BUTTON_PRESS):				
			self.network_button_press(event)
		# Double-click any mouse button
		elif (event.type == GDK._2BUTTON_PRESS):
			self.network_2button_press(event)
		# Drag with mouse
		elif (event.type == GDK.MOTION_NOTIFY):
			self.network_motion_notify(event)
		# Release any mouse button
		elif (event.type == GDK.BUTTON_RELEASE):
			self.network_button_release(event)

	def network_button_press(self, event):
		"""
		Deals with _any_ mouse button press on network.
		"""

		comment('mouse click at', event.x, event.y)

		# Make sure no canvas item was clicked on
		item = self.get_item_at(event.x, event.y)
		if (item == None):
			# Get the background node
			node = self.node0

			# Left mouse button: selecting a node
			if event.button != 2 and event.button !=3:				
				self.eventoccured = 1
				self.toggle_selected(node)

			# Middle mouse button: change the cursor
			if event.button == 2:
				# set cursor
				cursor = cursor_new(GDK.FLEUR)
				self.get_window().set_cursor(cursor)

			# Right mouse button: show the pop-up menu
			if (event.button == 3):	
				self.eventoccured = 1
				self.popup_pointer = (event.x, event.y)
				network_menu = PiedNetworkMenu(self.plugins)
				network_menu.show(self, node)

	def network_2button_press(self, event):
		"""
		Deal with a double click on the network. Currently does nothing.
		"""
		pass
	
	def network_motion_notify(self, event):
		"""
		Deal with mouse motion. Currently does nothing.
		"""
		pass

	def network_button_release(self, event):
		"""
		Deals with a button release. Button 2 clicks change the cursor.
		"""
		if event.button == 2:
			# set cursor
			cursor = cursor_new(GDK.HAND2)
			self.get_window().set_cursor(cursor)
			
	###helper functions

	def toggle_selected(self, node):
		"""
		A helper function for network events. This sets the network
		node as being selected when it is clicked on.
		"""

		comment('clicked on', node.type)

		# if this was not previously selected
		if self.selected != node:

			comment('was', self.selected.type, 'now', node.type)

			# if not the network node
			if self.selected != self.node0:
				# previously selected is now unselected
				self.selected.group.selection_box.hide()

			self.selected = node

			if self.selected != self.node0:
				# previously unselected is now selected
				# select the new node
				self.selected.group.selection_box.show()

				# place node group on top
				self.selected.group.raise_to_top()
				
				# set the node as selected in the dl
				self.document.setSelectedLoci(self.selected.dl_node)
				
			else:
				# if it it the network, clear the selected nodes
				# in the dl
				self.document.clearSelectedLoci()
	#################################################################
	# Submitting processes to the Definition Layer
	#################################################################
	def submit_wfd(self, *args):
		"""Submit the current network for processing."""
		self.document.run()
		
	#################################################################
	# Deal with node events
	#################################################################
	
	def node_event(self, widget, event, node):
		"""
		This event handler catches all events that occur on nodes within
		the network, and distributes them to different internal functions
		based on their type.
		"""
		# Single press any mouse button
		if event.type == GDK.BUTTON_PRESS:
			self.node_button_press(event, node)
		# Double-click any mouse button
		elif event.type == GDK._2BUTTON_PRESS:
			self.node_2button_press(event, node)
		# Drag with mouse
		elif event.type == GDK.MOTION_NOTIFY:
			self.node_motion_notify(event, node)
		# Release any mouse button
		elif event.type == GDK.BUTTON_RELEASE:
			self.node_button_release(event, node)

	def node_button_press(self, event, node):
		"""
		Deal with button presses. A left mouse button click selects a node
		(possibly for moving) and a right button click on a subnet node
		can be used for executing a script (this will change).
		"""
		# Single-click is for node selection and movement
		# Proceed only if mouse button 1 is pressed down
		if event.button == 1:
			pointer = self.get_pointer()
			self.pointer_x = pointer[0]
			self.pointer_y = pointer[1]

			self.node_offset_x = node.world_x - self.pointer_x
			self.node_offset_y = node.world_y - self.pointer_y
		
		# 2/13/00 Brad: For now, a shift+button 1 click on a node
		# will destroy it
		if ((event.button == 1) and ((event.state & SHIFT_MASK) == SHIFT_MASK)):
			self.remove_node(node)
			
		# Toggle that the text_box is selected
		# Proceed only if mouse button 2 is not pressed down
		if event.button != 2:
			self.toggle_selected(node)

		# Brad--connected a button 2 click with dragging and dropping
		if event.button == 2:
			### old code
			# set cursor
			cursor = cursor_new(GDK.FLEUR)
			self.get_window().set_cursor(cursor)
			###
			# begin drags
			self.begin_drag(node)

		# Right mouse button: show popup menu
		if event.button == 3:
			self.eventoccured = 1
			self.popup_pointer = (event.x, event.y)
			nodes_menu = PiedNodeMenu()
			nodes_menu.show(self, node)

	def node_2button_press(self, event, node):
		"""
		Deal with a double click on a node. This toggles the node gui
		window between hidden and shown.
		"""
		# double-click is for showing/hiding node window
		# proceed only if mouse button 1 is pressed down
		if event.button == 1:
			if node.windowlet.shown:
				node.hideGUI()
			else:
				node.showGUI()
		
	def node_motion_notify(self, event, node):
		"""
		Deal with mouse motion. If button 1 is being held then we are dragging
		a node and need to call node_move to move the nodes and its connections.
		"""
		if event.state & GDK.BUTTON1_MASK:

			pointer = self.get_pointer()
			
			pointer_drag_x = pointer[0]
			pointer_drag_y = pointer[1]
		 
			move_x = pointer_drag_x - self.pointer_x
			move_y = pointer_drag_y - self.pointer_y

			self.node_move(node, move_x, move_y)

			self.pointer_x, self.pointer_y = pointer_drag_x, pointer_drag_y

		
	def node_button_release(self, event, node):
		"""
		Deal with button releases. Not implemented since this is just dropping 
		a node.
		"""
		if event.button == 1 and ((event.state & SHIFT_MASK) != SHIFT_MASK):
			comment('node dropped at', node.world_x, node.world_y)
			# set the node position in the definition layer
			node.dl_node.setPosition(float(node.world_x), float(node.world_y))
		
		if event.button == 2:
			# set cursor
			cursor = cursor_new(GDK.HAND2)
			self.get_window().set_cursor(cursor)
		
	#### helper functions
	
	def begin_drag(self, node):
		"""Connect a node to be dragged and dropped.
		"""
		network_node = [('WSLOCUS', 0, 0)]
		self.drag_source_set(BUTTON2_MASK, network_node, ACTION_MOVE)
		self.drag_source_set_icon(self.get_colormap(), node.icon_pix, 
		  node.icon_mask)
		self.connect('drag_data_get', self.pass_dnd_node_info, node)
		
	def pass_dnd_node_info(self, w, context, selection_data, info, time, 
	  node):
		  """Pass node information to the target of the drop.
		  """
		  data_to_pass = node.name + '*' + node.label + '*' + node.type
		  if (info == 0):
			  selection_data.set(selection_data.target, 8, data_to_pass)
	
	def node_move (self, node, move_x, move_y):
		"""
		A helper function to move a node. This moves the node and, if the 
		node is connected to other nodes, moves the connected terminals in a 
		sensible manner.
		"""
		# determine the new position of node's x and y
		node.world_x = node.world_x + move_x
		node.world_y = node.world_y + move_y
		
		# don't let a node move off the canvas
		# to the left
		if (node.world_x < 0):
			node.world_x = 0
			move_x = 0
		# to the bottom
		if (node.world_y < 0):
			node.world_y = 0
			move_y = 0
		# move the node's group - that's the easy part
		node.group.move(move_x, move_y)

		#comment('node world x,y', node.world_x, node.world_y)
		#comment('network size', self.width, self.height)

		# for the following resizing, take into account if the node
		# gui is shown
		if (node.windowlet.shown == 1):
			windowX = node.gui.width / 2
			windowY = node.gui.height
		else:
			windowX = 0
			windowY = 0

		# resize the scroll region if the node is being moved off the
		# scroll region
		# to the right
		if (node.world_x + windowX > self.width):
			#comment('reset width')
			self.width = node.world_x + windowX
			self.set_usize(self.width, self.height)
			self.set_scroll_region(0, 0, self.width, self.height)
			# self.scroll_to(node.world_x, node.world_y)
			#comment('Scroll Offsets ', self.get_scroll_offsets())

		# to the bottom
		if (node.world_y + windowY > self.height):
			#comment('reset height')
			self.height = node.world_y + windowY
			self.set_usize(self.width, self.height)
			self.set_scroll_region(0, 0, self.width, self.height)
			# self.scroll_to(node.world_x, node.world_y)
			#comment('Scroll Offsets ', self.get_scroll_offsets())

		# Moving the node with respect to its mates will cause the connecting
		# terminals to separate.  The following code finds out how much the terminals
		# have separated and moves them back together.  Terminals are moved by
		# the amount of separation and NOT to an world position.

		# we have to consider that the node's terminal is moving with the node

		# move terminals and terminal groups (dots) between node and outputs
		# go through list of node.inputs
		for input_number in range(node.number_inputs_total):
			# if a node is connected to something other then the network
			if node.input[input_number] != self.node0:

				# abbreviate
				# node references self
				inputs_terminal_to_node = node.input[input_number].group.terminal_to_output[node.input[input_number].output.index(node)]

				# remember last position of half_x and half_y
				node.group.terminal_from_input[input_number].group.last_half_world_x = node.group.terminal_from_input[input_number].group.half_world_x
				node.group.terminal_from_input[input_number].group.last_half_world_y = node.group.terminal_from_input[input_number].group.half_world_y
				
				# determine new position of half_x and half_y
				node.group.terminal_from_input[input_number].group.half_world_x = node.input[input_number].world_x + (node.world_x - node.input[input_number].world_x) / 2
				node.group.terminal_from_input[input_number].group.half_world_y = node.input[input_number].world_y + (node.world_y - node.input[input_number].world_y) / 2

				# draw primary terminals
				# primary terminal: node's terminal
				# draw the node's terminal_from_input

				# minus move_x and move_y to cancel out node movement
				# terminal moves by new position minus last position
				# value will be -1, 0, or +1
				terminal_move_x = node.group.terminal_from_input[input_number].group.half_world_x - node.group.terminal_from_input[input_number].group.last_half_world_x - move_x
				terminal_move_y = node.group.terminal_from_input[input_number].group.half_world_y - node.group.terminal_from_input[input_number].group.last_half_world_y - move_y

				self.terminal_move(node.group.terminal_from_input[input_number], terminal_move_x, terminal_move_y)
				
				# draw secondary terminals
				# secondary terminal: input's terminal
				# draw the input's terminal_to_output (drawn to node)

				# remember last position of half_x and half_y
				inputs_terminal_to_node.group.last_half_world_x = inputs_terminal_to_node.group.half_world_x
				inputs_terminal_to_node.group.last_half_world_y = inputs_terminal_to_node.group.half_world_y
				
				# determine new position of half_x and half_y
				inputs_terminal_to_node.group.half_world_x = node.world_x + (node.input[input_number].world_x - node.world_x) / 2
				inputs_terminal_to_node.group.half_world_y = node.world_y + (node.input[input_number].world_y - node.world_y) / 2
				
				# terminal moves by new position minus last position
				# value will be -1, 0, or +1
				terminal_move_x = inputs_terminal_to_node.group.half_world_x - inputs_terminal_to_node.group.last_half_world_x
				terminal_move_y = inputs_terminal_to_node.group.half_world_y - inputs_terminal_to_node.group.last_half_world_y
				
				self.terminal_move(inputs_terminal_to_node, terminal_move_x, terminal_move_y)

		# move terminals and terminal groups (dots) between node and outputs
		# go through list of node.outputs
		for output_number in range(node.number_outputs_total):
			# if a node is connected to another node
			if node.output[output_number] != self.node0:

				# abbreviate
				# node references self
				outputs_terminal_from_node = node.output[output_number].group.terminal_from_input[node.output[output_number].input.index(node)]

				# remember last position of half_x and half_y
				node.group.terminal_to_output[output_number].group.last_half_world_x = node.group.terminal_to_output[output_number].group.half_world_x
				node.group.terminal_to_output[output_number].group.last_half_world_y = node.group.terminal_to_output[output_number].group.half_world_y

				# determine new position of half_x and half_y
				node.group.terminal_to_output[output_number].group.half_world_x = node.output[output_number].world_x + (node.world_x - node.output[output_number].world_x) / 2
				node.group.terminal_to_output[output_number].group.half_world_y = node.output[output_number].world_y + (node.world_y - node.output[output_number].world_y) / 2

				# draw primary terminals
				# primary terminal: node's terminal
				# draw the node's terminal_to_output

				# minus move_x and move_y to cancel out node movement
				# terminal moves by new position minus last position
                # value will be -1, 0, or +1
				terminal_move_x = node.group.terminal_to_output[output_number].group.half_world_x - node.group.terminal_to_output[output_number].group.last_half_world_x - move_x
				terminal_move_y = node.group.terminal_to_output[output_number].group.half_world_y - node.group.terminal_to_output[output_number].group.last_half_world_y - move_y

				self.terminal_move(node.group.terminal_to_output[output_number], terminal_move_x, terminal_move_y)

				# draw secondary terminals
				# secondary terminal: output's terminal
				# draw the output's terminal_from_input (drawn to node)

				# remember last position of half_x and half_y
				outputs_terminal_from_node.group.last_half_world_x = outputs_terminal_from_node.group.half_world_x
				outputs_terminal_from_node.group.last_half_world_y = outputs_terminal_from_node.group.half_world_y

				# determine new position of half_x and half_y
				outputs_terminal_from_node.group.half_world_x = node.world_x + (node.output[output_number].world_x - node.world_x) / 2
				outputs_terminal_from_node.group.half_world_y = node.world_y + (node.output[output_number].world_y - node.world_y) / 2

				# terminal moves by new position minus last position
				# value will be -1, 0, or +1
				terminal_move_x = outputs_terminal_from_node.group.half_world_x - outputs_terminal_from_node.group.last_half_world_x
				terminal_move_y = outputs_terminal_from_node.group.half_world_y - outputs_terminal_from_node.group.last_half_world_y
				
				self.terminal_move(outputs_terminal_from_node, terminal_move_x, terminal_move_y)



		
		
	#################################################################
	# Deal with terminal events
	#################################################################
	
	def terminal_event(self, widget, event, node, terminal):
		"""
		A general event handler to catch events which occur on terminals (ie. moving
		dots). Distributes the events based to the proper handler functions.
		"""
		# Single press any mouse button
		if event.type == GDK.BUTTON_PRESS:
			self.terminal_button_press(event, node, terminal)
		# Double-click any mouse button
		elif event.type == GDK._2BUTTON_PRESS:
			self.terminal_2button_press(event, node, terminal)
		# Drag with mouse
		elif event.type == GDK.MOTION_NOTIFY:
			self.terminal_motion_notify(event, node, terminal)
		# Release any mouse button
		elif event.type == GDK.BUTTON_RELEASE:
			self.terminal_button_release(event, node, terminal)

	def terminal_button_press(self, event, node, terminal):
		"""
		Deal with a single click. Left clicks are for terminal movement, so
		this gets set up to move. Right clicks pop up the nodes menu.
		"""
		# Single-click is for node selection and movement
		# Proceed only if mouse button 1 is pressed down
		if event.button == 1:
			pointer = self.get_pointer()
			self.pointer_x = pointer[0]
			self.pointer_y = pointer[1]

			self.terminal_offset_x = terminal.group.item_x - self.pointer_x
			self.terminal_offset_y = terminal.group.item_y - self.pointer_y

		# Toggle that the text_box is selected
		# Proceed only if mouse button 2 is not pressed down
		if event.button != 2:
			self.toggle_selected(node)

		# Middle mouse button: move terminal
		if event.button == 2:
			# set cursor
			cursor = cursor_new(GDK.FLEUR)
			self.get_window().set_cursor(cursor)

		# Right mouse button: show popup menu
		if event.button == 3:
			self.popup_pointer = (event.x, event.y)
			nodes_menu = PiedTerminalMenu()
			nodes_menu.show(self, node)

	def terminal_2button_press(self, event, node, terminal):
		"""
		Deal with a double click on a terminal. This toggles the node gui
		window between hidden and shown.
		"""
		# double-click is for showing/hiding node window
		# proceed only if mouse button 1 is pressed down
		if event.button == 1:
			if node.windowlet.shown:
				node.hideGUI()
			else:
				node.showGUI()

	def terminal_motion_notify(self, event, node, terminal):
		"""
		Deals with mouse movement. If the left button is held down then we
		are moving a terminal, and need to call terminal_move to move the terminal.
		"""
		# if button 1 is pressed while moving then we are dragging a terminal
		# call terminal move to move the terminal and any connected terminals
		if event.state & GDK.BUTTON1_MASK:

			pointer = self.get_pointer()
			pointer_drag_x = pointer[0]
			pointer_drag_y = pointer[1]

			move_x = pointer_drag_x - self.pointer_x
			move_y = pointer_drag_y - self.pointer_y

			# move the node's terminal
			self.terminal_move(terminal, move_x, move_y)

			# if terminal is connected to mate, then move the mate's terminal too
			if terminal.mate != self.node0:

				self.terminal_move(terminal.mate_terminal, move_x, move_y)

			self.pointer_x, self.pointer_y = pointer_drag_x, pointer_drag_y

	
	def terminal_button_release(self, event, node, terminal):
		"""
		Deal with button releases. We need to see if an unconnected dot was 
		dropped over the top of another dot, in which case we need to connect 
		the dots.
		"""
		if event.button == 1:
			# if terminal is not connected to mate, then...
			if terminal.mate == self.node0:

				# terminal is dropped; check what dropped over

				# is there a collision event I could use???!

				# get the square bounds of the dot (in item-relative coordinates)
				dot_bounds_i = terminal.group.dot.get_bounds()
				# convert item coordinates to world (canvas) coordinates
				dot_bounds_w = terminal.group.i2w(dot_bounds_i[0], dot_bounds_i[1])

				# _x1 & _y1 are upper left of square; _x2 & _y2 are lower right
				dot_bound_w_x1 = dot_bounds_w[0]
				dot_bound_w_y1 = dot_bounds_w[1]

				dot_bounds_w = terminal.group.i2w(dot_bounds_i[2], dot_bounds_i[3])

				dot_bound_w_x2 = dot_bounds_w[0]
				dot_bound_w_y2 = dot_bounds_w[1]

				comment('dot group xy', terminal.group.item_x, terminal.group.item_y)

				# determine (world) center of dot (an ellipse) for probing
				dot_center_x = dot_bound_w_x1 + (dot_bound_w_x2 - dot_bound_w_x1) / 2
				dot_center_y = dot_bound_w_y1 + (dot_bound_w_y2 - dot_bound_w_y1) / 2

				comment('dot center world xy', dot_center_x, dot_center_y)

				# we need to temporarily hide the terminal and terminal group so that
				# the items underneath are shown
				terminal.group.hide()

				# probe the dot's center to determine what item is under the dot
				on_item = self.get_item_at(dot_center_x, dot_center_y)

				# re-show the terminal and terminal group
				terminal.group.show()

				comment('dropped', terminal.type, 'terminal #', terminal.number)

				if on_item != None:
					mate = on_item.get_data('node')
					mate_terminal = on_item.get_data('terminal')
					comment('dropped terminal on mate', mate.label)

					# if item has a terminal associated with it, then...
					if mate_terminal != None:
						comment('dropped terminal on mates', mate_terminal.type, 'terminal #', mate_terminal.number)
						# connect node to mate
						self.connect_nodes(node, mate, terminal, mate_terminal)

					else:
						comment('terminal # is not known')

				else:
					 comment('dropped terminal on network')

		if event.button == 2:
			# set cursor
			cursor = cursor_new(GDK.HAND2)
			self.get_window().set_cursor(cursor)

		
		
	#### helper functions
	
	def terminal_move (self, terminal, move_x, move_y):
		"""
		A helper function to move terminals. This is called for two dots
		simultaneously if they are connected.
		"""

		# move the terminal group
		# includes line, dot, etc.
		terminal.group.move(move_x, move_y)

# should terminal.group.item_ be terminal.item_ ???

		# record new terminal group position
		terminal.group.item_x = terminal.group.item_x + move_x
		terminal.group.item_y = terminal.group.item_y + move_y

		# line origin was moved as terminal group was moved,
		# so we must set it back
		terminal.origin_x = terminal.origin_x - move_x
		terminal.origin_y = terminal.origin_y - move_y

		# redraw terminal line at new coordinates, origin to dot
		terminal.group.line.set(points=(terminal.origin_x,
									terminal.origin_y,
									terminal.dot_x,
									terminal.dot_y))

	#################################################################
	# Other functions for dealing with nodes--add and connect them
	#################################################################	

	def connect_nodes(self, node, mate, node_terminal, mate_terminal):
		"""
		Connects a passed node to its mate.
		This will reposition the node terminals to reflect the new connection
		and change the dot color.
		"""
		# terminals are moved by a group value and NOT to a world position

		# if not trying to connect node to self, and not connecting
		# two terminals of the same I/O type (input to input), then...
		if (mate != node) & (node_terminal.type != mate_terminal.type):

			if (node_terminal.type == 'input'):
				in_terminal = node_terminal
				out_terminal = mate_terminal
				# set the output and input nodes
				outnode = mate
				innode = node
			else:
				out_terminal = node_terminal
				in_terminal = mate_terminal
				# set the output and input nodes 
				outnode = node
				innode = mate		
			try:
				# connect the nodes in the dl
				self.network.connectLoci(innode.dl_node, 
				  in_terminal.group.get_data('dl_terminal'), outnode.dl_node, 
				  out_terminal.group.get_data('dl_terminal'))
			  
				# if everything went okay, finish the connection
				# self._create_connection(innode, outnode, in_terminal, out_terminal)
			
			# deal with connection problems
			except modules.piper.UIL2DL.TypeError, msg:
				comment('The Middle is complaining with a TypeError:' + 
				  msg.message)
				
		# if there is an invalid connection
		else:
			comment('invalid connection')
			
	def show_connection(self, dl_in_node, dl_in_conn, dl_out_node,
	  dl_out_conn):
		"""Used to actually show a connection in the definition layer.
		
		Gets passed the info form the definition layer about the nodes and
		terminals to connect, and then calls _create_connection() with the
		local objects to actually make the connection.
		"""
		# find the input and output node
		for l_node in self.held_nodes:
			#print 'l_node', l_node.dl_node
			#print 'input_node', dl_in_node
			#print 'output_node', dl_out_node
			if l_node.dl_node._is_equivalent(dl_in_node):
				input_node = l_node
			elif l_node.dl_node._is_equivalent(dl_out_node):
				output_node = l_node
				
		# now that we've got the input and output nodes, we need to 
		# get the specific terminals to connect.
		for input in input_node.group.terminal_from_input:
			if input.group.get_data('dl_terminal')._is_equivalent(dl_in_conn):
				in_terminal = input
					
		for output in output_node.group.terminal_to_output:
			if output.group.get_data('dl_terminal')._is_equivalent(dl_out_conn):
				out_terminal = output
				
		# now make the connection
		self._create_connection(input_node, output_node, in_terminal, out_terminal)
			
	def _create_connection(self, node, mate, node_terminal, mate_terminal):
		"""Actually create the gui node connection after it is made in the dl.
		"""
		comment('connecting node', node.label, 'terminal #', node_terminal.number, 'to mate', mate.label, 'terminal #', mate_terminal.number)

		comment('node position', node.world_x, node.world_y)
		comment('mate position', mate.world_x, mate.world_y)
		
		node.input[node_terminal.number] = mate
		mate.output[mate_terminal.number] = node
		node.number_inputs_connected = node.number_inputs_connected + 1
		mate.number_outputs_connected = mate.number_outputs_connected + 1
		
		# store mate and mate_terminal objects
		node_terminal.mate = mate
		node_terminal.mate_terminal = mate_terminal

		mate_terminal.mate = node
		mate_terminal.mate_terminal = node_terminal

# _terminal.group.<position> should be _terminal.<position> ???

		# determine position of node's half_x and half_y
		node_terminal.group.half_world_x = mate.world_x + (node.world_x - mate.world_x) / 2
		node_terminal.group.half_world_y = mate.world_y + (node.world_y - mate.world_y) / 2

		comment('half_pos', node_terminal.group.half_world_x, node_terminal.group.half_world_y)

		# move node's terminals from current (dropped) position to half_x and half_y
		terminal_world = node.group.i2w(node_terminal.group.item_x, node_terminal.group.item_y)
		terminal_move_x = node_terminal.group.half_world_x - terminal_world[0]
		terminal_move_y = node_terminal.group.half_world_y - terminal_world[1]

		# set new text for connected dots
		if mate_terminal.group.dot_text['text'] != '':
			connected_text = mate_terminal.group.dot_text['text'] + '/' + node_terminal.group.dot_text['text']
		else:
			connected_text = ''

		node_terminal.group.dot_text.set(text=connected_text)

		# determine dot size - based on size of text item
		number_half_width = node_terminal.group.dot_text['text_width'] / 2
		number_half_height = node_terminal.group.dot_text['text_height'] / 2
		if number_half_width > number_half_height:
			dot_radius = number_half_width + node.quarter_space
		else:
			dot_radius = number_half_height + node.quarter_space

		print node_terminal.group.dot_text['text'], dot_radius, number_half_width, number_half_height, node.quarter_space

# I should just let the canvas keep track of positions, like in the dot example below!

		dot_x1 = node_terminal.group.dot['x1']
		dot_y1 = node_terminal.group.dot['y1']
		dot_x2 = node_terminal.group.dot['x2']
		dot_y2 = node_terminal.group.dot['y2']

		dot_x = dot_x1 + (dot_x2 - dot_x1) / 2
		dot_y = dot_y1 + (dot_y2 - dot_y1) / 2

		comment('dots xy recorded', dot_x, dot_y)

		# change dot color to green (for connected)
		node_terminal.group.dot.set(x1=dot_x-dot_radius,
								y1=dot_y-dot_radius,
								x2=dot_x+dot_radius,
								y2=dot_y+dot_radius, 
								fill_color_gdk=self.color_green)

		comment('node terminalmovexy', terminal_move_x, terminal_move_y)

		self.terminal_move(node_terminal, terminal_move_x, terminal_move_y)

# temp - need to show/hide via terminal selection
		node_terminal.group.text.hide()
		node_terminal.group.text_box.hide()

# mate_terminal routines same as node_terminal ???  - combine ???

		# determine position of mate's half_x and half_y
		mate_terminal.group.half_world_x = node.world_x + (mate.world_x - node.world_x) / 2
		mate_terminal.group.half_world_y = node.world_y + (mate.world_y - node.world_y) / 2

		# move mate's terminals from current position to half_x and half_y
		terminal_world = mate.group.i2w(mate_terminal.group.item_x, mate_terminal.group.item_y)
		terminal_move_x = mate_terminal.group.half_world_x - terminal_world[0]
		terminal_move_y = mate_terminal.group.half_world_y - terminal_world[1]
				
		mate_terminal.group.dot_text.set(text=connected_text)
		
		# I should just let the canvas keep tract of positions, like in the dot example below!

		dot_x1 = mate_terminal.group.dot['x1']
		dot_y1 = mate_terminal.group.dot['y1']
		dot_x2 = mate_terminal.group.dot['x2']
		dot_y2 = mate_terminal.group.dot['y2']

		dot_x = dot_x1 + (dot_x2 - dot_x1) / 2
		dot_y = dot_y1 + (dot_y2 - dot_y1) / 2

		comment('dots xy recorded', dot_x, dot_y)

		# change dot color to green (for connected)
		mate_terminal.group.dot.set(x1=dot_x-dot_radius,
								y1=dot_y-dot_radius,
								x2=dot_x+dot_radius,
								y2=dot_y+dot_radius,
								fill_color_gdk=self.color_green)

		comment('mate terminalmovexy', terminal_move_x, terminal_move_y)

		self.terminal_move(mate_terminal, terminal_move_x, terminal_move_y)

# temp - need to show/hide via terminal selection
		mate_terminal.group.text.hide()
		mate_terminal.group.text_box.hide()
					
	def disconnect_nodes(self, input_node, input_terminal, output_node, 
	  output_terminal):
		"""Disconnect two nodes in the GUI representation.
		
		This compliments the connect_nodes function to provide the ability
		to disconnect two nodes. For now, the positions of the two dots
		are left the same, but their color is changed and they are no
		longer moved together.
		
		Arguments:
			input_node - the input node of the connection
			input_terminal - the input node terminal to disconnect
			output_node - the output node of the connection
			output_terminal - the output node terminal to disconnect
		"""		
		comment('inputs:', input_node.number_inputs_connected)
		comment('outputs:', output_node.number_outputs_connected)
		
		try:
			self.network.disconnectLoci(input_node.dl_node, 
			  input_terminal.group.get_data('dl_terminal'), output_node.dl_node,
			  output_terminal.group.get_data('dl_terminal'))
		except:
			comment('Unknown error raised in the dl.')
			
	def show_disconnection(self, dl_in_node, dl_in_conn, dl_out_node,
	  dl_out_conn):
		"""Used to actually show a connection in the definition layer.
		
		Gets passed the info form the definition layer about the nodes and
		terminals to connect, and then calls _create_connection() with the
		local objects to actually make the connection.
		"""
		# find the input and output node
		for l_node in self.held_nodes:
			if l_node.dl_node._is_equivalent(dl_in_node):
				input_node = l_node
			elif l_node.dl_node._is_equivalent(dl_out_node):
				output_node = l_node
				
		# now that we've got the input and output nodes, we need to 
		# get the specific terminals to connect.
		for input in input_node.group.terminal_from_input:
			if input.group.get_data('dl_terminal')._is_equivalent(dl_in_conn):
				in_terminal = input
					
		for output in output_node.group.terminal_to_output:
			if output.group.get_data('dl_terminal')._is_equivalent(dl_out_conn):
				out_terminal = output
				
		# now do the disconnection
		self._do_disconnect(input_node, in_terminal, output_node, out_terminal)
			
			
	def _do_disconnect(self, input_node, input_terminal, output_node,
	  output_terminal):
		"""Actually perform a disconnect in the GUI."""
		# disconnect the terminal of the input node
		input_node.input[input_terminal.number] = self.node0
		input_node.group.terminal_from_input[input_terminal.number].mate = self.node0
		input_node.group.terminal_from_input[input_terminal.number].group.dot.set(
		  fill_color_gdk=self.color_red)
		input_node.number_inputs_connected = \
		  input_node.number_inputs_connected - 1
			
		# disconnect the terminal for the output node
		output_node.output[output_terminal.number] = self.node0
		output_node.group.terminal_to_output[output_terminal.number].mate = self.node0
		output_node.group.terminal_to_output[output_terminal.number].group.dot.set(
		  fill_color_gdk=self.color_red)
		output_node.number_outputs_connected = \
		  output_node.number_outputs_connected - 1
		  
		  
	# called from menu--makes a new node in the network
	def add_node(self, button, type, initialX = None, initialY = None):
		"""
		This function adds a new node to the network of the specified type.
		By default, the node GUI will be drawn onto the network at the position
		of the popup_pointer (since nodes are normally added from the popup menu).
		"""
		x_pos = 0
		y_pos = 0
		if initialX is None:
			x_pos = self.popup_pointer[0]
			y_pos = self.popup_pointer[1]
		else:
			x_pos = initialX
			y_pos = initialY
			
		# set the initial X and Y as the next item location to add, so that
		# when show_node() is called, the node can be added in the proper
		# location
		self.pending_xy = [x_pos, y_pos]
			
		
		# call the definition layer to add this node
		try:
			if type == 'subnet' or type == 'iterator':
				self.network.addWorkspace(type)
			else:
				self.network.addLocus(type)
		except modules.piper.UIL2DL.TypeError:
			print "Error: Tried to add an invalid type of node."
			
	def show_node(self, dl_node):
		"""Show a node in the network."""
		x_to_add = 0
		y_to_add = 0
		# if we have an x/y position to add the node add, get that
		if self.pending_xy:
			x_to_add = self.pending_xy[0]
			y_to_add = self.pending_xy[1]
			self.pending_xy = None
		# otherwise get the x/y position from the definition layer
		else:
			x_to_add = dl_node.getXPosition()
			y_to_add = dl_node.getYPosition()
			
		###### initialize a node
		node = PiedNode(self, dl_node._get_type(), x_to_add, y_to_add, dl_node)

		###### set node options
	
		# need to determine network brightness
		network_bright = TRUE
		color_white = self.get_color('white')
		color_black = self.get_color('black')
		
		# set node colors: black & white, depending on network brightness
		if network_bright:
			node.setColors(color_white, color_black)
		else:
			node.setColors(color_black, color_white)
		
		##### create the node GUI
		node.buildNodeGUI()
	
		# add the added nodes to the nodes available in the network
		self.held_nodes.append(node)

		# added so that I could manipulate information about
		# a node from outside programs after adding it
		return node
		
	def remove_node(self, node):
		"""Remove a node from the network.
		
		Allows removal of a node from the network. When passed the node
		to remove, this function will update the dl with the removal of 
		the node, remove the gui representation and unconnect
		any nodes currently connected to the node.
		"""
		# first disconnect any connections with other nodes
		
		for input in node.group.terminal_from_input:
			if input.mate != self.node0:
				out_node = input.mate
				for output in out_node.group.terminal_to_output:
					if output.mate == node:
						out_terminal = output
				self.disconnect_nodes(node, input, out_node, out_terminal)
				
		for output in node.group.terminal_to_output:
			if output.mate != self.node0:
				in_node = output.mate
				for input in in_node.group.terminal_from_input:
					if input.mate == node:
						in_terminal = input
				self.disconnect_nodes(in_node, in_terminal, node, output)

		# now remove the node from the dl
		self.network.removeNode(node.dl_node)

	def show_node_removal(self, dl_node_to_remove):
		"""Show the removal of a node in the GUI."""
		# find the node to remove
		for l_node in self.held_nodes:
			if l_node.dl_node._is_equivalent(dl_node_to_remove):
				node_to_remove = l_node
				
		# remove the node from the list of nodes in the network
		self.held_nodes.remove(node_to_remove)
			
		# remove the GUI representation of the node
		node_to_remove.group.destroy()
				
	def getNodes(self):
		""" 
		Returns the nodes contained within the network.
		This allows you to get a hold of a particular node after finding
		it in the xml files.
		"""
		return self.held_nodes

		
	def _stripNumbers(self, toStrip):
		"""
		This is just a useful internal function to strip the numbers off of a 
		node name and return just the type of the class of nodes.
		"""
		stripName = toStrip
		import string
		# while there is a number at the end keep removing the last digit
		while (string.rfind(string.digits, stripName[len(stripName) - 1]) != -1):
			stripName = stripName[:(len(stripName)- 1)]
		
		return stripName
