import gtk

from dialog import *

# corba stubs
import modules.piper.UIL2DL

class PiedNetworkMenu:
	"""
	Builds the pop-up menu for the network which contains all of the
	options for things to do in Pied.
	"""
	def __init__(self, plugins):
		"""Initialize all of the possible nodes that are supported.
		
		Arguments:
			* plugins - A dictionary with all of the available plugins
			sorted into categories.
		"""
		self.plugins = plugins
		
	def show(self, network, nodes):
		"""
		Create the menu and pop it up.
		"""
		self.network = network
		
		dialog = PiedDialog()

		popup_menu = gtk.GtkMenu()
		popup_menu.set_title('title')
		ag = gtk.GtkAccelGroup()
		popup_menu.set_accel_group(ag)
		popup_menu.show()

		# Make submenu for Popup->Network
		submenu_1 = gtk.GtkMenu()
		submenu_1.show()

		# Make menu item Popup->Network->Details and append
		menu_item = gtk.GtkMenuItem('Details...')
		submenu_1.append(menu_item)
		menu_item.show()
		
		# Add menu item to submit the workflow diagram
		menu_item = gtk.GtkMenuItem("Submit WFD")
		menu_item.connect('activate', network.submit_wfd)
		submenu_1.append(menu_item)
		menu_item.show()

		# Make submenu for Popup->Network->Add
		submenu_1a = gtk.GtkMenu()
		submenu_1a.show()		
		
		# Make menu item Popup->Network->Add and append
		menu_item = gtk.GtkMenuItem('Add')
		menu_item.set_submenu(submenu_1a)
		submenu_1.append(menu_item)
		menu_item.show()

		# Make menu items Popup->Network->Add->* and append
		# Need to create submenus for different categories of plugins.
		
		self._add_plugins(self.plugins.keys(), 0, submenu_1a)

		# Make menu item Popup->Network and append
		menu_item = gtk.GtkMenuItem('Network')
		menu_item.set_submenu(submenu_1)
		popup_menu.append(menu_item)
		menu_item.show()

		# Make submenu for Session
		submenu_2 = gtk.GtkMenu()
		submenu_2.show()

		# Make menu item Popup->Session->blah and append
		menu_item = gtk.GtkMenuItem('Load')
		menu_item.connect('activate', dialog.load_dlg, network)
		submenu_2.append(menu_item)
		menu_item.show()
		
		menu_item = gtk.GtkMenuItem('Save')
		menu_item.connect('activate', dialog.save_dlg, network)
		submenu_2.append(menu_item)
		menu_item.show()
		
		menu_item = gtk.GtkMenuItem('Quit')
		menu_item.connect('activate', dialog.quit_dlg, network)
		submenu_2.append(menu_item)
		menu_item.show()

		# Make menu item Popup->Session and append
		menu_item = gtk.GtkMenuItem('Session')
		menu_item.set_submenu(submenu_2)
		popup_menu.append(menu_item)
		menu_item.show()

		# Make submenu for Help
		submenu_3 = gtk.GtkMenu()
		submenu_3.show()

		# Make menu item Popup->Help->About and append
		menu_item = gtk.GtkMenuItem('About')
		menu_item.connect('activate', dialog.about_dlg)
		submenu_3.append(menu_item)
		menu_item.show()		
		
		# Make menu item Popup->Help and append
		menu_item = gtk.GtkMenuItem('Help')
		menu_item.set_submenu(submenu_3)
		popup_menu.append(menu_item)
		menu_item.show()

		# Show popup menu
		popup_menu.popup(None, None, None, 3, 0)
		
	def _add_plugins(self, cats_to_add, level, parent_menu):
		"""Recursive function to add the plugins to the menu.
			
		WARNING: This is not extremely well tested for crazy cases, so may be
		a little buggy. Sorry bout that.
		"""
		cats_to_add.sort()
		# print 'level', level
		# print 'cats_to_add', cats_to_add
		while len(cats_to_add) != 0:
			current_cat = cats_to_add[0]
			# test if the category is unique
			same_prefix = []
			for test_cat in cats_to_add:
				if current_cat[level] == test_cat[level]:
					same_prefix.append(test_cat)
				
			# if the category name is unique
			if len(same_prefix) == 1:
				# if we are add the end of the category
				# add to tree at the current point
				if len(current_cat) == level + 1:
					# print 'adding', current_cat[level], 'with names'
					category_menu = gtk.GtkMenu()
					category_menu.show()
					for name in self.plugins[current_cat]:
						menu_item = gtk.GtkMenuItem(name)
						menu_item.connect('activate', self.network.add_node, 
						  name)
						category_menu.append(menu_item)
						menu_item.show()
					category_menu_item = gtk.GtkMenuItem(current_cat[level])
					category_menu_item.set_submenu(category_menu)
					parent_menu.append(category_menu_item)
					category_menu_item.show()
				# otherwise add a submenu and call 
				# _add_plugins again down one level
				else:
					# print 'adding', current_cat[level], 'with one submenu'
					# add the prefix to the tree
					# Make the submenu
					submenu = gtk.GtkMenu()
					submenu.show()		
	
					# call _add_plugins again for the same prefix 
					# at the next level
					self._add_plugins([tuple(current_cat)], level + 1, submenu)
						
					# Make menu item and add the submenu
					menu_item = gtk.GtkMenuItem(current_cat[level])
					menu_item.set_submenu(submenu)
					parent_menu.append(menu_item)
					menu_item.show()
					
				# remove it from the categories to process
				cats_to_add.remove(current_cat)
				
			else:
				# print 'adding', current_cat[level], 'with submenu categories'
				# add the prefix to the tree
				# Make the submenu
				submenu = gtk.GtkMenu()
				submenu.show()	
				
				# delete the ones with the same prefix since they are
				# being taken care of at a lower level
				for n in same_prefix:
					cats_to_add.remove(n)
	
				# call _add_plugins again for the same prefix 
				# at the next level
				self._add_plugins(same_prefix, level + 1, submenu)
						
				# Make menu item and add the submenu
				menu_item = gtk.GtkMenuItem(current_cat[level])
				menu_item.set_submenu(submenu)
				parent_menu.append(menu_item)
				menu_item.show()

