# standard modules
import sys
import os

# Gtk/Gnome stuff
from gtk import *
from gnome.ui import *

# import CORBA stubs
import modules.piper.UIL2DL

# windowlet
from windowlet import PiedWindowlet

# configuration variables
from piperconfig import *


class PiedTerminal:
	""" This class holds information about the terminal.  Note that windowlets
		and terminals fall under nodes, and nodes fall under networks.
	"""

	def __init__(self, node):
		""" 
		Initializes a terminal.
		"""

		# add node's object to terminal
		self.node = node

# ??? temporary
		self.message = 'Short message'
#		self.message = 'This is a very long message.  This point is to\n' + \
#						'to see how this fits in the message box and in\n' + \
#						'the tooltips window.'

		self.dot_x = self.node.initial_x
		self.dot_y = self.node.initial_y

		#### set up the colors/fonts etc from the network paraemeters
		self.color_white = self.node.color_white
		self.color_black = self.node.color_black
		self.color_red = self.node.color_red
		self.color_yellow = self.node.color_yellow
		self.color_green = self.node.color_green
		self.color_blue = self.node.color_blue
		self.wscolor_bg = self.node.wscolor_bg
		self.wscolor_fg = self.node.wscolor_bg

		self.space = self.node.space
		self.double_space = self.node.double_space
		self.half_space = self.node.half_space
		self.quarter_space = self.node.quarter_space
		self.line_width = self.node.line_width
		self.text_box_margin = self.node.text_box_margin
		self.text_font = self.node.text_font
		self.number_font = self.node.number_font

		# set default colors for the background and foreground
		self.color_bg = self.color_black
		self.color_fg = self.color_white

		# color of selection box
		self.color_selected = self.color_blue

		# set the position of the node in the dl
		#self.node.dl_node.setPosition(initialx, initialy)

		# GUI name
		self.gui_name = 'terminalinfo'

		#for param in self.node.dl_node._get_parameters():
			#if param._get_name() == 'widget':
				#self.gui_name = param.getInfo().value()
	

	def _add_line(self):
		"""
		"""

		self.group.line = self.group.add('line',
											points=(self.node.initial_x,
													self.node.initial_y,
													self.dot_x,
													self.dot_y),
							  				fill_color_gdk=self.color_fg,
											width_units=self.line_width,
											smooth=TRUE)

	def _add_dot(self):
		"""
		Called by buildNodeGUI, this adds the text description to the
		node. This is initially set as self.label, the type of the
		node.
		"""

		# determine dot size - based on size of text item
		number_half_width = \
		  self.group.dot_text['text_width'] / 2
		number_half_height = \
		  self.group.dot_text['text_height'] / 2
		# which is bigger, width or height?
		if number_half_width > number_half_height:
			self.dot_radius = number_half_width + self.quarter_space
		else:
			self.dot_radius = number_half_height + self.quarter_space

		# put dot at end of line from input
		self.group.dot = self.group.add('ellipse',
									  x1=self.dot_x-self.dot_radius,
									  y1=self.dot_y-self.dot_radius,
									  x2=self.dot_x+self.dot_radius,
									  y2=self.dot_y+self.dot_radius,
									  width_units=self.line_width,
									  fill_color_gdk=self.color_red,
									  outline_color_gdk=self.color_fg)

		# set_data used to attach python object to canvas item (dot)
		self.group.dot.set_data('node', self.node)
		self.group.dot.set_data('terminal', self)

		# can I connect just the group?
		self.group.dot.connect('event', self.node.network.terminal_event, self.node, self)

	def _add_dot_text(self):
		"""
		Called by buildNodeGUI, this adds the text description to the
		node. This is initially set as self.label, the type of the
		node.
		"""

		# put text in line group
		self.group.dot_text = self.group.add('text',  
											text=self.dot_text,
											x=self.dot_x,
											y=self.dot_y,  
											font=self.number_font,
											fill_color_gdk=self.color_fg,  
											anchor=ANCHOR_CENTER)

		# set_data used to attach python object to canvas item (text)
		self.group.dot_text.set_data('node', self.node)
		self.group.dot_text.set_data('terminal', self)

		# set callback
		self.group.dot_text.connect('event', self.node.network.terminal_event, self.node, self)

	def _add_text(self):

		# text position temporarily set to dot's
		self.group.text = self.group.add('text',  
                                           text=self.label,  
                                           x=self.dot_x,  
                                           y=self.dot_y,  
                                           font=self.text_font,  
                                           fill_color_gdk=self.color_fg,  
                                           anchor=ANCHOR_CENTER)

		# get dot text size and reset the position
		self.text_x = self.dot_x
		self.text_y = self.dot_y + self.dot_radius	\
							+ self.double_space	\
							+ self.group.text['text_height'] / 2
		self.group.text.set(y=self.text_y)

		# set_data used to attach python object to canvas item (text)
		self.group.text.set_data('node', self.node)
		self.group.text.set_data('terminal', None)

		# connect event
		self.group.text.connect('event', self.node.network.terminal_event, self.node, self)

	def _add_text_box(self):
		"""
		This function is called by buildNodeGUI, and adds the node 
		text box to the GUI representation of the node.
		"""
		# put node text_box on Network
		self.text_box_half_width = self.group.text['text_width'] / 2 + self.text_box_margin
		self.text_box_half_height = self.group.text['text_height'] / 2 + self.text_box_margin
		text_box_x1 = self.text_x - self.text_box_half_width
		text_box_y1 = self.text_y - self.text_box_half_height
		text_box_x2 = self.text_x + self.text_box_half_width
		text_box_y2 = self.text_y + self.text_box_half_height

		self.group.text_box = self.group.add('rect',  
                                           x1=text_box_x1,  
                                           y1=text_box_y1,  
                                           x2=text_box_x2,  
                                           y2=text_box_y2,  
                                           width_units=self.line_width,  
                                           fill_color_gdk=self.color_bg,  
                                           outline_color_gdk=self.color_fg)


		# set_data used to attach python object to canvas item (text_box)
		self.group.text_box.set_data('node', self.node)
		self.group.text_box.set_data('terminal', None)

		self.group.text_box.connect('event', self.node.network.terminal_event, self.node, self)

	def _add_selection_box(self):
		"""
		This function is called by buildNodeGUI, and adds the node 
		selection box to the GUI representation of the node.
		"""
		# get the square bounds of the node (in item-relative coordinates)
		node_bounds = self.group.get_bounds()

		selection_box_x1 = node_bounds[0] - self.space
		selection_box_y1 = node_bounds[1] - self.space
		selection_box_x2 = node_bounds[2] + self.space
		selection_box_y2 = node_bounds[3] + self.space

		# put node selection_box on Network
		self.group.selection_box = self.group.add('rect',  
                                           x1=selection_box_x1,  
                                           y1=selection_box_y1,  
                                           x2=selection_box_x2,  
                                           y2=selection_box_y2,  
                                           width_units=self.line_width,  
                                           fill_color_gdk=None,  
                                           outline_color_gdk=self.color_selected)

		# set_data used to attach python object to canvas item (selection_box)
		self.group.selection_box.set_data('node', self.node)
		self.group.selection_box.set_data('terminal', None)

		self.group.selection_box.connect('event', self.node.network.terminal_event, self.node, self)

		self.group.selection_box.hide()

	def _shuffle_terminal_group(self):
		"""
		This function is called by buildNodeGUI, and just arranges the added
		GUI components so they look pretty
		"""
		# put the line on the bottom
		self.group.line.lower_to_bottom()

		self.group.selection_box.lower_to_bottom()
		# lines ???
		self.group.text_box.raise_to_top()
		self.group.text.raise_to_top()
		self.group.dot.raise_to_top()
		self.group.dot_text.raise_to_top()

	def _add_initial_widget(self):
		"""
		Called by buildNodeGUI, this adds the initial widget to the 
		node.
		"""

		exec 'from widgets.' + self.gui_name + ' import *'
		self.gui = PiedWidget(self)
	
		windowlet_width = self.gui.width + 30 # margin
		windowlet_height = self.gui.height + 30 # margin
		windowlet_x = self.dot_x
		windowlet_y = self.text_y + self.group.text['text_height'] / 2	\
										+ self.space
		self.windowlet = PiedWindowlet(windowlet_width,
										windowlet_height,
										windowlet_x,
										windowlet_y)
		self.windowlet.set_gui(self.gui)
		self.windowlet.set_message(self.message)
		self.windowlet.add_to_group(self.group)

		self.gui.show(self.node.network)

		# toggle that the terminal's node is selected
		self.node.network.toggle_selected(self.node)

		# connect gui events to an event handler to avoid competition
		# between the gui event handlers and the main event handlers
		# clicks
		self.gui.set_data('node', self.node)
		self.gui.connect('event', self.guiEvent)

	def add_input(self, dl_input_terminal = None):
		"""Add an input to the node.

		Argument:
			dl_input_terminal - The dl terminal object to be assigned to the
			new input. If this value is None, then the input is assumed to be
			being added from scratch, and the dl is queried for the terminal.
		"""

		# The number of the new terminal being added
		# determined by the length of the list containing the lines
		self.number = len(self.node.group.terminal_from_input)

		self.type = 'input'
		self.name = self.type
		self.label = self.name
		self.dot_text = ''

		# determine terminal position
# JWB - need a prettier way to spread terminals
		self.dot_x = self.node.initial_x - 50
		self.dot_y = self.node.initial_y - 50 - 20 * self.number

		# make terminal group (contains all canvas items)
		self.group = self.node.group.add('group')

		# add the line
		self._add_line()

		# append the terminal's line (terminal_from_input) to list
		self.node.group.terminal_from_input.append(self)

		# set node0 as mate
		self.mate = self.node.network.node0

		# append mate (node0) as an input node
		self.node.input.append(self.mate)

		# attach a dl corba terminal object to the output
		if dl_input_terminal is None:
			dl_input_terminal = self.node.dl_node._get_inputs()[self.number]
		# otherwise we are adding a new object and need to up the number
		# of inputs
		else:
			self.node.number_inputs_total = self.node.number_inputs_total + 1

		self.group.set_data('dl_terminal', dl_input_terminal)

		# set_data used to attach python object to canvas item (line)
		self.group.set_data('node', self.node)
		self.group.set_data('terminal', self)

		# add text in dot
		self._add_dot_text()

		# add dot to end of line
		self._add_dot()

		# set arrow head on input lines
		# set arrowhead dimensions according to node dot size
		# arrow_inset starts from the arrow tip
		arrow_inset = self.node.dot_half_width + self.space
		arrow_length = self.node.dot_half_width + self.double_space
		arrow_width = self.space
		self.group.line.set(first_arrowhead=TRUE,
							arrow_shape_a=arrow_inset,
							arrow_shape_b=arrow_length,
							arrow_shape_c=arrow_width)

		self._add_text()
		self._add_text_box()
		self._add_selection_box()
		self._add_initial_widget()
		self._shuffle_terminal_group()

		# record item position
		# item value will change with terminal group position
		# dot will keep a constant value
		self.group.item_x = self.dot_x
		self.group.item_y = self.dot_y

		# record origin position
		# origin value will change with terminal group position
		self.origin_x = self.node.initial_x
		self.origin_y = self.node.initial_y

	def add_output(self, dl_output_terminal = None):
		"""Add an output to the node.

		Argument:
			dl_output_terminal - The dl terminal object to be assigned to the
			new output. If this value is None, then the input is assumed to be
			being added from scratch, and the dl is queried for the terminal.
		"""

		# The number of the new terminal being added
		# determined by the length of the list containing the lines
		self.number = len(self.node.group.terminal_to_output)

		self.type = 'output'
		self.name = self.type
		self.label = self.name
		self.dot_text = ''

		# determine terminal position
# JWB - need a prettier way to spread terminals
		self.dot_x = self.node.initial_x + 50
		self.dot_y = self.node.initial_y + 50 + 20 * self.number

		# make terminal group (contains all canvas items)
		self.group = self.node.group.add('group')

		# add the line
		self._add_line()

		# append the terminal's line (terminal_to_output) to list
		self.node.group.terminal_to_output.append(self)

		# set node0 as mate
		self.mate = self.node.network.node0

		# append mate (node0) as an output node
		self.node.output.append(self.mate)

		# attach a dl corba terminal object to the output
		if dl_output_terminal is None:
			dl_output_terminal = self.node.dl_node._get_outputs()[self.number]
		# otherwise we are adding a new object and need to up the number
		# of outputs
		else:
			self.number_outputs_total = self.number_outputs_total + 1

		self.group.set_data('dl_terminal', dl_output_terminal)

		# set_data used to attach python object to canvas item (line)
		self.group.set_data('node', self.node)
		self.group.set_data('terminal', self)

		# add text in dot
		self._add_dot_text()

		# add dot to end of line
		self._add_dot()

		# set arrow head on output lines
		# set arrowhead dimensions according to dot size
		# arrow_inset starts from the arrow tip
		arrow_inset = self.dot_radius + self.space
		arrow_length = self.dot_radius + self.double_space
		arrow_width = self.space
		self.group.line.set(last_arrowhead=TRUE,
							arrow_shape_a=arrow_inset,
							arrow_shape_b=arrow_length,
							arrow_shape_c=arrow_width)

		self._add_text()
		self._add_text_box()
		self._add_selection_box()
		self._add_initial_widget()
		self._shuffle_terminal_group()

		# record item position
		# item value will change with terminal group position
		# dot will keep a constant value
		self.group.item_x = self.dot_x
		self.group.item_y = self.dot_y

		# record origin position
		# origin value will change with terminal group position
		self.origin_x = self.node.initial_x
		self.origin_y = self.node.initial_y

	def remove_input(self):
		"""Remove an input from the node.

		Arguments:
	
		Raises:
			IndexError - If the found input is still connected.
		"""

		if self.node.input[in_line.number] != self.node.network.node0:
			raise IndexError("Removal can only occur on unconnected inputs.")
	
		self.group.destroy()
		self.node.group.terminal_from_input.remove(self)

		# fix the numbers for the other inputs lines
		for line_num in range(len(self.node.group.terminal_from_input)):
			self.node.group.terminal_from_input[line_num].number = line_num
	
		# remove one from the number of inputs
		self.node.number_inputs_total = self.node.number_inputs_total - 1

		self.destroy()

	def remove_output(self):
		"""Remove an output from the node.

		Raises:
			IndexError - If the found output to delete is connected.
		"""

		if self.node.output[out_line.number] != self.node.network.node0:
			raise IndexError("Removal can only occur on unconnected outputs.")

		self.group.destroy()
		self.node.group.terminal_to_output.remove(self)

		# fix the numbers for the other output lines
		for line_num in range(len(self.node.group.terminal_to_output)):
			self.node.group.terminal_to_output[line_num].number = line_num

		# remove one from the number of outputs
		self.node.number_outputs_total = self.node.number_outputs_total - 1

		self.destroy()

	def guiEvent(self, widget, event):
		"""
		This is just an event handler to avoid competition between
		the gui event handler and the main canvas event handlers. 
		This causes button 3 signals not to be propogated beyond
		the current gui.
		"""
		# if we have a popup menu press, we need to return true to 
		# prevent the signal from being passed on to underlying
		# networks.
		if ((event.type == GDK.BUTTON_PRESS)):
			if (event.button == 3):
				return TRUE
	
	def showGUI(self):
		"""
		This shows the current GUI for the node.
		"""
		self.group.window_widget.grab_focus()
		self.windowlet.ebox.show()
		self.group.window_widget.show()
		self.windowlet.shown = TRUE
		self.node.network.toggle_selected(self.node)

	def hideGUI(self):
		"""
		This hides the current GUI for the node.
		"""
		self.windowlet.ebox.hide()
		self.group.window_widget.hide()
		self.windowlet.shown = FALSE

	def set_gui(self, newGUI):
		"""
		Takes the passed Widget newGUI and sets this as the new node.gui
		element of the widget. The window is hidden by default.
		"""
		self.hideGUI()

		# resize and reset the window for the new widget
		self.group.window_widget.set(width = newGUI.width, 
		height = newGUI.height, 
		y = self.text_y + self.group.text['text_height'] / 2 + newGUI.height / 2 + self.space)

		# get rid of the old gui	
		self.windowlet.pane.remove(self.scrolled_window)

		# add the new gui
		self.gui = newGUI

		self.scrolled_window = GtkScrolledWindow()
		self.scrolled_window.set_border_width(5)
		self.scrolled_window.set_policy(POLICY_AUTOMATIC, POLICY_AUTOMATIC)
		self.scrolled_window.set_usize(self.node.network.width, self.node.network.height)
		self.windowlet.pane.add(self.scrolled_window)

		self.scrolled_window.add_with_viewport(self.gui)

		# attach the new gui to the event handlers which prevent button
		# 3 clicks from being propogated to the main network.
		self.gui.set_data('node', self)
		self.gui.connect('event', self.guiEvent)

		self.scrolled_window.show()
		self.gui.show(self.node.network)

		# toggle that this node is selected
		self.node.network.toggle_selected(self.node)

	def setText(self, textToSet):
		"""
		Sets the passed text as the text label of the terminal.
		"""
		self.group.text.set(text=textToSet)
		self.label = textToSet
		# reset the text box to fit the new label

	def setSelectColor(self, color):
		"""
		This can be used to set a user selected color for some elements (to 
		allow some customization of the GUI). The selected color is set
		as the passed argument color.
		"""
		self.color_selected = color
										   
	def setColors(self, bg, fg):
		""" 
		Set the terminal background and foreground colors.
		"""
		self.color_bg = bg
		self.color_fg = fg

