"""viewcallback.py

CORBA servers to allow the definition layer to make changes in the
user interface. These implement the view portion of the 
Model-View-Controller user interface design, and allow updates of the
user interface independently of actions done by the user.

classes:
	LocusView
	NetworkView
	DocumentView
"""
# pyGtk
import gtk

# omniORB
from omniORB import CORBA

# import the corba stubs and skeletons
import modules.piper.UIL2DL
import modules.piper__POA.UIL2DL

class LocusView(modules.piper__POA.UIL2DL.UilLocusView):

	def __init__(self, node):
		self.node = node

	# --- corba functions
	def showNewInput(self, input_to_show):
		print 'Locus:showNewInput'
		if self.node is None:
			return
			#raise NotImplementedError("No node defined to update.")

		self.node.add_input(input_to_show)

	def showNewOutput(self, output_to_show):
		print 'Locus:showNewOutput'
		if self.node is None:
			return 
			#raise NotImplementedError("No node defined to update.")

		self.node.add_output(output_to_show)

	def showNewParameter(self, param_to_show):
		if self.node is None:
			return
			#raise NotImplementedError("No node defined to update.")

	def hideInput(self, input_to_hide):
		print 'Locus:hideInput'
		if self.node is None:
			return
			#raise NotImplementedError("No node defined to update.")

		self.node.remove_input(input_to_hide)

	def hideOutput(self, output_to_hide):
		print 'Locus:hideOutput'
		if self.node is None:
			return
			#raise NotImplementedError("No locus defined to update.")

		self.node.remove_output(output_to_hide)

	def hideParameter(self, param_to_hide):
		if self.node is None:
			return
			#raise NotImplementedError("No locus defined to update.")

class NetworkView(modules.piper__POA.UIL2DL.UilNetworkView, LocusView):

	def __init__(self, node, workspace):
		LocusView.__init__(self, node)
		self.network = workspace

	# --- corba functions
	def showLocus(self, dl_node):
		print 'Network:showLocus'
		new_node = self.network.show_node(dl_node)
		new_view = LocusView(new_node)		
		return new_view._this()


	def showNetwork(self, dl_network):
		print 'Network:showNetwork'
		new_node = self.network.show_node(dl_network)

		new_view = NetworkView(new_node, new_node.gui)		
		return new_view._this()

	def hideLocus(self, dl_node):
		print 'Network:hideLocus'
		self.network.show_node_removal(dl_node)

	def showConnection(self, in_node, in_connector, out_node, 
	  out_connector):
		print 'Network:showConnection'
		self.network.show_connection(in_node, in_connector, out_node,
		  out_connector)

	def hideConnection(self, in_node, in_connector, out_node, 
	  out_connector):
		print 'Network:hideConnection'
		self.network.show_disconnection(in_node, in_connector, out_node,
			 out_connector)

class DocumentView(modules.piper__POA.UIL2DL.UilDocumentView):
	
	def __init__(self, update_document):
		self.update_document = update_document

	# --- corba functions
	def showNetwork(self, dl_network):
		print 'Document:ShowNetwork'
		dl_network._narrow(modules.piper.UIL2DL.Network)
		new_network = self.update_document.add_network(dl_network)

		new_view = NetworkView(None, new_network)		

		return new_view._this()

