from gtk import *
from gnome.ui import *

class PiedWidget(GtkVBox):
	"""
	This class provides a widget that is meant to model the command line for
	a particular program and make it easier for the user to build the
	command line using Loci.
	"""
	get_type = GtkVBox(spacing=5).get_type()
	def __init__(self):
		"""
		Initialize the widget. It starts off as an empty widget
		and must be set for a particular program by calling 
		setProgram.
		"""
		self._o = GtkVBox(spacing=5)._o
		
		self.width = 100
		self.height = 300
		
		# initialize toolbar and pictures for it
		self.toolbar = GtkToolbar(ORIENTATION_VERTICAL, TOOLBAR_ICONS)
		
		self.pix_win = GtkWindow()
		
		# arrow
		self.arrow_pix, self.arrow_mask = create_pixmap_from_xpm(self.pix_win, None, "front/gnome/library/pixmaps/file/longarrow.xpm")

		# option select
		self.flag_pix, self.flag_mask = create_pixmap_from_xpm(self.pix_win, None, "front/gnome/library/pixmaps/file/chile.xpm")
		
		# file in button
		self.filein_pix, self.filein_mask = create_pixmap_from_xpm(self.pix_win, None, "front/gnome/library/pixmaps/file/filein.xpm")
		
		# file out button
		self.fileout_pix, self.fileout_mask = create_pixmap_from_xpm(self.pix_win, None, "front/gnome/library/pixmaps/file/fileout.xpm")
		
		# program button
		self.prog_pix, self.prog_mask = create_pixmap_from_xpm(self.pix_win, None, "front/gnome/library/pixmaps/file/application.xpm")
		
		self.toolbar.show()
		self.add(self.toolbar)
		self.show()
	
		
	def addProgram(self, text = "", function = None):
		"""
		Adds a button for the program to the command line. This is
		usually (probably always) the first item in the command line.
		Text is some info to display in the tooltip for the button, and
		function is a function to call when the button is selected. 
		"""
		self.toolbar.append_item(text, text, text, GtkPixmap(self.prog_pix, self.prog_mask), function)
		
	def addArrow(self):
		"""
		Appends an arrow to the toolbar--this should be used as the
		"spacer" between different command line options.
		"""
		arrow = GtkPixmap(self.arrow_pix, self.arrow_mask)
		arrow.show()
		self.toolbar.append_widget(arrow, "", "")
	
	def addFlagSelect(self, text = "", function = None):
		"""
		Adds a button for a flag selection item to the command line.
		The argument text is some info to display in the tooltip for the 
		button, and the argument function is a function to call when the
		button is selected. 
		"""
		self.toolbar.append_item(text, text, text, GtkPixmap(self.flag_pix, self.flag_mask), function)
	
	def addInFileSelect(self, text = "", function = None):
		"""
		Adds a button for setting an input file for the command line
		(ie. some kind of infile for a program)
		The argument text is some info to display in the tooltip for the 
		button, and the argument function is a function to call when the
		button is selected. 
		"""
		self.toolbar.append_item(text, text, text, GtkPixmap(self.filein_pix, self.filein_mask), function)
			
	def addOutFileSelect(self, text = "", function = None):
		"""
		Adds a button for setting an output file for the command line
		(ie. some kind of outfile for a program)
		The argument text is some info to display in the tooltip for the 
		button, and the argument function is a function to call when the
		button is selected. 
		"""
		self.toolbar.append_item(text, text, text, GtkPixmap(self.fileout_pix, self.fileout_mask), function)
		
