# standard modules
import os
import os.path

# pyGtk/pyGnome
from gtk import *
from gnome.ui import *
from GDK import *

# converting xml into a gui
from modules.xml2gui import *

# file dialog window
from modules.fileDialog import *

# configuration variables
from piperconfig import *

####constants
FILE_PIXMAP_DIR = os.path.join(piper_pixmap_dir, 'file')

class PiedWidget(GtkVBox):
	"""
	A general container widget class that calls different container subclasses
	(container_list.py, container_tree.py) to show different views of a container.
	This also contains event handling function for events within the container.
	"""
	get_type = GtkVBox(spacing=5).get_type()
	
	def __init__(self, locus):
		""" 
		This function requires initialization with the locus it is located
		in. This allows it to access the loci's xml file and update it
		when information about the locus is changed.
		"""
		
		self._o = GtkVBox(spacing=5)._o
		
		self.width = 260
		self.height = 325
		
		self.locus = locus
	
		# make a window with container info
		self._make_container_win()
		
		# determine what we are holding
		self.info_type = ''
			
	# makes a little window to deal with container stuff
	def _make_container_win(self):
		"""
		This function creates the Gtk Window for a container. It initially
		sets the container as a filesystem container, and sets the initial
		view as a tree.
		"""	
		######## setup the window
		
		self.pix_win = GtkWindow()
		self.set_pix, self.set_mask = create_pixmap_from_xpm(self.pix_win, None, 
		  os.path.join(FILE_PIXMAP_DIR, "open.xpm"))
		self.view_pix, self.view_mask = create_pixmap_from_xpm(self.pix_win, None, 
		  os.path.join(FILE_PIXMAP_DIR, "pixmap.xpm"))
		self.db_pix, self.db_mask = create_pixmap_from_xpm(self.pix_win, None, 
		  os.path.join(FILE_PIXMAP_DIR, "spreadsheet.xpm"))
		self.arrow_pix, self.arrow_mask = create_pixmap_from_xpm(self.pix_win, None, 
		  os.path.join(FILE_PIXMAP_DIR, "arrow.xpm"))
		arrow = GtkPixmap(self.arrow_pix, self.arrow_mask)
		arrow.show()
		
		arrowButtonStyle = GtkButton()
		arrowButtonStyle.connect('clicked', self.show_view)
		arrowButtonStyle.add(arrow)
		arrowButtonStyle.show()
		
		self.toolbar = GtkToolbar(ORIENTATION_HORIZONTAL, TOOLBAR_BOTH)
		self.toolbar.set_space_size(10)
		self.toolbar.set_space_style(TOOLBAR_SPACE_LINE)
		self.toolbar.append_item("File", "Add filesystem information", "Add a filesystem", GtkPixmap(self.set_pix, self.set_mask), self.set_dir)
		self.toolbar.append_item("DB", "Add database information", "Add database information", 
		GtkPixmap(self.db_pix, self.db_mask), self.set_db)
		self.toolbar.append_space()
		self.toolbar.append_item("Style", "Select view style", "Select view style", GtkPixmap(self.view_pix, self.view_mask), self.show_view)
		self.toolbar.append_widget(arrowButtonStyle, "Select view style", "Select view style")

		self.toolbar.show()
		self.add(self.toolbar)
		
		########### set some labels at the bottom of the container to indicate the container type and view
		self.tlabel = GtkLabel('Type: ' + 'Filesystem')
		self.vlabel = GtkLabel('View: ' + 'Tree')
		self.tlabel.show()
		self.vlabel.show()
		
		self.label_box = GtkHBox()
		self.label_box.add(self.tlabel)
		self.label_box.add(self.vlabel)
		self.label_box.show()
		
		######### add an initial representation of the container contents
		
		# this is currently set up to display a tree on startup 
		# maybe this could be a user preference that could
		# be set
		self.tree_viewer = None
		self.list_viewer = None
		
		file_tree = GtkCTree()
		from container_tree import *
		self.tree_viewer = LociWidget(file_tree)
		
		file_tree.connect("select_row", self.drag_item)

		self.add(self.tree_viewer)
		self.add(self.label_box)

			
	def set_db(self, button = None, db_to_set = None, connect_info = None):
		"""Set a database whose information is to be displayed in the container.
		
		Arguments:
			--db_to_set: The database to be set as the contents on the container.
			If this is none, a selection dialog is opened to get the database
			info.
			--connect_info: The connection info to connect to the database.
		"""
		self.set_as_database()
		if (db_to_set == None):
			# open a database selection dialog
			dbinfo = dbSelectDialog()
			mainloop()
			# create a string with all of the database info to send to the 
			# middle for a connection.
			db_string = dbinfo.DBtype + '*' + dbinfo.host + '*' + \
			  dbinfo.user + '*' +  dbinfo.port + '*' + dbinfo.socket
			db_name = dbinfo.DBtype
			db_to_set = ''
		else:
			db_string = ''
			for n in connect_info:
				db_string = db_string + n + '*'
			db_name = connect_info[0]
		""" Brad -- commented this all out for now. I'll have to come back and
		implement this with the corba interface. I need to save it though so
		I can see what I was doing later and replicate this with corba stuff.
		
		# if a database is selected and info is available, connect to 
		# the database
		if (db_name != ''):
			# create XML about the file chosen to send to the middle.
			######################
			di = DOMImplementation.DOMImplementation()
			dom_tree = di.createDocument(None, None, None)
			
			front_node = dom_tree.createElement('front')
	
			dom_tree.appendChild(front_node)
			
			add_node = dom_tree.createElement('add_info')
			front_node.appendChild(add_node)
			
			locus_node = dom_tree.createElement('locus')
			locus_node.setAttribute('id', self.locus.name)
			add_node.appendChild(locus_node)
			
			info_node = dom_tree.createElement('info')
			info_node.setAttribute('type', 'database')
			add_node.appendChild(info_node)
			
			db_node = dom_tree.createElement('database')
			db_node.setAttribute('name', db_to_set)
			db_node.setAttribute('connect_string', db_string)
			info_node.appendChild(db_node)
			
			# now send the XML to the middle
			self.locus.workspace.middle_connect.send_dom(dom_tree)
			
			ext.ReleaseNode(dom_tree)
			
			# check the reply
			reply = self.locus.workspace.middle_connect.wait_for_reply()
			reply_handler = AddInfoHandler(reply)
			
			# if everything is okay, refresh the view with the new info
			if (reply_handler.is_success() == 1):
				returned_info = reply_handler.get_info()
				self.refresh(dom_info=returned_info)
			else:
				print 'Error! Bad!'
		"""

	def set_dir(self, button = None, dir_to_set = None):
		"""Sets the directory or file for the container to hold.
		
		Note: This sets the container as a 'filesystem' container. 
		
		Arguments: 
			--dir_to_set: This is the directory to set the container to hold.
			If no directory is passed, the function opens a Gtk widget file 
			selection window to allow sers to select a directory or file to store in the container.
			If a new file or directory is added, it is added to the xmlfile
			for the locus. This is done by passing the file information to the
			middle.
		"""
		self.set_as_filesystem()
		if (dir_to_set == None):
			# get the file information from a file selection window
			file_to_hold = self._get_file()
		else:
			file_to_hold = dir_to_set
			
		""" Brad -- commented this all out for now. I'll have to come back and
		implement this with the corba interface. I need to save it though so
		I can see what I was doing later and replicate this with corba stuff.
		
		# if a new file or directory has been selected, add the xml to the
		# xmlfile for this locus by contacting the middle.
		if (file_to_hold != None):
			
			# create XML about the file chosen to send to the middle.
			######################
			di = DOMImplementation.DOMImplementation()
			dom_tree = di.createDocument(None, None, None)
			
			front_node = dom_tree.createElement('front')
	
			dom_tree.appendChild(front_node)
			
			add_node = dom_tree.createElement('add_info')
			front_node.appendChild(add_node)
			
			locus_node = dom_tree.createElement('locus')
			locus_node.setAttribute('id', self.locus.name)
			add_node.appendChild(locus_node)
			
			info_node = dom_tree.createElement('info')
			info_node.setAttribute('type', 'filesystem')
			add_node.appendChild(info_node)
			
			file_node = dom_tree.createElement('file')
			filename_node = dom_tree.createTextNode(file_to_hold)
			file_node.appendChild(filename_node)
			info_node.appendChild(file_node)
			
			# now send the XML to the middle
			self.locus.workspace.middle_connect.send_dom(dom_tree)
			
			ext.ReleaseNode(dom_tree)
			
			# check the reply
			reply = self.locus.workspace.middle_connect.wait_for_reply()
			reply_handler = AddInfoHandler(reply)
			
			# if everything is okay, refresh the view with the new info
			if (reply_handler.is_success() == 1):
				returned_info = reply_handler.get_info()
				self.refresh(dom_info=returned_info)
			else:
				print 'Error! Bad!'
		"""
			
	def _get_file(self):
		"""
		Creates a file selection window from the fileDialog class.
		This selection window creates another Gtk mainloop, so the main program 
		will pause while a file is selected.
		"""
		file_win = fileDialog()
		file_win.set_title("Select container contents")
		file_win.show()
		mainloop()
		return file_win.selectedFile
		
	def set_contents(self, item_to_hold, type, connect_info = None):
		"""
		This can be used to set the contents of the container. 
		item_to_hold is the element for the container to represent, and
		type is the type of the information. Possible values for type
		are:
			- 'filesystem': file/directory information
			- 'database': database information
			
		If a database is being created, the connection info for the
		database must also be passed. This argument should be a list
		containing each of the parameters for connecting.
		"""
		if (type == 'filesystem'):
			self.set_as_filesystem()
			self.set_dir(dir_to_set = item_to_hold)
		elif (type == 'database'):
			self.set_as_database()
			self.set_db(db_to_set = item_to_hold, connect_info = connect_info)
				
	def set_as_filesystem(self):
		"""
		This sets the container as a filesystem container. 
		Currently it doesn't do anything more than change the label.
		"""
		self.info_type = 'filesystem'
		self.tlabel.set_text('Type: ' + 'Filesystem')
		
	def set_as_database(self):
		"""
		This sets the container as a database container. 
		Currently it doesn't do anything more than change the label.
		"""
		self.info_type = 'database'
		self.tlabel.set_text('Type: ' + 'Database')
			
	def show_view(self, button):
		"""
		This function is called by the select view toolbar button. It 
		displays a pop up window allowing the user to select different
		possible views of the contents. The two choices are currently
		a tree view and a list view. Based on the users choice, it 
		calls either show_tree_view or show_list_view to change the view.
		"""
		viewMenu = GtkMenu()

		treeItem = GtkMenuItem('View as Tree')
		treeItem.connect('activate', self.show_tree_view)
		listItem = GtkMenuItem('View as List')
		listItem.connect('activate', self.show_list_view)
		# iconItem = GtkMenuItem('View as Icon')
		
		viewMenu.append(treeItem)
		viewMenu.append(listItem)
		# viewMenu.append(iconItem)
		
		treeItem.show()
		listItem.show()
		# iconItem.show()
		
		viewMenu.show()
		
		viewMenu.popup(None, None, None, 3, 0)
		
	def show_tree_view(self, button = None):
		"""
		This sets the users selected view of the contents as a tree
		and updates the view of the contents in a tree format.
		"""
		self.vlabel.set_text('View: ' + 'Tree')
		self.refresh()
	
	def show_list_view(self, button = None):
		"""
		This sets the users selected view of the contents as a list
		and updates the view of the contents in a list format.
		"""
		self.vlabel.set_text('View: ' + 'List')
		self.refresh()
		
	def refresh(self, button = None, dom_info=None):
		"""
		This function is called by the refresh toolbar button or at
		any other time the displayed contents need refreshing (ie. after
		adding new contents). It redraws the view of the container contents 
		as the currently selected view.
		Arguments:
			--button: If refresh is called from a toolbar/button.
			--dom_info: Information to refresh with. If dom_info is none, then
			the information will be obtained from the middle.
		"""
		pass
		
		""" Brad -- commented this all out for now. I'll have to come back and
		implement this with the corba interface. I need to save it though so
		I can see what I was doing later and replicate this with corba stuff.
		
		if (dom_info == None):
			# get the xml information from the middle for the locus
			######################
			di = DOMImplementation.DOMImplementation()
			dom_tree = di.createDocument(None, None, None)
			
			front_node = dom_tree.createElement('front')
	
			dom_tree.appendChild(front_node)
			
			get_node = dom_tree.createElement('get_info')
			front_node.appendChild(get_node)
			
			locus_node = dom_tree.createElement('locus')
			locus_node.setAttribute('id', self.locus.name)
			get_node.appendChild(locus_node)
			
			# now send the XML to the middle
			self.locus.workspace.middle_connect.send_dom(dom_tree)
			
			ext.ReleaseNode(dom_tree)
			
			# check the reply
			reply = self.locus.workspace.middle_connect.wait_for_reply()
			reply_handler = GetInfoHandler(reply)
		
			if (reply_handler.is_success() == 1):
				dom_info = reply_handler.get_info()
			else:
				print 'Error'
				
		if (self.tree_viewer != None):
			self.show_tree(dom_info)
		elif (self.list_viewer != None):
			self.show_list(dom_info)
		"""

	def show_list(self, dom_info):
		"""Create the list view of the locus contents.
		
		It then removes the old view, and inserts this new 
		list view, calling the container_list class to create the widget.
		Arguments:
			--dom_info: The information to insert into the container, in dom
			format.
		"""
		# import XML conversion utilities and make the list	
		gui_builder = Xml2Gui(dom_info)
		# file_list = gui_builder.make_list()
		### Brad: list isn't supported right now so I'm just making this be
		# a tree too :-)
		file_list = gui_builder.make_tree()
		
		# clear out any old info
		if (self.tree_viewer != None):
			self.remove(self.tree_viewer)
			self.tree_viewer = None
		elif (self.list_viewer != None):
			self.remove(self.list_viewer)
		self.remove(self.label_box)
		
		#from front.gnome.library.modules.widgets.container_tree import *
		#self.list_viewer = LociWidget(file_list)
		
		from container_tree import *
		self.tree_viewer = LociWidget(file_tree)
		
		file_list.connect("select_row", self.drag_item)
		
		#self.add(self.list_viewer)
		self.add(self.tree_viewer)
		self.add(self.label_box)
		
	
	def show_tree(self, dom_info):
		"""
		This function actually shows the tree view of the contents. It calls
		XML2gui to get the file listing for the current file information stored
		in the container. It then removes the old view, and inserts this new 
		tree view, calling the container_tree class to create the widget.
		"""
		# import XML conversion utilities	
		gui_builder = Xml2Gui(dom_info)
		file_tree = gui_builder.make_tree()
		
		# clear out any old info
		if (self.tree_viewer != None):
			self.remove(self.tree_viewer)
		elif (self.list_viewer != None):
			self.remove(self.list_viewer)
			self.list_viewer = None
		self.remove(self.label_box)
		
		from container_tree import *
		self.tree_viewer = LociWidget(file_tree)
		
		file_tree.connect("select_row", self.drag_item)

		self.add(self.tree_viewer)
		self.add(self.label_box)
		
	def drag_item(self, widget, row, column, z=None):
		"""Connect row selections with possible dragging and droppping.
		"""
		workspace_target = [('WSTARGET', 0, 0)]
		item, spacing, icon_pix, icon_mask = widget.get_pixtext(row, column)
		
		# the item selected
		self.item_name = item
		
		# need to find a way to determine level if we start using trees
		self.level = 1
		                             
		widget.drag_source_set(BUTTON1_MASK, workspace_target, ACTION_MOVE)
		widget.drag_source_set_icon(self.get_colormap(), icon_pix, icon_mask)
		widget.connect('drag_data_get', self.pass_item_info)
	
	def pass_item_info(self, w, context, selection_data, info, time):
		"""Pass the info of an item selected and dropped to the target.
		
		This event handler is called during drag and drop when an item
		is dropped. It passes a string with information separated by *'s as
		follows:
			locus_name*item_name*level
			where:
				locus_name: The name of the container locus the item came from
				locus_type: The type of container locus the item came from. Valid
				  choices are: 'filesystem' and 'database'
				item_name: The name of the item in the container
				level: The level at which the item is located in the container.
		"""
		
		data_to_pass = self.locus.name + '*' + self.info_type + '*' \
		  + self.item_name + '*' + repr(self.level)
		if (info == 0):
			selection_data.set(selection_data.target, 8, data_to_pass)
		
