# pyGtk/pyGnome
from gtk import *
from gnome.ui import *
from GDK import *

# omniORB generated stubs
import modules.piper.UIL2DL

# omniORB corba implementation
from omniORB import CORBA

# configuration info from Piper
from piperconfig import *

class PiedWidget(GtkVBox):
	"""General widget to allow control over a info in a node.
	
	This widget is just meant to provide a really simple interface for
	setting the parameters of a node and getting information about the 
	node.
	"""
	
	get_type = GtkVBox(spacing=5).get_type()
	
	def __init__(self, node):
		self._o = GtkVBox(spacing=5)._o
		
		self.width = 375
		self.height = 100
		
		self.node = node
		
		# add a notebook to display the information
		main_nb = GtkNotebook()
		
		# the first window is for setting parameter information
		param_scroll = GtkScrolledWindow()
		param_scroll.set_policy(POLICY_AUTOMATIC, POLICY_AUTOMATIC)
		param_win = GtkVBox()
		
		param_label = GtkLabel("Parameters")
		
		# add the column information
		label1 = GtkLabel("Name")
		label2 = GtkLabel("Type")
		label3 = GtkLabel("Value")
		label4 = GtkLabel("            ")
		label1.show()
		label2.show()
		label3.show()
		label4.show()
		
		label_box = GtkHBox()
		label_box.set_homogeneous(FALSE)
		label_box.pack_start(label1)
		label_box.pack_start(label2)
		label_box.pack_start(label3)
		label_box.pack_start(label4)
		label_box.show()
		param_win.pack_start(label_box)
		
		line_separator = GtkHSeparator()
		line_separator.show()
		param_win.pack_start(line_separator)
		
		self._set_parameter_info(param_win)
		
		param_scroll.add_with_viewport(param_win)
		main_nb.append_page(param_scroll, param_label)
		param_win.show()
		param_label.show()
		param_scroll.show()
		
		# the second window is a text box to display info about
		# the node
		comment_scroll = GtkScrolledWindow()
		comment_scroll.set_policy(POLICY_AUTOMATIC, POLICY_AUTOMATIC)
		comment_win = GtkText()
		comment_win.set_word_wrap(TRUE)
		comment_win.set_editable(TRUE)
		
		# set the text window to have a white background and set font stuff
		style = comment_win.get_style()
		fg = style.fg[STATE_NORMAL]
		bg = style.white
		font = load_font('-adobe-helvetica-medium-r-normal-*-*-80-*-*-*-*-*-*')
		comment_win.set_style(style)
		
		# get the text to put into the comment window
		node_description = self.node.dl_node.getDescription()
		comment_win.insert(font, fg, bg, node_description)
		
		comment_label = GtkLabel("Comments")
		
		
		comment_scroll.add(comment_win)
		main_nb.append_page(comment_scroll, comment_label)
		comment_win.show()
		comment_label.show()
		comment_scroll.show()
		
		
		main_nb.show()
		self.add(main_nb)
		self.show()
		
	def _set_parameter_info(self, parameter_window):
		"""Set the parameters of a node from the info in the defintion layer.
		"""
		parameters = self.node.dl_node._get_parameters()
		for param in parameters:
			label = string.replace(param._get_name(), "_", "\n")
			final_label = string.replace(label, " ", "\n")
			item_label = GtkLabel(final_label)
			item_label.set_line_wrap(TRUE)
			item_label.show()
		
			item_type_menu = GtkMenu()
			
			local_types = self._get_available_types(param._get_type())
			for type in local_types:
				menu_item = GtkMenuItem(type)
				menu_item.show()
				menu_item.connect('activate', self.set_type, type, param)
				item_type_menu.append(menu_item)
				
			# set the dl connector to have the same type as the top item
			# displayed in the menu choices, so that things are consistent
			self.set_type(None, type[0], param)
		
			item_type_omenu = GtkOptionMenu()
			item_type_omenu.set_menu(item_type_menu)
			item_type_menu.show()
			item_type_omenu.show()
		
			# try to get the currently set value if it is set
			try:
				current_info = param.getInfo().value()
			except modules.piper.UIL2DL.LookupError:
				current_info = ""
				
			item_value = GnomeEntry()
			item_value.entry.set_text(current_info)
			item_value.entry.connect('changed', self.set_value, param)
			item_value.show()
		
			item_info = GtkButton("More...")
			item_info.connect('clicked', self.display_param_info, param)
			item_info.show()
		
			item_box = GtkHBox()
			item_box.set_usize(self.width - 30, 20)
			item_box.pack_start(item_label)
			item_box.pack_start(item_type_omenu)
			item_box.pack_start(item_value)
			item_box.pack_start(item_info)
			item_box.show()
			parameter_window.pack_start(item_box)
		
	def _get_available_types(self, corba_type):
		"""Determine the available types for a parameter given the corba type."""
		if corba_type == modules.piper.UIL2DL.Parameter.P_VALUE:
			return ["int", "float", "bool"]
		elif corba_type == modules.piper.UIL2DL.Parameter.P_STRING:
			return ["string"]
		elif corba_type == modules.piper.UIL2DL.Parameter.P_INT:
			return ["int"]
		elif corba_type == modules.piper.UIL2DL.Parameter.P_FLOAT:
			return ["float"]
		elif corba_type == modules.piper.UIL2DL.Parameter.P_BOOL:
			return ["bool"]
		elif corba_type == modules.piper.UIL2DL.Parameter.P_SUBNET_PARAM:
			return ["subnet_param"]
		elif corba_type == modules.piper.UIL2DL.Parameter.P_ANY:
			return ["int", "float", "string", "bool", "subnet_param"]
			
	def set_type(self, button, selected_type, dl_parameter):
		"""When a type is selected from the menu, set it in the dl."""
		if selected_type == "int":
			dl_parameter._set_type(modules.piper.UIL2DL.Parameter.P_INT)
		elif selected_type == "string":
			dl_parameter._set_type(modules.piper.UIL2DL.Parameter.P_STRING)
		elif selected_type == "float":
			dl_parameter._set_type(modules.piper.UIL2DL.Parameter.P_FLOAT)
		elif selected_type == "bool":
			dl_parameter._set_type(modules.piper.UIL2DL.Parameter.P_BOOL)
		elif selected_type == "subnet_param":
			dl_parameter._set_type(modules.piper.UIL2DL.Parameter.P_SUBNET_PARAM)
			
	def set_value(self, entry_box, dl_parameter):
		"""Set the value of the parameter when a user changes it."""
		current_value = entry_box.get_text()
		
		# get the type code for a string
		string_type_code = CORBA._tc_string
		
		# pass the info to the dl
		dl_parameter.setInfo(CORBA.Any(string_type_code, current_value))
		
	class ParamInfoDisplay(GtkDialog):
		"""A simple dialog window to display information about a parameter."""
		def __init__(self, parameter_name, info_to_display):
			GtkDialog.__init__(self)
			
			self.set_title(parameter_name + " Info")
			self.set_uposition(dialog_upos_x, dialog_upos_y)
			
			display_info = GtkLabel(info_to_display)
			display_info.show()
			self.vbox.add(display_info)
			
			ok_button = GtkButton("OK")
			ok_button.show()
			
			self.action_area.pack_start(ok_button)
			
			self.connect('destroy', self.quit)
			self.connect('delete_event', self.quit)
			ok_button.connect('clicked', self.quit)
			
			self.show()
			
		def quit(self, *args):
			self.hide()
			self.destroy()
			mainquit()
				
	def display_param_info(self, button, dl_parameter):
		"""Display a window with more information about a parameter."""
		# get the info to display
		param_name = dl_parameter._get_name()
		description = dl_parameter.getDescription()
		
		info_dialog = self.ParamInfoDisplay(param_name, description)
		mainloop()
		
			
		
			
