# standard modules
import os
import string

#pyGtk/Gnome
from gtk import *
from gnome.ui import *

#the 4DOM DOM implementation
from xml.dom import ext
from xml.dom import DOMImplementation

# file dialog window
from modules.fileDialog import *

# configuration variables
from piperconfig import *

FILE_PIXMAP_DIR = os.path.join(piper_pixmap_dir, 'file')

class PiedWidget(GtkVBox):
	"""
	Provides a widget for file selection. The widget contains a button for
	selecting a file, and also has the option of adding a button to select
	a file that will be terminaled in from a connected loci.
	"""
	get_type = GtkVBox(spacing=5).get_type()
	def __init__(self, locus):
		"""
		Initialize the widget. The widget starts with a toolbar
		button to select files using a file selection dialog, and a 
		display of the currently selected file. There is also a label
		that can be filled with information about the file being
		selected. The widget must be initialized with the locus that
		contains it, so it can write to its xml file to update it.
		"""
		self._o = GtkVBox(spacing=5)._o
		
		self.locus = locus
				
		self.width = 300
		self.height = 150
		
		# initialize the toolbar and possible buttons
		self.toolbar = GtkToolbar(ORIENTATION_HORIZONTAL, TOOLBAR_ICONS)
		self.pix_win = GtkWindow()
		
		# file select
		self.open_pix, self.open_mask = create_pixmap_from_xpm(self.pix_win,
		  None, os.path.join(FILE_PIXMAP_DIR, "open.xpm"))
		# locus select
		self.terminal_pix, self.terminal_mask = create_pixmap_from_xpm(self.pix_win, 
		  None, os.path.join(FILE_PIXMAP_DIR, "terminal.xpm"))
		
		# add the file select button
		self.toolbar.append_item("Set", "Select a file", "Select a file", 
		  GtkPixmap(self.open_pix, self.open_mask), self.file_select)
		self.toolbar.show()
		
		# add an entry box to hold the current file
		self.current_item = GtkEntry()
		self.current_item.set_editable(TRUE)
		self.current_item.show()
		
		# add a type element so we know if we have filesystem or database info
		self.current_item_type = 'filesystem'
		
		# add a label to hold info about the file being selected
		self.hlabel = GtkLabel('Select a file for the loci to hold.')
		self.hlabel.set_line_wrap(TRUE)
		self.hlabel.show()
		
		self.add(self.toolbar)
		self.add(self.current_item)
		self.add(self.hlabel)
		self.show()
		
	def _update_xml(self, extra_db_info = ''):
		"""Update the xmlfile for the locus with contents of the file box.
		
		This communicates with the middle to update the xml for the locus.
		Arguments:
			extra_db_info - A connect string if we are setting the contents
			of a database.
		"""	
		pass
		"""
		# create XML about the file chosen to send to the middle.
		######################
		di = DOMImplementation.DOMImplementation()
		dom_tree = di.createDocument(None, None, None)
			
		front_node = dom_tree.createElement('front')
	
		dom_tree.appendChild(front_node)
			
		add_node = dom_tree.createElement('add_info')
		front_node.appendChild(add_node)
			
		locus_node = dom_tree.createElement('locus')
		locus_node.setAttribute('id', self.locus.name)
		add_node.appendChild(locus_node)
			
		info_node = dom_tree.createElement('info')
		info_node.setAttribute('type', self.current_item_type)
		add_node.appendChild(info_node)
		
		if self.current_item_type == 'filesystem':
			file_node = dom_tree.createElement('file')
			filename_node = dom_tree.createTextNode(self.current_item.get_text())
			file_node.appendChild(filename_node)
			info_node.appendChild(file_node)
		elif self.current_item_type == 'table':
			db_node = dom_tree.createElement('database')
			# got to get just the table name if we have a database
			db, table = string.split(dom_tree.createElement('database'), ':')
			db_node.setAttribute('name', table)
			db_node.setAttribute('connect_string', extra_db_info)
			
		# now send the XML to the middle
		self.locus.workspace.middle_connect.send_dom(dom_tree)
			
		ext.ReleaseNode(dom_tree)
			
		# check the reply
		reply = self.locus.workspace.middle_connect.wait_for_reply()
		reply_handler = AddInfoHandler(reply)
			
		if (reply_handler.is_success() == 1):
			pass
		else:
			print 'Error! Bad!'
		"""
		
	def set_label_info(self, info):
		"""
		Sets the information label of the widget with the passed 
		information.
		"""
		self.hlabel.set_text(info)
	
	def get_current_item(self):
		"""
		Returns the current file selected in the widget.
		"""
		return self.current_item.get_text()
		
	def set_current_item(self, item, item_type, connect_string = ''):
		"""
		Sets the current file of the widget. This value is displayed
		in the entry box in the widget.
		
		Arguments:
			item - The item to set in the text box.
			item_type - The type of item the widget is holding. Valid types
			 are 'filesystem' and 'database'
			connect_string - If we have a database item, this is the connection
			string associated with the item.
		"""
		self.current_item.set_text(item)
		self.current_item_type = item_type
		if item_type == 'filesystem':
			self._update_xml()
		elif item_type == 'database':
			self._update_xml(connect_string)
		
	def add_locus_terminal(self, function = None):
		"""
		This adds a terminal button to the toolbar so that the file to be 
		contained by the locus can be obtained through a terminal with
		another locus (ie. you could set the command line input file
		as the output file produced by another locus). When the button
		is clicked it is terminaled to the function passed as a parameter. 
		"""
		self.toolbar.append_item("Terminal", "Set as a terminal to another locus", 
		  "Set as a terminal to another locus", 
		  GtkPixmap(self.terminal_pix, self.terminal_mask), function)
		
	def file_select(self, button):
		"""
		Opens a file selection window to allow the user to select a file
		for the locus to hold. Uses the internal _fileDialog class, which
		allows this loop of the program to pause while the file is being 
		selected.
		"""
		file_win = fileDialog()
		file_win.set_title("Select file")
		file_win.show()
		mainloop()
		# if a file was selected, set it as the current file.
		if os.path.isfile(file_win.selectedFile):
			self.set_current_item(file_win.selectedFile, 'filesystem')

		
