from gtk import *
from gnome.ui import *

class PiedWidget(GtkVBox):
	"""
	A widget inheriting from GtkScrolled Window that displays a text box. 
	This is meant as a general viewer for output from programs that have 
	textual output. It can also be used as a text box for getting general
	information from a user that must be typed (ie. a phrase to grep for).
	"""
	# get widget characteristics
	get_type = GtkVBox(spacing=5).get_type()
	
	def __init__(self):
		"""
		Creates a text box with white background and black writing that
		is non-editable to start with.
		"""
		self._o = GtkVBox(spacing=5)._o

		# set a size
		self.width = 300
		self.height = 200
		
		scrollWin = GtkScrolledWindow()
		scrollWin.set_policy(POLICY_AUTOMATIC, POLICY_AUTOMATIC)
		scrollWin.show()
		self.add(scrollWin)
		
		# get Text Box and set properties
		self.textBox = GtkText()
		self.textBox.set_word_wrap(TRUE)
		self.textBox.set_usize(self.width - 50, self.height - 50)
		
		# set a white background
		style = self.textBox.get_style()
		self.font = load_font('-adobe-helvetica-medium-r-normal-*-*-80-*-*-*-*-*-*')
		self.fg = style.fg[STATE_NORMAL]
		self.bg = style.white
		self.textBox.set_style(style)
		
		# fonts to use in writing to the text box
		self.small_font = load_font('-adobe-helvetica-medium-r-normal-*-*-80-*-*-*-*-*-*')
		
		self.textBox.show()
		
		scrollWin.add(self.textBox)
		
		self.show()

	def setInfo(self, input):
		"""
		Used to set the contents of the text box. Note that currently
		the information is displayed in a small style font (adobe 80 pt.).
		"""
		self.textBox.insert(self.small_font, self.fg, self.bg, input)
			
	def getInfo(self):
		"""
		Returns the current contents of the text box.
		"""
		return self.textBox.get_chars(0, -1)

	def setEdit(self, edit = 1):
		"""
		Sets the editability of the text box. If passed 1, then the text
		box will be set as editable. If passed 0, it will be set as 
		non-editable.
		"""
		if (edit == 1):
			self.textBox.set_editable(TRUE)
		else:
			self.textBox.set_editable(FALSE)

		
