import gtk


class PiedWindowlet:
	""" This class holds information about the windowlet.  Note that windowlets
		and terminals fall under nodes, and nodes fall under networks.
	"""

	def __init__(self, width=100, height=100, x=0, y=0):

		self.sill_height = 20
		self.width = width
		self.height = height
		self.x = x
		self.y = y
		self.shaded = 0


	def shade(self, widget, event, canvas_group):

		if self.shaded:
			self.frame.show()
			canvas_group.window_widget.set(height=self.height)
			self.shaded = 0
		else:
			self.frame.hide()
			canvas_group.window_widget.set(height=self.sill_height)
			self.shaded = 1
			

	def set_gui(self, gui):

		self.gui = gui


	def set_message(self, message):

		self.message = message


	def add_to_group(self, canvas_group):

		window_sill_button_height = self.sill_height
		window_sill_button_width = window_sill_button_height

		# put windowlet on Network
		self.ebox = gtk.GtkEventBox()

		canvas_group.window_widget = canvas_group.add('widget',
													widget=self.ebox,
													x=self.x,
													y=self.y,
													width=self.width,
													height=self.height,
													anchor=gtk.ANCHOR_NORTH)
		self.ebox.show()

		vbox = gtk.GtkVBox()
		self.ebox.add(vbox)
		vbox.show()

		# note that both the ebox and the canvas widget can catch events
		self.frame = gtk.GtkFrame()
		self.frame.set_shadow_type(gtk.SHADOW_OUT)
		vbox.pack_start(self.frame)
		self.frame.show()

		self.scrolled_window = gtk.GtkScrolledWindow()
		self.scrolled_window.set_border_width(0)
		self.scrolled_window.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
		self.frame.add(self.scrolled_window)
		self.scrolled_window.show()

		self.scrolled_window.add_with_viewport(self.gui)

		sill_hbox = gtk.GtkHBox()
		sill_hbox.set_usize(self.width, self.sill_height)
		vbox.pack_start(sill_hbox, expand=0)
		sill_hbox.show()

		self.sill_status = gtk.GtkButton('s')
		self.sill_status.set_usize(window_sill_button_width, window_sill_button_height)
		self.sill_status.unset_flags(gtk.CAN_FOCUS)
		sill_hbox.pack_start(self.sill_status, expand=0)
		self.sill_status.show()

		self.sill_message = gtk.GtkButton()
		self.sill_message.unset_flags(gtk.CAN_FOCUS)
		sill_hbox.pack_start(self.sill_message)
		self.sill_message.show()

		self.sill_message_label = gtk.GtkLabel()
		self.sill_message_label.set_justify(gtk.JUSTIFY_LEFT)
		self.sill_message_label.set_text(self.message)
		self.sill_message.add(self.sill_message_label)
		self.sill_message_label.show()

		self.sill_view = gtk.GtkButton('v')
		self.sill_view.set_usize(window_sill_button_width, window_sill_button_height)
		self.sill_view.unset_flags(gtk.CAN_FOCUS)
		sill_hbox.pack_start(self.sill_view, expand=0)
		self.sill_view.show()

		self.sill_parent = gtk.GtkButton('p')
		self.sill_parent.set_usize(window_sill_button_width, window_sill_button_height)
		self.sill_parent.unset_flags(gtk.CAN_FOCUS)
		sill_hbox.pack_start(self.sill_parent, expand=0)
		self.sill_parent.show()

		self.sill_shade = gtk.GtkButton('h')
		self.sill_shade.set_usize(window_sill_button_width, window_sill_button_height)
		self.sill_shade.unset_flags(gtk.CAN_FOCUS)
		self.sill_shade.connect('button_press_event', self.shade, canvas_group)
		sill_hbox.pack_start(self.sill_shade, expand=0)
		self.sill_shade.show()

		self.sill_resize = gtk.GtkButton('r')
		self.sill_resize.set_usize(window_sill_button_width, window_sill_button_height)
		self.sill_resize.unset_flags(gtk.CAN_FOCUS)
		sill_hbox.pack_start(self.sill_resize, expand=0)
		self.sill_resize.show()

		# node window starts off hidden
		self.ebox.hide()
		canvas_group.window_widget.hide()
		self.shown = 0


#        node.titlebar_label.connect('button_press_event', self.titlebar_label_press,
#                           canvas,
#                           node)
#        node.titlebar_label.connect('motion_notify_event', self.titlebar_label_motion_notify,
#                           canvas,
#                           node)
#        node.titlebar_label.connect('button_release_event', self.titlebar_label_release,
#                           node)


