#!/usr/bin/env python

# standard modules
import os
import sys

# Gtk/gnome stuff
import GtkExtra
from gtk import *
from gnome.ui import *

# omniORB
from omniORB import CORBA

# import stubs and skeletons generated by omniidl
import modules.piper.UIL2DL
import modules.piper__POA.UIL2DL

# hack to include the configuration file in the PYTHONPATH
sys.path.insert(0, os.path.join(os.pardir, os.pardir, 'config'))

# get useful constants
from piperconfig import *

# local modules
from modules.network import PiedWidget
from modules.viewcallback import DocumentView

# --- constants
STYLE_DIR = os.path.join(piper_dir, 'uil', 'pied', 'config', 'gtkstyles')

UIL2DL_IOR = os.path.join(os.environ['HOME'], 'piper_info', 'ior', 
 'uil2dl.ior')
 
class GnomeClient(modules.piper__POA.UIL2DL.UilClient):
	"""Client implementation for connecting with the dl.
	
	This class must define the functions defined for UilClient in UIL2DL.idl.
	"""
	def __init__(self, username, password):
		self.username = username
		self.password = password
		
	def _get_username(self):
		return self.username
		
	def _get_password(self):
		return self.password
		
	def ping(self):
		return 
		
class DocumentUpdate:
	"""A class for updating a document upon recieving callbacks from the dl.
	"""
	def __init__(self, scroll_window, main_window):
		"""Initialize with the window to add a network to."""
		self.scroll_window = scroll_window
		self.main_window = main_window
		self.dl_document = None
		
	def set_document(self, document):
		"""Set the dl document object associated with this document."""
		self.dl_document = document
		
	def add_network(self, dl_network):
		"""Add a network to the document."""
		new_network = PiedWidget(self.dl_document, dl_network)
		new_network.show()
		
		# see if we already have a main window. If so we need to trash it.
		child_widgets = self.scroll_window.children()
		for widget in child_widgets:
			self.scroll_window.remove(widget)
		
		self.scroll_window.add_with_viewport(new_network)
		self.main_window.show()
		
		# set cursor
		cursor = cursor_new(GDK.HAND2)
		self.main_window.get_window().set_cursor(cursor)
		
		return new_network
		
class PiedGnome:
	"""
	This is the class that is loaded to begin pied. It creates a main
	window containing an initial locus.
	"""
	def __init__(self):
		"""
		Creates the main window and adds the initial network.
		"""

        # get gtk sytles
		rc_parse(os.path.join(STYLE_DIR, 'simple.gtkrc'))
		# set up the main window as a scrolled window
		self.win = GtkWindow()		
		self.win.set_name('main window')
		self.win.set_title(piper_title + ' ' + piper_version)
		self.win.set_uposition(network_upos_x, network_upos_y)
		self.win.connect('delete_event', mainquit)
		self.win.connect('destroy', mainquit)

		# set up vbox in window
		self.vbox = GtkVBox(spacing=0)
#		self.vbox.set_border_width(0)
		self.win.add(self.vbox)
		self.vbox.show()

		self.scroll_win = GtkScrolledWindow()
		self.scroll_win.set_border_width(0)
		self.scroll_win.set_policy(POLICY_AUTOMATIC, POLICY_AUTOMATIC)
		self.scroll_win.set_usize(network_width, network_height)
		self.vbox.pack_start(self.scroll_win)
		self.scroll_win.show()
		
		# connect with the middle
		# get the orb started
		orb = CORBA.ORB_init([], CORBA.ORB_ID)
		
		# get the POA and activate the POA Manager
		poa = orb.resolve_initial_references("RootPOA")
		poa_manager = poa._get_the_POAManager()
		poa_manager.activate()
		
		try:
			# grab our string IOR from the file
			ior_file = open(UIL2DL_IOR, 'r')
			string_ior = ior_file.read()
			ior_file.close()
		except IOError:
			print "Could not find IOR file for the dl at %s" % UIL2DL_IOR
			print "The dl must be running before the Gnome user interface."
			sys.exit(-1)
			
		# now convert the stringified ior into an object reference
		dl_connection = orb.string_to_object(string_ior)
		
		# get a client to send to the middle
		# Note, for now I'm just having the username and password be set
		# at "default" until we get some kind of authentication system.	
		new_client = GnomeClient("default", "default")
		
		# get an object reference for the client
		client_obj = new_client._this()
		
		# get a document callback object
		doc_callback = DocumentUpdate(self.scroll_win, self.win)
		
		# get a document view object to send to the middle
		doc_view = DocumentView(doc_callback)
		view_obj = doc_view._this()
		
		# now connect with the dl server and get the "document" for our
		# program.
		print "Getting document..."
		try:
			# first determine if there is any crash info
			try:
				new_document = dl_connection.addDocument(client_obj, view_obj)
				new_document._narrow(modules.piper.UIL2DL.Document)
				doc_callback.set_document(new_document)
				print "Loading crash info..."
				new_document.loadCrashInfo()
				
			# there isn't any crash info
			except modules.piper.UIL2DL.LookupError:
				print "no crash info"
				print "Adding a Network..."
				new_document.setNetwork('composite')
				
		# if we get an error then the dl is not reachable
		except CORBA.COMM_FAILURE:
			print "Could not contact the dl, it appears not to be started."
			print "The dl must be running before the Gnome user interface."
			sys.exit(-1)

if __name__ == '__main__':		
	pied_gnome = PiedGnome()
	mainloop()
